@@ -5,15 +5,19 @@
 /*                      Copyright (c) 1995,1996                          */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -33,10 +37,10 @@
 /*                                                                       */
 /*=======================================================================*/
 
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
-#include <ctype.h>
+#include <cstdio>
+#include <cstring>
+#include <cstdlib>
+#include <cctype>
 #include <sys/stat.h>
 #include "EST_Wave.h"
 #include "EST_Option.h"
@@ -49,9 +53,9 @@
 
 #define	VOL(volume)		((1 << 16) * (volume) / 100)
 #define au_serverrate 16000
-static nas_playing = 0;
+static int nas_playing = 0;
 
-bool nas_supported = TRUE;
+int nas_supported = TRUE;
 
 int endian_int = 1;
 #define NAS_BIG_ENDIAN (((char *)&endian_int)[0] == 0)
@@ -69,7 +73,7 @@ static void na_sync_play_cb(AuServer *aud, AuEventHandlerRec *handler,
     return;
 }
 
-int play_na_wave(EST_Wave &inwave, EST_Option &al)
+int play_nas_wave(EST_Wave &inwave, EST_Option &al)
 {
     AuServer *aud = NULL;
     /* legal sampling frequencies for Sun dbri device */
@@ -87,7 +91,7 @@ int play_na_wave(EST_Wave &inwave, EST_Option &al)
     int num_samps, samp_rate;
 
     if (al.present("-display"))
-	auservername = strdup(al.val("-display"));
+	auservername = wstrdup(al.val("-display"));
 
     aud = AuOpenServer(auservername, 0, NULL, 0, NULL, NULL);
     if (aud == NULL) 
@@ -98,7 +102,7 @@ int play_na_wave(EST_Wave &inwave, EST_Option &al)
 
     /* Check sample rate of server -- should really check the if it   */
     /* only supports individual sample rate of which this wave is not */
-    /* one then we should reample                                     */
+    /* one then we should resample.                                   */
     samp_rate = inwave.sample_rate();
     bool samp_rate_ok = FALSE;
     for (i=0; dev_sr[i] != -1; i++)
@@ -148,10 +152,20 @@ int play_na_wave(EST_Wave &inwave, EST_Option &al)
 
     return 1;
 }
+
+int record_nas_wave(EST_Wave &wave, EST_Option &al)
+{
+    (void)wave;
+    (void)al;
+
+    cerr << "NAS: record not written yet\n";
+    return -1;
+}
+
 #else
 int nas_supported = FALSE;
 
-int play_na_wave(EST_Wave &inwave, EST_Option &al)
+int play_nas_wave(EST_Wave &inwave, EST_Option &al)
 {
     (void)inwave;
     (void)al;
@@ -159,4 +173,13 @@ int play_na_wave(EST_Wave &inwave, EST_Option &al)
     return -1;
 }
 
+
+int record_nas_wave(EST_Wave &wave, EST_Option &al)
+{
+    (void)wave;
+    (void)al;
+    cerr << "NAS record not supported" << endl;
+    return -1;
+}
+
 #endif
