@@ -5,15 +5,19 @@
 /*                      Copyright (c) 1995,1996                          */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -33,11 +37,13 @@
 /*                                                                       */
 /*=======================================================================*/
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <fstream.h>
-#include <math.h>
-#include <limits.h>
+#include <cstdlib>
+#include <cstdio>
+#include <fstream>
+#include <cmath>
+#include <climits>
+using namespace std;
+
 #include "EST_String.h"
 #include "EST_types.h"
 #include "EST_FileType.h"
@@ -45,17 +51,15 @@
 #include "EST_FMatrix.h"
 #include "EST_cutils.h"  // for swap functions 
 #include "EST_Token.h"
-#include "rateconv.h"
 
-EST_IMatrix::EST_IMatrix(EST_IMatrix &a, int b) 
-:EST_TSimpleMatrix<int>(a.num_rows(), a.num_columns())
-{
-    int vv = 0;
-    if (b < 0)
-	return;
-    if (b == 0)
-	fill(vv);
-}
+
+/* EST_FVector may used as EST_Val */
+VAL_REGISTER_CLASS(fvector,EST_FVector)
+
+/* EST_FMatrix may used as EST_Val */
+VAL_REGISTER_CLASS(fmatrix,EST_FMatrix)
+
+EST_String EST_FMatrix::default_file_type = "est_ascii";
 
 EST_FMatrix::EST_FMatrix(const EST_FMatrix &a, int b)
 :EST_TSimpleMatrix<float>(a.num_rows(), a.num_columns())
@@ -80,13 +84,55 @@ EST_FMatrix & EST_FMatrix::operator+=(const EST_FMatrix &a)
 	cerr <<"Matrix addition error: bad number of rows\n";
 	return *this;
     }
-    for (i = 0; i < a.num_rows(); ++i)
-	for (j = 0; j < a.num_columns(); ++j)
+    for (i = 0; i < num_rows(); ++i)
+	for (j = 0; j < num_columns(); ++j)
 	    a_no_check(i, j) += a.a_no_check(i,j);
 
     return *this;
 }
 
+EST_FMatrix & EST_FMatrix::operator-=(const EST_FMatrix &a)
+{
+    int i, j;
+    if (a.num_columns() != num_columns())
+    {
+	cerr <<"Matrix subtraction error: bad number of columns\n";
+	return *this;
+    }
+    if (a.num_rows() != num_rows())
+    {
+	cerr <<"Matrix subtraction error: bad number of rows\n";
+	return *this;
+    }
+    for (i = 0; i < num_rows(); ++i)
+	for (j = 0; j < num_columns(); ++j)
+	    a_no_check(i, j) -= a.a_no_check(i,j);
+
+    return *this;
+}
+
+EST_FMatrix & EST_FMatrix::operator*=(const float f)
+{
+
+    int i,j;
+    for (i = 0; i < num_rows(); ++i)
+	for (j = 0; j < num_columns(); ++j)
+	    a_no_check(i, j) *= f;
+
+    return *this;
+}
+
+EST_FMatrix & EST_FMatrix::operator/=(const float f)
+{
+
+    int i,j;
+    for (i = 0; i < num_rows(); ++i)
+	for (j = 0; j < num_columns(); ++j)
+	    a_no_check(i, j) /= f;
+
+    return *this;
+}
+
 EST_FMatrix operator+(const EST_FMatrix &a, const EST_FMatrix &b)
 {
     EST_FMatrix ab;
@@ -145,6 +191,19 @@ EST_FMatrix operator*(const EST_FMatrix &a, const float x)
     return b;
 }
 
+int operator !=(const EST_FVector &fv1, 
+		const EST_FVector &fv2)
+{
+    int i;
+    if(fv1.length() != fv2.length())
+	return FALSE;
+    for(i=0;i<fv1.length();i++)
+	if(fv1.a_no_check(i) != fv2.a_no_check(i))
+	    return FALSE;
+
+    return TRUE;
+}
+
 EST_FVector operator*(const EST_FMatrix &a, const EST_FVector &v)
 {    
     // treat the vector as a column vector
@@ -169,6 +228,41 @@ EST_FVector operator*(const EST_FMatrix &a, const EST_FVector &v)
     return b;
 }
 
+EST_FVector operator+(const EST_FVector &a, const EST_FVector &b)
+{
+    EST_FVector ab;
+    int i;
+    if (a.length() != b.length())
+    {
+	cerr <<"Vector addition error: mismatched lengths\n";
+	return ab;
+    }
+
+    ab.resize(a.length());
+    for (i = 0; i < a.length(); ++i)
+	ab.a_no_check(i) = a.a_no_check(i) + b.a_no_check(i);
+
+    return ab;
+}
+
+EST_FVector operator-(const EST_FVector &a, const EST_FVector &b)
+{
+    EST_FVector ab;
+    int i;
+    if (a.length() != b.length())
+    {
+	cerr <<"Vector subtraction error: mismatched lengths\n";
+	return ab;
+    }
+
+    ab.resize(a.length());
+    for (i = 0; i < a.length(); ++i)
+	ab.a_no_check(i) = a.a_no_check(i) - b.a_no_check(i);
+
+    return ab;
+}
+
+
 EST_FVector operator*(const EST_FVector &v,const EST_FMatrix &a)
 {    
     // treat the vector as a row vector
@@ -194,7 +288,6 @@ EST_FVector operator*(const EST_FVector &v,const EST_FMatrix &a)
 }
 
 
-
 #if 0
 EST_FMatrix operator/(const EST_FMatrix &a, float x)
 {
@@ -424,12 +517,12 @@ EST_read_status EST_FMatrix::load(const EST_String &filename)
     if ((r = est_load(filename)) == format_ok)
 	return r;
     else if (r == wrong_format)
-    {   // maybe its an acient ascii file
+    {   // maybe its an ancient ascii file
 	EST_TokenStream ts, tt;
 	EST_StrList sl;
 	int i, j, n_rows=0, n_cols=0;
 	EST_String t;
-	EST_TBI *p;
+	EST_Litem *p;
 	if (((filename == "-") ? ts.open(cin) : ts.open(filename)) != 0)
 	{
 	    cerr << "Can't open fmatrix file " << filename << endl;
@@ -452,7 +545,7 @@ EST_read_status EST_FMatrix::load(const EST_String &filename)
 	// resize track and copy values in
 	resize(n_rows, n_cols);
 	
-	for (p = sl.head(), i = 0; p != 0; ++i, p = next(p))
+	for (p = sl.head(), i = 0; p != 0; ++i, p = p->next())
 	{
 	    tt.open_string(sl(p));
 	    for (j = 0; !tt.eof(); ++j)
@@ -470,6 +563,24 @@ EST_read_status EST_FMatrix::load(const EST_String &filename)
 	return r;
 }
 
+
+EST_FVector & EST_FVector::operator+=(const EST_FVector &s)
+{
+    int i;
+    if(n() != s.n()){
+	cerr << "Cannot elementwise add vectors of differing lengths" 
+	    << endl;
+	return *this;
+    }
+    
+    for (i = 0; i < n(); ++i)
+	(*this)[i] += s(i);
+
+    
+    return *this;
+}
+
+
 EST_FVector& EST_FVector::operator*=(const EST_FVector &s)
 {
     if(n() != s.n()){
@@ -484,6 +595,142 @@ EST_FVector& EST_FVector::operator*=(const EST_FVector &s)
     return *this;
 }
 
+EST_FVector& EST_FVector::operator*=(const float f)
+{
+    for (int i = 0; i < n(); ++i)
+	(*this)[i] *= f;
+
+    return *this;
+}
+
+
+EST_FVector& EST_FVector::operator/=(const float f)
+{
+    for (int i = 0; i < n(); ++i)
+	(*this)[i] /= f;
+
+    return *this;
+}
+
+
+
+EST_read_status EST_FVector::est_load(const EST_String &filename)
+{    
+    // ascii/binary load with short header for byte swap and sizes
+    int i,k;
+    int l, swap;
+    EST_TokenStream ts;
+    EST_read_status r;
+    EST_EstFileType t;
+    EST_Option hinfo;
+    bool ascii;
+    
+    if (((filename == "-") ? ts.open(cin) : ts.open(filename)) != 0)
+    {
+        cerr << "FVector: can't open FVector input file " 
+             << filename << endl;
+        return misc_read_error;
+    }
+    if ((r = read_est_header(ts, hinfo, ascii, t)) != format_ok)
+        return r;
+    if (t != est_file_fvector)
+        return misc_read_error;
+    if (hinfo.ival("version") != 1)
+    {
+	cerr << "FVector load: " << ts.pos_description() <<
+            " wrong version of FVector format expected 1 but found " <<
+            hinfo.ival("version") << endl;
+	return misc_read_error;
+    }
+    l = hinfo.ival("length");
+    resize(l);
+    
+    if (ascii)
+    {   // an ascii file
+	for (i = 0; i < length(); ++i)
+            a_no_check(i) = atof(ts.get().string());
+    }
+    else
+    {   // a binary file
+        float *buff;
+	if ((EST_BIG_ENDIAN && (hinfo.sval("ByteOrder")=="LittleEndian")) ||
+	    (EST_LITTLE_ENDIAN && (hinfo.sval("ByteOrder") == "BigEndian")))
+            swap = TRUE;
+	else
+            swap = FALSE;
+	
+	buff = walloc(float,l);
+	// A single read is *much* faster than multiple reads
+	if (ts.fread(buff,sizeof(float),l) != l)
+        {
+	    cerr << "EST_FVector: binload: short file in \""  
+		 << filename << "\"" << endl;
+	    return misc_read_error;
+        }
+	if (swap)
+            swap_bytes_float(buff,l);
+	for (k = i = 0; i < length(); ++i)
+            a_no_check(i) = buff[k++];
+	wfree(buff);
+    }
+    
+    ts.close();
+    return read_ok;
+}
+
+EST_read_status EST_FVector::load(const EST_String &filename)
+{    
+
+    EST_read_status r;
+
+    if ((r = est_load(filename)) == format_ok)
+	return r;
+    else if (r == wrong_format)
+    {   // maybe its an ancient ascii file
+      EST_TokenStream ts;
+      EST_String s;
+      int i;
+
+      i = 0;
+      
+      if (((filename == "-") ? ts.open(cin) : ts.open(filename)) != 0)
+	{
+	  cerr << "can't open vector input file " << filename << endl;
+	  return misc_read_error;
+	}
+      ts.set_SingleCharSymbols(";");
+      
+      while (!ts.eof())
+	{
+	  ts.get();
+	  ++i;
+	}
+      resize(i);
+      
+      ts.close();
+      if (((filename == "-") ? ts.open(cin) : ts.open(filename)) != 0)
+	{
+	  cerr << "can't open vector input file " << filename << endl;
+	  return misc_read_error;
+	}
+      
+      for (i = 0; !ts.eof(); ++i)
+	{
+	  s = ts.get().string();
+	  (*this)[i] = (float)(atof(s));  // actually returns double
+	}
+      ts.close();
+      return format_ok;
+    }
+    else
+      return r;
+
+    return format_ok;
+
+}
+
+/*
+
 EST_read_status EST_FVector::load(EST_String &filename)
 {    
     EST_TokenStream ts;
@@ -521,98 +768,47 @@ EST_read_status EST_FVector::load(EST_String &filename)
     ts.close();
     return format_ok;
 }
+*/
 
-EST_read_status EST_DVector::load(EST_String &filename)
-{    
-    EST_TokenStream ts;
-    EST_String s;
-    int i;
+//  EST_read_status EST_DVector::load(EST_String &filename)
+//  {    
+//      EST_TokenStream ts;
+//      EST_String s;
+//      int i;
 
-    i = 0;
+//      i = 0;
     
-    if (((filename == "-") ? ts.open(cin) : ts.open(filename)) != 0)
-    {
-	cerr << "can't open vector input file " << filename << endl;
-	return misc_read_error;
-    }
-    ts.set_SingleCharSymbols(";");
+//      if (((filename == "-") ? ts.open(cin) : ts.open(filename)) != 0)
+//      {
+//  	cerr << "can't open vector input file " << filename << endl;
+//  	return misc_read_error;
+//      }
+//      ts.set_SingleCharSymbols(";");
 
-    while (!ts.eof())
-    {
-	ts.get();
-	++i;
-    }
-    resize(i);
+//      while (!ts.eof())
+//      {
+//  	ts.get();
+//  	++i;
+//      }
+//      resize(i);
 
-    ts.close();
-    if (((filename == "-") ? ts.open(cin) : ts.open(filename)) != 0)
-    {
-	cerr << "can't open vector input file " << filename << endl;
-	return misc_read_error;
-    }
+//      ts.close();
+//      if (((filename == "-") ? ts.open(cin) : ts.open(filename)) != 0)
+//      {
+//  	cerr << "can't open vector input file " << filename << endl;
+//  	return misc_read_error;
+//      }
 
-    for (i = 0; !ts.eof(); ++i)
-    {
-	s = ts.get().string();
-	(*this)[i] = atof(s);  // actually returns double
-    }
-    ts.close();
-    return format_ok;
-}
+//      for (i = 0; !ts.eof(); ++i)
+//      {
+//  	s = ts.get().string();
+//  	(*this)[i] = atof(s);  // actually returns double
+//      }
+//      ts.close();
+//      return format_ok;
+//  }
     
 
-EST_SMatrix::EST_SMatrix(EST_SMatrix &a, int b) 
-:EST_TSimpleMatrix<short>(a.num_rows(), a.num_columns())
-{
-    short vv = 0;
-    if (b < 0)
-	return;
-    if (b == 0)
-	fill(vv);
-}
-
-int EST_SMatrix::rateconv(int in_samp_freq, int out_samp_freq)
-{
-  short *in_buf = new short[num_rows()];
-  short ** results = new short *[num_columns()];
-  int *len = new int[num_columns()];
-  int max_len=0;
-
-  for(int c=0; c<num_columns(); c++)
-    {
-      short *out_buf;
-      int osize;
-
-      copy_column(c, in_buf);
-
-      if (::rateconv(in_buf,
-		   num_rows(), &out_buf, &osize,
-		   in_samp_freq, out_samp_freq) == 0)
-	{
-	  results[c]=out_buf;
-	  len[c]=osize;
-	  if (osize > max_len)
-	    max_len = osize;
-	}
-      else
-	return -1;
-    }
-  delete [] in_buf;
-
-  resize(max_len, EST_CURRENT, 0);
-  fill(0);
-
-  for(int c1=0; c1<num_columns(); c1++)
-    {
-      set_column(c1, results[c1], 0, len[c1]);
-      delete [] results[c1];
-    }
-
-  delete [] results;
-  delete [] len;
-  return 0;
-}
-
 float operator*(const EST_FVector &v1, const EST_FVector &v2)
 {    
     // dot product
@@ -633,3 +829,92 @@ float operator*(const EST_FVector &v1, const EST_FVector &v2)
     return b;
 }
 
+
+EST_write_status EST_FVector::save(const EST_String &filename,
+				   const EST_String &type)
+{
+
+    if ((type == "est_ascii") || (type == "est_binary"))
+	return est_save(filename,type);
+    else
+    {   // the old stuff raw unheadered
+	int i;
+	ostream *outf;
+	if (filename == "-")
+	    outf = &cout;
+	else
+	    outf = new ofstream(filename);
+
+	outf->precision(25);
+	if (!(*outf))
+	{
+	    cerr << "FVector: can't open file \"" << filename 
+		<<"\" for writing" << endl;
+	    return misc_write_error;
+	}
+    
+	for (i = 0; i < length(); ++i)
+	    *outf << a_no_check(i) << " ";
+	*outf << endl;
+	
+	if (outf != &cout)
+	    delete outf;
+	
+	return write_ok;
+    }
+}
+
+EST_write_status EST_FVector::est_save(const EST_String &filename,
+				      const EST_String &type)
+{
+    // Binary save with short header for byte swap and sizes
+    int i;
+    FILE *fd;
+    if (filename == "-")
+	fd = stdout;
+    else if ((fd = fopen(filename, "wb")) == NULL)
+    {
+	cerr << "EST_FVector: binsave: failed to open \"" << filename << 
+	    "\" for writing" << endl;
+	return misc_write_error;
+    }
+
+    fprintf(fd,"EST_File fvector\n");
+    fprintf(fd,"version 1\n");
+    if (type == "est_binary")
+    {
+	fprintf(fd,"DataType binary\n");
+	if (EST_LITTLE_ENDIAN)
+	    fprintf(fd,"ByteOrder LittleEndian\n");
+	else
+	    fprintf(fd,"ByteOrder BigEndian\n");
+    }
+    else
+	fprintf(fd,"DataType ascii\n");
+
+    fprintf(fd,"length %d\n",length());
+    fprintf(fd,"EST_Header_End\n");
+
+    if (type == "est_binary")
+    {
+	for (i = 0; i < length(); ++i)
+	    if (fwrite(&a_no_check(i),sizeof(float),1,fd) != 1)
+	    {
+		cerr << "EST_FVector: binsave: failed to write item " 
+		     << i << " to \"" << filename << "\"" << endl;
+		return misc_write_error;
+	    }
+    }
+    else
+    {   // est_ascii
+	for (i = 0; i < length(); ++i)
+	    fprintf(fd,"%.25f ",a_no_check(i));
+	fprintf(fd,"\n");
+    }
+    
+    if (fd != stdout)
+	fclose(fd);
+
+    return write_ok;
+}
+
