@@ -5,15 +5,19 @@
  /*                       Copyright (c) 1996,1997                        */
  /*                        All Rights Reserved.                          */
  /*                                                                      */
- /*  Permission to use, copy, modify, distribute this software and its   */
- /*  documentation for research, educational and individual use only, is */
- /*  hereby granted without fee, subject to the following conditions:    */
+ /*  Permission is hereby granted, free of charge, to use and distribute */
+ /*  this software and its documentation without restriction, including  */
+ /*  without limitation the rights to use, copy, modify, merge, publish, */
+ /*  distribute, sublicense, and/or sell copies of this work, and to     */
+ /*  permit persons to whom this work is furnished to do so, subject to  */
+ /*  the following conditions:                                           */
  /*   1. The code must retain the above copyright notice, this list of   */
  /*      conditions and the following disclaimer.                        */
  /*   2. Any modifications must be clearly marked as such.               */
  /*   3. Original authors' names are not deleted.                        */
- /*  This software may not be used for commercial purposes without       */
- /*  specific prior written permission from the authors.                 */
+ /*   4. The authors' names are not used to endorse or promote products  */
+ /*      derived from this software without specific prior written       */
+ /*      permission.                                                     */
  /*                                                                      */
  /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK       */
  /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING     */
@@ -39,6 +43,7 @@
  /*                                                                      */
  /************************************************************************/
 
+#include <cstdio>
 #include "EST_System.h"
 #include "EST_Pathname.h"
 
@@ -138,9 +143,11 @@ EST_Pathname EST_Pathname::append(EST_Pathname directory, EST_Pathname addition)
   if (addition.is_absolute())
     return addition;
 
+  EST_String add(addition);
+
   EST_Pathname result(directory.as_directory());
 
-  result += addition;
+  result.EST_String::operator += (add);
 
   return result;
 }
@@ -180,3 +187,15 @@ EST_String EST_Pathname::basename(int remove_all) const
       }
     return result;
 }
+
+EST_Pathname operator + (const EST_Pathname p, const EST_Pathname addition) 
+{return EST_Pathname::append(p, addition); }
+
+EST_Pathname operator + (const char *p, const EST_Pathname addition) 
+{return EST_Pathname::append(p, addition); }
+
+EST_Pathname &operator += (EST_Pathname p, const EST_Pathname addition)
+{ p = EST_Pathname::append(p, addition); return p; }
+EST_Pathname &operator += (EST_Pathname p, const EST_String addition)
+{ p = EST_Pathname::append(p, addition); return p; }
+
