@@ -5,15 +5,19 @@
  /*                       Copyright (c) 1996,1997                        */
  /*                        All Rights Reserved.                          */
  /*                                                                      */
- /*  Permission to use, copy, modify, distribute this software and its   */
- /*  documentation for research, educational and individual use only, is */
- /*  hereby granted without fee, subject to the following conditions:    */
+ /*  Permission is hereby granted, free of charge, to use and distribute */
+ /*  this software and its documentation without restriction, including  */
+ /*  without limitation the rights to use, copy, modify, merge, publish, */
+ /*  distribute, sublicense, and/or sell copies of this work, and to     */
+ /*  permit persons to whom this work is furnished to do so, subject to  */
+ /*  the following conditions:                                           */
  /*   1. The code must retain the above copyright notice, this list of   */
  /*      conditions and the following disclaimer.                        */
  /*   2. Any modifications must be clearly marked as such.               */
  /*   3. Original authors' names are not deleted.                        */
- /*  This software may not be used for commercial purposes without       */
- /*  specific prior written permission from the authors.                 */
+ /*   4. The authors' names are not used to endorse or promote products  */
+ /*      derived from this software without specific prior written       */
+ /*      permission.                                                     */
  /*                                                                      */
  /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK       */
  /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING     */
@@ -40,29 +44,39 @@
 template<class K, class V>
 EST_THash<K,V>::EST_THash(int size,  unsigned int (*hash_function)(const K &key, unsigned int size))
 {
-p_num_entries =0;
+    p_num_entries =0;
 
-p_num_buckets = size;
+    p_num_buckets = size;
 
-p_buckets = new EST_Hash_Pair<K,V> *[size];
-for(int i=0; i<size;i++)
-  p_buckets[i] = NULL;
+    p_buckets = new EST_Hash_Pair<K,V> *[size];
+    for(int i=0; i<size;i++)
+	p_buckets[i] = NULL;
 
-p_hash_function = hash_function;
+    p_hash_function = hash_function;
+}
+
+template<class K, class V>
+EST_THash<K,V>::EST_THash(const EST_THash<K,V> &from)
+{
+  p_buckets=NULL;
+  copy(from);
 }
 
 template<class K, class V>
 void EST_THash<K,V>::clear(void)
 {
-  for(unsigned int i=0; i<p_num_buckets;i++)
+  if (p_buckets != NULL)
     {
-      EST_Hash_Pair<K,V> *p, *n;
-      for(p=p_buckets[i]; p != NULL; p=n)
+      for(unsigned int i=0; i<p_num_buckets;i++)
 	{
-	  n = p->next;
-	  delete p;
+	  EST_Hash_Pair<K,V> *p, *n;
+	  for(p=p_buckets[i]; p != NULL; p=n)
+	    {
+	      n = p->next;
+	      delete p;
+	    }
+	  p_buckets[i]=NULL;
 	}
-      p_buckets[i]=NULL;
     }
   p_num_entries=0;
 }
@@ -77,6 +91,7 @@ EST_THash<K,V>::~EST_THash(void)
     }
 }
 
+
 template<class K, class V>
 int EST_THash<K,V>::present(const K &key) const
 {
@@ -89,7 +104,7 @@ int EST_THash<K,V>::present(const K &key) const
   EST_Hash_Pair<K,V> *p;
 
   for(p=p_buckets[b]; p!=NULL; p=p->next)
-    if (p->key == key)
+    if (p->k == key)
       return TRUE;
   
 return FALSE;
@@ -107,16 +122,34 @@ V &EST_THash<K,V>::val(const K &key, int &found) const
   EST_Hash_Pair<K,V> *p;
 
   for(p=p_buckets[b]; p!=NULL; p=p->next)
-    if (p->key == key)
+    if (p->k == key)
       {
 	found=1;
-	return p->value;
+	return p->v;
       }
   
 found=0;
 return Dummy_Value;
 }
 
+template<class K, class V>
+const K &EST_THash<K,V>::key(const V &val, int &found) const
+{
+
+  for(unsigned int b=0; b<p_num_buckets; b++)
+	{
+	EST_Hash_Pair<K,V> *p;
+	  for(p=p_buckets[b]; p!=NULL; p=p->next)
+	    if (p->v == val)
+	      {
+		found=1;
+		return p->k;
+	      }
+	}
+found=0;
+return Dummy_Key;
+}
+
 template<class K, class V>
 void EST_THash<K,V>::map(void (*func)(K&, V&))
 {
@@ -125,7 +158,7 @@ void EST_THash<K,V>::map(void (*func)(K&, V&))
       EST_Hash_Pair<K,V> *p;
 
       for(p=p_buckets[i]; p!=NULL; p=p->next)
-	(*func)(p->key, p->value);
+	(*func)(p->k, p->v);
     }
   
 }
@@ -143,17 +176,18 @@ int EST_THash<K,V>::add_item(const K &key, const V &value, int no_search)
 
   if (!no_search)
     for(p=p_buckets[b]; p!=NULL; p=p->next)
-      if (p->key == key)
+      if (p->k == key)
 	{
-	  p->value = value;
+	  p->v = value;
 	  return FALSE;
 	}
       
   p = new EST_Hash_Pair<K,V>;
-  p->key = key;
-  p->value = value;
+  p->k = key;
+  p->v = value;
   p->next = p_buckets[b];
   p_buckets[b] = p;
+  p_num_entries++;
   return TRUE;
 }
 
@@ -169,11 +203,12 @@ int EST_THash<K,V>::remove_item(const K &rkey, int quiet)
   EST_Hash_Pair<K,V> **p;
 
   for (p = &(p_buckets[b]); *p!=NULL; p=&((*p)->next))
-    if ( (*p)->key == rkey )
+    if ( (*p)->k == rkey )
       {
 	EST_Hash_Pair<K,V> *n = (*p)->next;
 	delete *p;
 	*p = n;
+	p_num_entries--;
 	return 0;
       }
       
@@ -182,6 +217,12 @@ int EST_THash<K,V>::remove_item(const K &rkey, int quiet)
   return -1;
 }
 
+template<class K, class V>
+EST_THash<K,V> &EST_THash<K,V>::operator = (const EST_THash<K,V> &from)
+{
+  copy(from);
+  return *this;
+}
 
 template<class K, class V>
 void EST_THash<K,V>::dump(ostream &stream, int all)
@@ -192,13 +233,38 @@ void EST_THash<K,V>::dump(ostream &stream, int all)
 	stream << i << ": ";
 	EST_Hash_Pair<K,V> *p;
 	for(p=p_buckets[i]; p!=NULL; p=p->next)
-	  stream << "[" << p->key << "],(" << p->value << ") ";
+	  stream << "[" << p->k << "],(" << p->v << ") ";
 	stream << "\n";
       }
 }
 
+template<class K, class V>
+void EST_THash<K,V>::copy(const EST_THash<K,V> &from)
+{
+  clear();
+  p_num_entries = from.p_num_entries;
+  p_num_buckets = from.p_num_buckets;
+  p_hash_function = from.p_hash_function;
+
+  if (p_buckets != NULL)
+    delete [] p_buckets;
+
+  p_buckets = new EST_Hash_Pair<K,V> *[p_num_buckets];
+
+
+  for(unsigned int b=0; b<p_num_buckets; b++)
+    {
+      p_buckets[b]=NULL;
+      for(EST_Hash_Pair<K,V> *p=from.p_buckets[b]; p; p=p->next)
+	{
+	  EST_Hash_Pair<K,V> *n = new EST_Hash_Pair<K,V>(*p);
+	  n->next = p_buckets[b];
+	  p_buckets[b]=n;
+	}
+    }
+}
+
 
 
 
 
-
