@@ -39,7 +39,8 @@
 #include "EST_String.h"
 #include "EST_THash.h"
 
-Declare_TStringHash(double)
+template <> EST_String EST_THash<EST_String, double>::Dummy_Key = "DUMMY";
+template <> double EST_THash<EST_String, double>::Dummy_Value = 0.0;
 
 #if defined(INSTANTIATE_TEMPLATES)
 
