@@ -5,13 +5,19 @@
  /*                        Copyright (c) 1997                             */
  /*                        All Rights Reserved.                           */
  /*                                                                       */
- /*  Permission to use, copy, modify, distribute and sell this software   */
- /*  and its documentation for any purpose is  hereby granted without     */
- /*  fee, subject to the following conditions:                            */
- /*   1. The code must retain the above copyright notice, this list of    */
- /*      conditions and the following disclaimer.                         */
- /*   2. Any modifications must be clearly marked as such.                */
- /*   3. Original authors' names are not deleted.                         */
+ /*  Permission is hereby granted, free of charge, to use and distribute */
+ /*  this software and its documentation without restriction, including  */
+ /*  without limitation the rights to use, copy, modify, merge, publish, */
+ /*  distribute, sublicense, and/or sell copies of this work, and to     */
+ /*  permit persons to whom this work is furnished to do so, subject to  */
+ /*  the following conditions:                                           */
+ /*   1. The code must retain the above copyright notice, this list of   */
+ /*      conditions and the following disclaimer.                        */
+ /*   2. Any modifications must be clearly marked as such.               */
+ /*   3. Original authors' names are not deleted.                        */
+ /*   4. The authors' names are not used to endorse or promote products  */
+ /*      derived from this software without specific prior written       */
+ /*      permission.                                                     */
  /*                                                                       */
  /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
  /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -37,12 +43,14 @@
  /*************************************************************************/
 
 
-#include <iostream.h>
-#include <string.h>
-#include <stdio.h>
-#include <ctype.h>
+#include <iostream>
+#include <cstring>
+#include <cstdio>
+#include <cctype>
 #include "EST_String.h"
+// #include "EST_error.h"
 #include "string_version.h"
+#include "EST_math.h"
 
 extern "C" {
 #include "regexp.h"
@@ -97,7 +105,7 @@ int EST_String::locate(const char *s, int len, int from, int &start, int &end) c
 	  sub=nextsub;
 	}
     }
-  else if (from>=0 && from < size)
+  else if (from>=0 && from <= size)
     sub= strstr(str()+from, s);
   
   if (sub != NULL)
@@ -134,7 +142,7 @@ int EST_String::locate(EST_Regex &ex, int from, int &start, int &end, int *start
 	}
       return found >0;
     }
-  else if (from >=0 && from < size)
+  else if (from >=0 && from <= size)
     {
       if (ex.run(str(), from, match_start, match_end, starts, ends))
 	{
@@ -237,7 +245,7 @@ EST_String EST_String::chop_internal(const char *it, int len, int from, EST_chop
 
 EST_String EST_String::chop_internal (EST_Regex &it, int from, EST_chop_direction mode) const
 {
-  int start, end;
+  int start=0, end=0;
   
   if (locate(it, from, start, end))
     switch (mode)
@@ -304,7 +312,7 @@ int EST_String::gsub_internal (const char *os, int olength, const char *s, int l
 	}
       else
 	{
-	  ::make_updatable(memory, size);
+	  cp_make_updatable(memory, size);
 	  to = memory;
 	}
 
@@ -408,7 +416,7 @@ int EST_String::gsub_internal (EST_Regex &ex, const char *s, int length)
 	}
       else
 	{
-	  ::make_updatable(memory, size);
+	  cp_make_updatable(memory, size);
 	  to = memory;
 	}
 
@@ -455,12 +463,113 @@ int EST_String::gsub_internal (EST_Regex &ex, const char *s, int length)
 
 }
 
-// pass in the two possible separators as pointers so we don't have to
-// duplicat eall the code. Inline definitions of the friend functions
+int EST_String::subst(EST_String source, 
+		      int (&starts)[EST_Regex_max_subexpressions], 
+		      int (&ends)[EST_Regex_max_subexpressions])
+{
+  int n=0, change=0;
+  EST_ChunkPtr new_memory;
+
+  const char *from;
+  char *to;
+  
+#if __GSUB_REENTRANT__
+  struct subst *substitutions=NULL;
+
+  int num_substitutions=0;
+#endif
+
+  // printf("match '%s'\n", (const char *)(*this));
+
+  int i;
+  if (size > 0)
+    {
+    int escaped=0;
+
+      for(i=0; i<size; i++)
+	{
+	  if (escaped)
+	    {
+	      if (memory[i] >= '0' &&memory[i] <= '9')
+		{
+		  int snum = memory[i] - '0';
+		  if (ends[snum] >= 0 && starts[snum] >=0)
+		    {
+		      if (num_substitutions <= n)
+			substitutions = wrealloc(substitutions, struct subst, (num_substitutions +=10));
+		
+		      substitutions[n].start = i-1;
+		      substitutions[n].end = i+1;
+		      substitutions[n].s = ((char *)(void *)(const char *)source.memory) + starts[snum];
+		      substitutions[n].slen = ends[snum] - starts[snum];
+		      change += substitutions[n].slen - 2;
+
+		      n++;
+		    }
+		}
+	      escaped=0;
+	    }
+	  else if (memory[i] == '\\')
+	    escaped=1;
+	}
+
+
+      // dubious dealings with the inside of the string
+
+      from = (const char *)memory;
+
+      if (change > 0)
+	{
+	  // Spurious braces make temporary ref to chunk go away
+	  {new_memory = chunk_allocate(size+change+1);}
+	  to = new_memory;
+	}
+      else
+	{
+	  cp_make_updatable(memory, size);
+	  to = memory;
+	}
+
+      int at=0;
+      char *p=to;
+      
+      for(i=0; i<n; i++)
+	{
+	  int start = substitutions[i].start;
+	  int end = substitutions[i].end;
+	  memcpy(p, from+at, start-at);
+	  p += start-at;
+
+	  memcpy(p, substitutions[i].s, substitutions[i].slen);
+	  substitutions[i].s=NULL;
+	  p += substitutions[i].slen;
+	  at=end;
+	}
+      memcpy(p, from+at, size-at);
+
+      p += size-at;
+      *p = '\0';
+  
+      if (change > 0)
+	memory = new_memory; 
+      
+      size += change;
+    }
+
+#if __GSUB_REENTRANT__
+  if (substitutions)
+    wfree(substitutions);
+#endif
+
+  return n;
+}
+
+// Pass in the two possible separators as pointers so we don't have to
+// duplicate all the code. Inline definitions of the friend functions
 // takes care of the pretty interface.
 
 int EST_String::split_internal(EST_String result[], int max, 
-			       const EST_String *s_seperator, 
+			       const char *s_seperator, int slen,
 			       EST_Regex *re_seperator, 
 			       char quote) const
 {
@@ -498,7 +607,7 @@ int EST_String::split_internal(EST_String result[], int max,
 	    {
 	      int mstart, mend, matched;
 	      if (s_seperator)
-		matched = locate(*s_seperator, pos, mstart, mend);
+		matched = locate(s_seperator, slen, pos, mstart, mend);
 	      else
 		matched = locate(*re_seperator, pos, mstart, mend);
 	      
@@ -646,8 +755,8 @@ EST_String operator * (const EST_String &s, int n)
 
   EST_String it(NULL, 0, sz);
   
-  for(int i=0; i<n; i++)
-    strncpy(((char *)it)+i*l, (const char *)s, l);
+  for(int j=0; j<n; j++)
+    strncpy(((char *)it)+j*l, (const char *)s, l);
 
   return it;
 }
@@ -698,7 +807,6 @@ EST_String &EST_String::operator += (const EST_String b)
     return *this;
 }
 
-#if !__STRING_INLINE_AGGRESSIVELY__
 EST_String::EST_String(const char *s) 
 {
       CHECK_STRING_ARG(s);
@@ -736,8 +844,8 @@ EST_String::EST_String(const char *s, int start_or_fill, int len)
 	{
 	  memory = chunk_allocate(len+1);
 	  char *p = memory;
-	  for(int i=0; i<len;i++)
-	    p[i] = fill;
+	  for(int j=0; j<len;j++)
+	    p[j] = fill;
 	  p[len]='\0';
 	}
       else
@@ -809,6 +917,7 @@ EST_String &EST_String::operator = (const char *str)
       return *this;
 }
 
+#if 0
 EST_String &EST_String::operator = (const char c)
 {
       memory = chunk_allocate(2, &c, 1);
@@ -818,16 +927,28 @@ EST_String &EST_String::operator = (const char c)
 
 EST_String &EST_String::operator = (const EST_String &s) 
 {
-#if 1
-  static EST_ChunkPtr hack = s.memory;
-  memory = NON_CONST_CHUNKPTR(s.memory);
-  size = s.size;
-#else
-      *(struct EST_dumb_string *)this = *(struct EST_dumb_string *)(&s);
-#endif
-      return *this;
+    const char *str = (const char *)s;
+    CHECK_STRING_ARG(str);
+    int len = safe_strlen(str);
+    if (!len)
+	memory = NULL;
+    else if (!shareing() && len < size)
+	memcpy((char *)memory, str, len+1);
+    else if (len)
+	memory = chunk_allocate(len+1, str, len);
+    size=len;
+    return *this;
+    //      
+    //#if 1
+    ///*  static EST_ChunkPtr hack = s.memory;  */
+    //  memory = NON_CONST_CHUNKPTR(s.memory);
+    //  size = s.size;
+    //#else
+    //      *(struct EST_dumb_string *)this = *(struct EST_dumb_string *)(&s);
+    //#endif
+    //      return *this;
 }
-#endif    
+#endif
     
 EST_String downcase(const EST_String &s)
 {
@@ -894,7 +1015,7 @@ EST_String::freq(EST_Regex &ex) const
 {
   int pos=0;
   int n=0;
-  int start, end;
+  int start, end=0;
 
   while (locate(ex, pos, start, end))
     {
@@ -971,15 +1092,15 @@ ostream &operator << (ostream &s, const EST_String &str)
     return (s << "");
 }
 
-EST_String EST_String::cat(EST_String s1, 
-			  EST_String s2, 
-			  EST_String s3, 
-			  EST_String s4, 
-			  EST_String s5,
-			  EST_String s6,
-			  EST_String s7,
-			  EST_String s8,
-			  EST_String s9
+EST_String EST_String::cat(const EST_String s1, 
+			   const EST_String s2, 
+			   const EST_String s3, 
+			   const EST_String s4, 
+			   const EST_String s5,
+			   const EST_String s6,
+			   const EST_String s7,
+			   const EST_String s8,
+			   const EST_String s9
 			   )
 {
   int   len=(s1.length()+s2.length()+s3.length()+s4.length()+s5.length() +
@@ -1085,3 +1206,178 @@ int operator == (const EST_String &a, const EST_String &b)
     else 
 	return a.size == b.size && a(0) == b(0) && memcmp(a.str(),b.str(),a.size)==0;
 };
+
+EST_String EST_String::Number(int i, int b)
+{
+  char buf[64];
+  const char *format;
+
+  switch (b)
+    {
+    case 8:
+      format="0%o";
+      break;
+    case 10:
+      format="%d";
+      break;
+    case 16:
+      format="0x%x";
+      break;
+    default:
+      format="??%d??";
+      break;
+    }
+  sprintf(buf, format, i);
+
+  return EST_String(buf);
+}
+
+EST_String EST_String::Number(long i, int b)
+{
+  char buf[64];
+  const char *format;
+
+  switch (b)
+    {
+    case 8:
+      format="0%lo";
+      break;
+    case 10:
+      format="%ld";
+      break;
+    case 16:
+      format="0x%lx";
+      break;
+    default:
+      format="??%ld??";
+      break;
+    }
+  sprintf(buf, format, i);
+
+  return EST_String(buf);
+}
+
+EST_String EST_String::Number(float f)
+{
+  char buf[64];
+
+  sprintf(buf, "%f", f);
+
+  return EST_String(buf);
+}
+
+EST_String EST_String::Number(double d)
+{
+  char buf[64];
+
+  sprintf(buf, "%f", d);
+
+  return EST_String(buf);
+}
+
+long EST_String::Long(bool *valid) const
+{
+  char *end;
+
+  long val = strtol(str(), &end, 10);
+
+  if (end==NULL|| *end != '\0')
+  {
+    if (valid != NULL)
+      {
+	*valid=0;
+	return 0L;
+      }
+    else
+      {
+	printf("bad integer number format '%s'\n",
+		(const char *)str());
+	exit(0);
+      }
+  }
+
+  if (valid)
+    *valid=1;
+  
+  return val;
+}
+
+int EST_String::Int(bool *valid) const
+{
+  long val = Long(valid);
+
+  if (valid && !*valid)
+    return 0L;
+
+  if (val > INT_MAX || val < INT_MIN)
+  {
+    if (valid != NULL)
+      {
+	*valid=0;
+	return 0L;
+      }
+    else
+      {
+	printf("number out of range for integer %ld",
+	       val);
+	exit(0);
+      }
+  }
+
+  return val;
+}
+
+double EST_String::Double(bool *valid) const
+{
+  char *end;
+
+  double val = strtod(str(), &end);
+
+  if (end==NULL|| *end != '\0')
+  {
+    if (valid != NULL)
+      {
+	*valid=0;
+	return 0.0;
+      }
+    else
+      {
+	printf("bad decimal number format '%s'",
+		(const char *)str());
+	exit(0);
+      }
+  }
+
+  if (valid)
+    *valid=1;
+  
+  return val;
+}
+
+float EST_String::Float(bool *valid) const
+{
+  double val = Double(valid);
+
+  if (valid && !*valid)
+    return 0.0;
+
+  if (val > FLT_MAX || val < -FLT_MAX) 
+  {
+    if (valid != NULL)
+      {
+	*valid=0;
+	return 0.0;
+      }
+    else
+      {
+	printf("number out of range for float %f",
+	       val);
+	exit(0);
+      }
+  }
+
+  return val;
+}
+
+
+
