@@ -37,17 +37,21 @@
 /* An EST_Ngram class for building and manipulating bigrams trigrams etc */
 /*                                                                       */
 /*=======================================================================*/
-#include <iostream.h>
-#include <fstream.h>
-#include <string.h>
-#include <math.h>
-#include <limits.h>
-#include <float.h>
+#include <iostream>
+#include <fstream>
+#include <cstring>
+#include <cmath>
+#include <climits>
+#include <cfloat>
+
+using namespace std;
+
 #include "EST_Ngrammar.h"
 #include "EST_Pathname.h"
 #include "EST_Token.h"
 #include "EST_io_aux.h"
 
+
 const EST_DiscreteProbDistribution PSTnullProbDistribution;
 static EST_String NOVOCAB("NOVOCAB");
 
@@ -302,7 +306,7 @@ void EST_BackoffNgrammarState::print_freqs(ostream &os,
     // not right - just print out, then recurse through children
     // change to use 'backoff_traverse'
     
-    int k;
+    EST_Litem *k;
     double freq;
     EST_String name;
     for (k=p_pdf.item_start();
@@ -369,7 +373,7 @@ void EST_BackoffNgrammarState::zap()
 {
 
     // recursively delete this state and all its children
-    int k;
+    EST_Litem *k;
     double freq;
     EST_String name;
     for (k=p_pdf.item_start();
@@ -452,7 +456,8 @@ bool EST_BackoffNgrammarState::set_backoff_weight(const EST_StrVector &words, co
 
 void EST_BackoffNgrammarState::frequency_of_frequencies(EST_DVector &ff)
 {
-    int k,max=ff.n();
+    int max=ff.n();
+    EST_Litem *k;
     double freq;
     EST_String name;
     for (k=p_pdf.item_start();
@@ -579,7 +584,7 @@ bool EST_Ngrammar::init_dense_representation()
 	return false;
     }
     
-    p_num_states = (int)pow(vocab->length(),p_order-1);
+    p_num_states = (int)pow(float(vocab->length()),float(p_order-1));
     p_states = new EST_NgrammarState[p_num_states];
     for (i=0; i < p_num_states; i++)
 	p_states[i].init(i,pred_vocab);
@@ -597,7 +602,7 @@ bool EST_Ngrammar::init_sparse_representation()
 	return false;
     }
     
-    p_num_states = (int)pow(vocab->length(),p_order-1);
+    p_num_states = (int)pow(float(vocab->length()),float(p_order-1));
     p_states = new EST_NgrammarState[p_num_states];
     
     return (bool)(p_states != NULL);
@@ -798,7 +803,7 @@ bool EST_Ngrammar::build(const EST_StrList &filenames,
     bool skip_this;
     EST_String new_filename;
     EST_Litem *p;
-    for (p = filenames.head(); p; p = next(p))
+    for (p = filenames.head(); p; p = p->next())
     {
 	cerr << "Building from " << filenames(p) << endl;
 
@@ -1498,7 +1503,7 @@ bool EST_Ngrammar::compute_backoff_weights(const int mincount,
     
     backoff_restore_unigram_states();
     
-    Good_Turing_discount(*this,maxcount);
+    Good_Turing_discount(*this,maxcount,0.5);
     
     // and since some frequencies will have been set to zero
     // we have to prune away those branches of the tree
@@ -1581,7 +1586,7 @@ void EST_Ngrammar::prune_backoff_representation(EST_BackoffNgrammarState *start_
     // remove any branches with zero frequency count
     
     // find children of this state with zero freq and zap them
-    int k;
+    EST_Litem *k;
     double freq;
     EST_String name;
     for (k=start_state->pdf_const().item_start();
@@ -1678,8 +1683,15 @@ int EST_Ngrammar::find_dense_state_index(const EST_IVector &words,
 					 int index) const
 {
     int i,ind=0;
+    int vl,wa;
     for(i=0;i<p_order-1;i++)
-	ind = ind*vocab->length() + words.a_no_check(i+index);
+    {
+        vl = vocab->length();
+        wa = words.a_no_check(i+index);
+        if (wa < 0) wa = 0;
+        //        printf("awb_debug ngrammar i %d ind %d v.length() %d words.a_no_check() %d\n",i,ind,vl,wa);
+	ind = ind*vl + wa;
+    }
 	
     return ind;
 }
@@ -2000,7 +2012,7 @@ double EST_Ngrammar::reverse_probability(const EST_StrVector &words,
       case EST_Ngrammar::dense:
         {
 	    const EST_NgrammarState &s = find_state_const(words);
-	    // need number of occurences of words[p_order-1]
+	    // need number of occurrences of words[p_order-1]
 	    return s.frequency(lastword(words))/
 		vocab_pdf.frequency(lastword(words));
         }
@@ -2029,7 +2041,7 @@ double EST_Ngrammar::reverse_probability(const EST_IVector &words,
       case EST_Ngrammar::dense:
         { 
 	    const EST_NgrammarState &s = find_state_const(words);
-	    // need number of occurences of words[p_order-1]
+	    // need number of occurrences of words[p_order-1]
 	    return s.frequency(lastword(words))/
 		vocab_pdf.frequency(lastword(words));
 	}
@@ -2320,7 +2332,8 @@ void EST_Ngrammar::print_freqs(ostream &os,double floor)
 	backoff_representation->print_freqs(os,p_order);
     else
     {
-	int i,j,k;
+	int i,j;
+        EST_Litem *k;
 	EST_IVector window(p_order-1);
 	
 	for (i=0; i < p_num_states; i++)
@@ -2661,7 +2674,7 @@ EST_Ngrammar::backoff_traverse(EST_BackoffNgrammarState *start_state,
     function(start_state,params);
     
     // and recurse down the tree
-    int k;
+    EST_Litem *k;
     double freq;
     EST_String name;
     for (k=start_state->pdf_const().item_start();
@@ -2692,7 +2705,7 @@ EST_Ngrammar::backoff_traverse(EST_BackoffNgrammarState *start_state,
     {
 	// and recurse down the tree if we haven't
 	// reached the level yet
-	int k;
+	EST_Litem *k;
 	double freq;
 	EST_String name;
 	
