@@ -38,10 +38,14 @@
 /*   union, composition                                                  */
 /*                                                                       */
 /*=======================================================================*/
-#include <iostream.h>
-#include <stdlib.h>
+#include <iostream>
+#include <cstdlib>
 #include "EST_WFST.h"
 #include "wfst_aux.h"
+#include "EST_String.h"
+#include "EST_TList.h"
+#include "EST_TKVL.h"
+#include "EST_THash.h"
 
 Declare_TList_T(EST_WFST_MultiState *,EST_WFST_MultiStateP)
 
@@ -53,8 +57,8 @@ Declare_TList_T(EST_WFST_MultiState *,EST_WFST_MultiStateP)
 	static EST_IList int_EST_IList_kv_def_EST_IList_s; 
 	static int int_EST_IList_kv_def_int_s; 
 	
-	EST_IList *EST_TKVL< int, EST_IList >::default_val=&int_EST_IList_kv_def_EST_IList_s; 
-	int *EST_TKVL< int, EST_IList >::default_key=&int_EST_IList_kv_def_int_s; 
+	template <> EST_IList *EST_TKVL< int, EST_IList >::default_val=&int_EST_IList_kv_def_EST_IList_s; 
+	template <> int *EST_TKVL< int, EST_IList >::default_key=&int_EST_IList_kv_def_int_s; 
 	
 	Declare_TList_N(KVI_int_EST_IList_t, 0)
 
@@ -71,7 +75,7 @@ Declare_TList_T(EST_WFST_MultiState *,EST_WFST_MultiStateP)
         template class EST_TKVL<int, EST_IList>; 
         template class EST_TKVI<int, EST_IList>; 
 // ostream &operator<<(ostream &s, EST_TKVI< int , EST_IList > const &i){  return s << i.k << "\t" << i.v << "\n"; } 
-//	ostream& operator << (ostream& s,  EST_TKVL< int , EST_IList > const &l) {EST_Litem *p; for (p = l.list.head(); p ; p = next(p)) s << l.list(p).k << "\t" << l.list(p).v << endl; return s;} 
+//	ostream& operator << (ostream& s,  EST_TKVL< int , EST_IList > const &l) {EST_Litem *p; for (p = l.list.head(); p ; p = p->next()) s << l.list(p).k << "\t" << l.list(p).v << endl; return s;} 
         Instantiate_TIterator_T(KVL_int_EST_IList_t, KVL_int_EST_IList_t::IPointer_k, int, KVL_int_EST_IList_kitt) 
         Instantiate_TStructIterator_T(KVL_int_EST_IList_t, KVL_int_EST_IList_t::IPointer, KVI_int_EST_IList_t, KVL_int_EST_IList_itt) 
         Instantiate_TIterator_T(KVL_int_EST_IList_t, KVL_int_EST_IList_t::IPointer, KVI_int_EST_IList_t, KVL_int_EST_IList_itt) 
@@ -102,7 +106,7 @@ void EST_WFST_MultiState::add(int i)
     EST_Litem *p;
 
     if (p_type == wfst_ms_set)
-	for (p=head(); p != 0; p=next(p))
+	for (p=head(); p != 0; p=p->next())
 	{
 	    if ((*this)(p) == i)
 		return;
@@ -116,21 +120,6 @@ void EST_WFST_MultiState::add(int i)
     append(i);
 }
 
-// Current indexing a multistate to a single number is done with
-// a StringTrie index.  It'll probably be worth optimising this
-// later.
-
-// Will remove this later
-typedef struct {
-    int name;
-} EST_WFST_MSindex;
-
-static void pi_delete(void *n)
-{
-    EST_WFST_MSindex *t = (EST_WFST_MSindex *)n;
-    delete t;
-}
-
 int multistate_index(EST_WFST_MultiStateIndex &index,
 		     EST_WFST_MultiState *ms,int proposed) 
 {
@@ -140,21 +129,36 @@ int multistate_index(EST_WFST_MultiStateIndex &index,
     // I'll have to make this more efficient in future.
     EST_String istring("");
     EST_Litem *p;
+    int ns,found;
 
-    for (p=ms->head(); p != 0; p = next(p))
+    for (p=ms->head(); p != 0; p = p->next())
 	istring += itoString((*ms)(p)) + " ";
 
-    EST_WFST_MSindex *msi;
-
-    if ((msi = (EST_WFST_MSindex *)index.lookup(istring)) == 0)
+    ns = index.val(istring,found);
+    if (found)
+	return ns;
+    else
     {
-	EST_WFST_MSindex *nr = new EST_WFST_MSindex;
-	nr->name = proposed;
-	index.add(istring,(void *)nr);
-	msi = nr;
+        index.add_item(istring,proposed);
+	return proposed;
     }
+}
+
+static int pair_check(EST_THash<int,int> &pairs_done, int i, int o, int odim)
+{
+    int p;
+    int found;
+
+    p = (i*odim)+o;  // unique number representing i/o pair
+
+    pairs_done.val(p,found);
+    if (!found)
+    {   // first time seeing this pair
+	pairs_done.add_item(p,1);
+	return 0;
+    }
+    return 1;
 
-    return msi->name;
 }
 
 void EST_WFST::determinize(const EST_WFST &ndwfst)
@@ -163,9 +167,10 @@ void EST_WFST::determinize(const EST_WFST &ndwfst)
     EST_WFST_MultiState *start_state,*nms,*current;
     int ns;
     Agenda multistate_agenda;
-    EST_WFST_MultiStateIndex index;
+    EST_WFST_MultiStateIndex index(100);
     int i,o, new_name;
     int c=0;
+    EST_Litem *sp, *tp;
 
     clear();
     p_in_symbols.copy(ndwfst.p_in_symbols);
@@ -183,6 +188,7 @@ void EST_WFST::determinize(const EST_WFST &ndwfst)
 
     while (multistate_agenda.length() > 0)
     {
+	EST_THash<int,int> pairs_done(100);
 	current = multistate_agenda.first();
 	multistate_agenda.remove(multistate_agenda.head());
 	if ((c % 100) == 99)
@@ -190,10 +196,20 @@ void EST_WFST::determinize(const EST_WFST &ndwfst)
 		<< multistate_agenda.length() << endl;
 	c++;
 
-	for (i=0; i < p_in_symbols.length(); i++)
-	{   // start at 2 to skip any and epsilon characters -- hmm bad
-	    for (o=0; o < p_out_symbols.length(); o++)
+	for (sp=current->head(); sp != 0; sp=sp->next())
+	{
+	    const EST_WFST_State *s = ndwfst.state((*current)(sp));
+	    for (tp=s->transitions.head(); tp != 0; tp = tp->next())
 	    {
+		i = s->transitions(tp)->in_symbol();
+		o = s->transitions(tp)->out_symbol();
+		// Need to check if i/o has already been proposed
+		if (pair_check(pairs_done,i,o,p_out_symbols.length()) == 1)
+		    continue;  // already prosed those
+//	for (i=0; i < p_in_symbols.length(); i++)
+//	{   // start at 2 to skip any and epsilon characters -- hmm bad
+//	    for (o=0; o < p_out_symbols.length(); o++)
+//	    {
 		if ((i==o) && (i==0)) 
 		    continue;  // don't deal here with epsilon transitions
 		nms = apply_multistate(ndwfst,current,i,o);
@@ -229,7 +245,6 @@ void EST_WFST::determinize(const EST_WFST &ndwfst)
 	// Probably want some progress summary
     }
 
-    index.clear(pi_delete);
 }
 
 EST_WFST_MultiState *EST_WFST::apply_multistate(const EST_WFST &wfst,
@@ -242,7 +257,7 @@ EST_WFST_MultiState *EST_WFST::apply_multistate(const EST_WFST &wfst,
     
     new_ms->clear();
     
-    for (p=ms->head(); p != 0; p=next(p))
+    for (p=ms->head(); p != 0; p=p->next())
 	// Add all new possible states from ms(p) given in/out
 	wfst.transition_all((*ms)(p),in,out,new_ms);
 
@@ -260,7 +275,7 @@ enum wfst_state_type EST_WFST::ms_type(EST_WFST_MultiState *ms) const
     EST_Litem *p;
     enum wfst_state_type r = wfst_nonfinal;
 
-    for (p=ms->head(); p != 0; p = next(p))
+    for (p=ms->head(); p != 0; p = p->next())
 	if (p_states((*ms)(p))->type() == wfst_error)
 	    return wfst_error;
 	else if (p_states((*ms)(p))->type() == wfst_licence)
@@ -285,7 +300,7 @@ void EST_WFST::transition_all(int state,
     const EST_WFST_State *s = p_states(state);
     EST_Litem *i;
 
-    for (i=s->transitions.head(); i != 0; i=next(i))
+    for (i=s->transitions.head(); i != 0; i=i->next())
     {
 	if ((in == s->transitions(i)->in_symbol()) &&
 	    (out == s->transitions(i)->out_symbol()))
@@ -295,7 +310,7 @@ void EST_WFST::transition_all(int state,
 
 static int is_a_member(const EST_IList &ii, int i)
 {
-    for (EST_Litem *p=ii.head(); p != 0; p=next(p))
+    for (EST_Litem *p=ii.head(); p != 0; p=p->next())
 	if (ii(p) == i)
 	    return TRUE;
     return FALSE;
@@ -304,21 +319,21 @@ static int is_a_member(const EST_IList &ii, int i)
 void EST_WFST::add_epsilon_reachable(EST_WFST_MultiState *ms) const
 {
     // As ms->add() adds in order we need to copy to a new list and append
-    // to it any new eplison accessible states
+    // to it any new epsilon accessible states
     EST_Litem *p;
     EST_IList ii;
     int ie = p_in_symbols.name(get_c_string(epsilon_label()));
     int oe = p_out_symbols.name(get_c_string(epsilon_label()));
 
-    for (p=ms->head(); p != 0; p=next(p))
+    for (p=ms->head(); p != 0; p=p->next())
 	ii.append((*ms)(p));
 
-    for (p=ii.head(); p != 0; p=next(p))
+    for (p=ii.head(); p != 0; p=p->next())
     {
 	const EST_WFST_State *s = p_states(ii(p));
 	EST_Litem *i;
 
-	for (i=s->transitions.head(); i != 0; i=next(i))
+	for (i=s->transitions.head(); i != 0; i=i->next())
 	{
 	    if ((ie == s->transitions(i)->in_symbol()) &&
 		(oe == s->transitions(i)->out_symbol()))
@@ -345,7 +360,7 @@ void EST_WFST::intersection(wfst_list &wl)
     EST_WFST_MultiState *nms,*current;
     int ns;
     Agenda multistate_agenda;
-    EST_WFST_MultiStateIndex index;
+    EST_WFST_MultiStateIndex index(100);
     int i,o, new_name, n;
     EST_Litem *p,*q;
     int c=0;
@@ -357,7 +372,7 @@ void EST_WFST::intersection(wfst_list &wl)
 
     // Determinize each input WFST and make a new start state consisting of 
     // the start states of each of the input WFSTs
-    for (p=wl.tail(); p != 0; p=prev(p))
+    for (p=wl.tail(); p != 0; p=p->prev())
     {
 	if (!wl(p).deterministic())
 	{
@@ -391,11 +406,11 @@ void EST_WFST::intersection(wfst_list &wl)
 		if ((i==o) && (i==0)) // shouldn't be epsilon/epsilon here 
 		    continue;
 		nms = new EST_WFST_MultiState(wfst_ms_list);
-		// Increment multistate to new mulitstate for each individual
+		// Increment multistate to new multistate for each individual
 		// state using each WFST
 		for (n=0,p=wl.head(),q=current->head(); 
 		     (p != 0) && (q != 0);
-		     p=next(p),q=next(q),n++)
+		     p=p->next(),q=q->next(),n++)
 		    nms->add(wl(p).transition((*current)(q),i,o));
 		if (intersect_state_type(wl,nms) == wfst_error)
 		{
@@ -424,7 +439,6 @@ void EST_WFST::intersection(wfst_list &wl)
 	// Probably want some progress summary
     }
 
-    index.clear(pi_delete);
 }
 
 static enum wfst_state_type intersect_state_type(wfst_list &wl,
@@ -438,7 +452,7 @@ static enum wfst_state_type intersect_state_type(wfst_list &wl,
 
     for (p=wl.head(),q=ms->head(); 
 	 (p != 0) && (q != 0); 
-	 p=next(p),q=next(q))
+	 p=p->next(),q=q->next())
     {
 	if ((*ms)(q) == WFST_ERROR_STATE)
 	    return wfst_error;
@@ -529,7 +543,7 @@ static int check_distinguished(const EST_WFST &nmwfst,
     }
     else 
     {   // Have to check their transitions individually
-	for (t=nmwfst.state(p)->transitions.head(); t != 0; t=next(t))
+	for (t=nmwfst.state(p)->transitions.head(); t != 0; t=t->next())
 	{
 	    int in = nmwfst.state(p)->transitions(t)->in_symbol();
 	    int out = nmwfst.state(p)->transitions(t)->out_symbol();
@@ -558,7 +572,7 @@ static int check_distinguished(const EST_WFST &nmwfst,
 	// assume they are undistinguished
 	add_assumption(p,q,assumptions);
 	for (yp=from_p.head(),zp=from_q.head(); 
-	     yp != 0; yp=next(yp),zp=next(zp))
+	     yp != 0; yp=yp->next(),zp=zp->next())
 	    if (check_distinguished(nmwfst,from_p(yp),from_q(zp),
 				    marks,
 				    assumptions))
@@ -581,7 +595,7 @@ static int check_distinguished(const EST_WFST &nmwfst,
 
 void EST_WFST::extend_alphabets(const EST_WFST &b)
 {
-    // Extend current in/out alphabets to accomodate anthing in b's
+    // Extend current in/out alphabets to accommodate anything in b's
     // that are not in a's
     // This guarantees that the number in this will still be valid
     EST_StrList ivocab, ovocab;
@@ -612,7 +626,7 @@ EST_WFST_State *EST_WFST::copy_and_map_states(const EST_IVector &state_map,
 
     ns->set_type(s->type());
 
-    for (i=s->transitions.head(); i != 0; i=next(i))
+    for (i=s->transitions.head(); i != 0; i=i->next())
     {
 	int mapped_state= state_map(s->transitions(i)->state());
 	if (mapped_state != WFST_ERROR_STATE)
@@ -656,7 +670,7 @@ static int noloopstostart(const EST_WFST &a)
     {
 	const EST_WFST_State *s = a.state(i);
 	EST_Litem *p;
-	for (p=s->transitions.head(); p != 0; p=next(p))
+	for (p=s->transitions.head(); p != 0; p=p->next())
 	{
 	    if (s->transitions(p)->state() == a.start_state())
 		return FALSE;
@@ -676,14 +690,14 @@ int EST_WFST::deterministiconstartstates(const EST_WFST &a, const EST_WFST &b) c
     tab.fill(0);
 
     for (EST_Litem *t=a.state(a.start_state())->transitions.head();
-	 t != 0; t=next(t))
+	 t != 0; t=t->next())
     {
 	tab(a.state(a.start_state())->transitions(t)->in_symbol(),
 	    a.state(a.start_state())->transitions(t)->out_symbol()) = 1;
     }
 
     for (EST_Litem *tt=b.state(b.start_state())->transitions.head();
-	 tt != 0; tt=next(tt))
+	 tt != 0; tt=tt->next())
     {
 	in = a.in_symbol(b.in_symbol(b.state(b.start_state())->transitions(tt)->in_symbol()));
 	out = a.out_symbol(b.out_symbol(b.state(b.start_state())->transitions(tt)->out_symbol()));
@@ -713,7 +727,7 @@ void EST_WFST::uunion(const EST_WFST &a,const EST_WFST &b)
 	deterministiconstartstates(a,b))
     {
 	// This does the union without the epsilon and will preserve
-	// determistic wfsts in this special case
+	// deterministic wfsts in this special case
 	smap.resize(b.num_states());
 	smap[0] = start_state();
 	for (i=1; i < b.num_states(); ++i)
@@ -726,7 +740,7 @@ void EST_WFST::uunion(const EST_WFST &a,const EST_WFST &b)
 
 	const EST_WFST_State *s = b.state(b.start_state());
 	EST_Litem *p;
-	for (p=s->transitions.head(); p != 0; p=next(p))
+	for (p=s->transitions.head(); p != 0; p=p->next())
 	{
 	    int mapped_state= smap(s->transitions(p)->state());
 	    if (mapped_state != WFST_ERROR_STATE)
@@ -800,7 +814,7 @@ void EST_WFST::compose(const EST_WFST &a,const EST_WFST &b)
     EST_WFST_MultiState *start_state = new EST_WFST_MultiState(wfst_ms_list);
     EST_WFST_MultiState *nms,*current;
     Agenda multistate_agenda;
-    EST_WFST_MultiStateIndex index;
+    EST_WFST_MultiStateIndex index(100);
     wfst_list wl;
     EST_WFST t;
     int i,new_name;
@@ -834,7 +848,7 @@ void EST_WFST::compose(const EST_WFST &a,const EST_WFST &b)
 	    wfst_translist transa;
 
 	    wl.first().transduce(current->first(),i,transa);
-	    for (p=transa.head(); p != 0; p=next(p))
+	    for (p=transa.head(); p != 0; p=p->next())
 	    {
 		wfst_translist transb;
 		// feed a's out to b'is in
@@ -842,7 +856,7 @@ void EST_WFST::compose(const EST_WFST &a,const EST_WFST &b)
                           current->last(),
 			  b.in_symbol(a.out_symbol(transa(p)->out_symbol())),
 			  transb);
-		for (q = transb.head(); q != 0; q=next(q))
+		for (q = transb.head(); q != 0; q=q->next())
 		{
 		    nms = new EST_WFST_MultiState(wfst_ms_list);
 		    nms->add(transa(p)->state());
@@ -876,7 +890,6 @@ void EST_WFST::compose(const EST_WFST &a,const EST_WFST &b)
 	// Probably want some progress summary
     }
 
-    index.clear(pi_delete);
 }
 
 /***********************************************************************/
@@ -937,7 +950,7 @@ int EST_WFST::can_reach_final(int state)
 	// temporarily set this to error to stop infinite recursion
 	p_states[state]->set_type(wfst_error);
 
-	for (i=p_states[state]->transitions.head(); i != 0; i=next(i))
+	for (i=p_states[state]->transitions.head(); i != 0; i=i->next())
 	    // if any transition goes to something that reaches a final state
 	    // set the value back to its original
 	    if (can_reach_final(p_states[state]->transitions(i)->state()))
@@ -969,7 +982,7 @@ int EST_WFST::deterministic() const
     for (int i=0; i < p_num_states; i++)
     {
 	tab.fill(0);
-	for (EST_Litem *t=state(i)->transitions.head(); t != 0; t=next(t))
+	for (EST_Litem *t=state(i)->transitions.head(); t != 0; t=t->next())
 	{
 	    if (tab(state(i)->transitions(t)->in_symbol(),
 		    state(i)->transitions(t)->out_symbol()) == 1)
