@@ -5,15 +5,19 @@
 /*                    Copyright (c) 1994,1995,1996                       */
 /*                         All Rights Reserved.                          */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -37,13 +41,13 @@
 #define __EST_OPTION_H__
 
 #include "EST_String.h"
-#include "EST_KV.h"
+#include "EST_TKVL.h"
 #include "EST_rw_status.h"
 
 /** Provide a high level interface for String String key value lists.
 */
 
-class EST_Option: public KVL<EST_String, EST_String> {
+class EST_Option: public EST_TKVL<EST_String, EST_String> {
 public:
     /// add prefix to every key 
     void add_prefix(EST_String prefix);
