@@ -5,13 +5,19 @@
 /*                        Copyright (c) 1997                             */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute and sell this software   */
-/*  and its documentation for any purpose is  hereby granted without     */
-/*  fee, subject to the following conditions:                            */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -31,19 +37,22 @@
 class EST_String;
 class EST_Regex;
 
-#include <string.h>
-#include "EST_iostream.h"
-#include <limits.h>
+#define EST_Regex_max_subexpressions 10
+
+#include <cstring>
+#include <iostream>
+#include <climits>
+using namespace std;
 #include "EST_Chunk.h"
-#include "EST_walloc.h"
 #include "EST_strcasecmp.h"
+#include "EST_bool.h"
 
 extern "C" void abort(void);
 
 /** A non-copyleft implementation of a string class to use with
   * compilers that aren't GNU C++.
   *
-  * Strings are reference-counted and reasonably efficiant (eg you
+  * Strings are reference-counted and reasonably efficient (eg you
   * can pass them around, into and out of functions and so on
   * without worrying too much about the cost).
   *
@@ -55,7 +64,7 @@ extern "C" void abort(void);
   * @see string_example
   * @author Alan W Black <awb@cstr.ed.ac.uk>
   * @author Richard Caley <rjc@cstr.ed.ac.uk>
-  * @version $Id: EST_String.h,v 1.1 1998/08/12 09:29:25 awb Exp $
+  * @version $Id: EST_String.h,v 1.11 2017/02/22 15:50:14 awb Exp $
   */
 
 class EST_String {
@@ -74,7 +83,7 @@ class EST_String {
       */
 #   define __GSUB_REENTRANT__ (1)
 
-/// Gripe about wierd arguments like Nulls
+/// Gripe about weird arguments like Nulls
 #define __STRING_ARG_GRIPE__ (1)
 
 ///  When we find something to gripe about we die then and there.
@@ -127,7 +136,7 @@ private:
 	Chop_After  = 1
     };
 
-    /// Simple utility which removes const-ness frommemory
+    /// Simple utility which removes const-ness from memory
     static inline EST_ChunkPtr &NON_CONST_CHUNKPTR(const EST_ChunkPtr &ecp) 
 	{ return *((EST_ChunkPtr *)&ecp);}
 
@@ -180,8 +189,12 @@ private:
     //@}
 
     /// Split the string down into parts. 
-    int split_internal(EST_String result[], int max, const EST_String* s_seperator, EST_Regex *re_separator, char quote) const;
+    int split_internal(EST_String result[], int max, const char* s_seperator, int slen, EST_Regex *re_separator, char quote) const;
 
+    int Int(bool *ok_p) const;
+    long Long(bool *ok_p) const;
+    float Float(bool *ok_p) const;
+    double Double(bool *ok_p) const;
 public:
 
     /// Construct an empty string.
@@ -201,17 +214,19 @@ public:
 
     /** Copy constructor
       * We have to declare our own copy constructor to lie to the
-      * compier about the constness of the RHS.
+      * compiler about the constness of the RHS.
       */
+    #if 0
     EST_String(const EST_String &s) {
       memory = NON_CONST_CHUNKPTR(s.memory);
       size = s.size;
     }
+    #endif
 
-#if __FSF_COMPATABILITY__
+#if __FSF_COMPATIBILITY__
     /** Construct from single char.
       * This constructor is not usually included as it can mask errors.
-      * @see  __FSF_COMPATABILITY__
+      * @see  __FSF_COMPATIBILITY__
       */
     EST_String(const char c);
 #endif
@@ -230,13 +245,41 @@ public:
     const char *str(void) const { return size==0?"":(const char *)memory; }
     /// Get a writable pointer to the actual memory.
     char *updatable_str(void) { return size==0?(char *)"":(char *)memory; }
-    void make_updatable(void) { ::make_updatable(memory, size+1);}
+    void make_updatable(void) { cp_make_updatable(memory, size+1);}
 
 
     /// Build string from a single character.
     static EST_String FromChar(const char c) 
 	{ const char s[2] = { c, 0 }; return EST_String(s); }
 
+    /// Build string from an integer.
+    static EST_String Number(int i, int base=10);
+
+    /// Build string from a long integer.
+    static EST_String Number(long i, int base=10);
+
+    /// Build string from a double.
+    static EST_String Number(double d);
+
+    /// Build string from a float
+    static EST_String Number(float f);
+
+    /// Convert to an integer
+    int Int(bool &ok) const { return Int(&ok); }
+    int Int(void) const { return Int((bool *)NULL); }
+
+    /// Convert to a long
+    long Long(bool &ok) const { return Long(&ok); }
+    long Long(void) const { return Long((bool *)NULL); }
+
+    /// Convert to a float
+    float Float(bool &ok) const { return Float(&ok); }
+    float Float(void) const { return Float((bool *)NULL); }
+
+    /// Convert to a double
+    double Double(bool &ok) const { return Double(&ok); }
+    double Double(void) const { return Double((bool *)NULL); }
+
     /**@name Before */
     //@{
     /// Part before position
@@ -305,7 +348,7 @@ public:
 
     /// Find a match of the regular expression.
     int search(EST_Regex &re, int &mlen, int pos=0, int *starts=NULL, int *ends=NULL) const
-	{ int start, end;
+	{ int start=0, end=0;
 	if (locate(re, pos, start, end, starts, ends))
 	{ mlen=end-start; return start; }
 	return -1;
@@ -344,9 +387,9 @@ public:
 
     /**@name Does string exactly match? */
     //@{
-    /// Exatly match this string?
+    /// Exactly match this string?
     int matches(const char *e, int pos=0) const;
-    /// Exatly match this string?
+    /// Exactly match this string?
     int matches(const EST_String &e, int pos=0) const;
     /// Exactly matches this regular expression, can return ends of sub-expressions.
     int matches(EST_Regex &e, int pos=0, int *starts=NULL, int *ends=NULL) const;
@@ -376,13 +419,17 @@ public:
     /// Substitute string for matches of regular expression.
     int gsub(EST_Regex &ex, int bracket_num)
 	{ return gsub_internal(ex, NULL, bracket_num); }
+    /// Substitute the result of a match into a string.
+    int subst(EST_String source, 
+	      int (&starts)[EST_Regex_max_subexpressions], 
+	      int (&ends)[EST_Regex_max_subexpressions]);
     //@}
 
     /**@name Frequency counts */
     //@{
-    /// Number of occurances of substring
+    /// Number of occurrences of substring
     int freq(const char *s) const;
-    /// Number of occurances of substring
+    /// Number of occurrences of substring
     int freq(const EST_String &s) const;
     /// Number of matches of regular expression.
     int freq(EST_Regex &s) const;
@@ -400,7 +447,7 @@ public:
     EST_String unquote_if_needed(const char quotec) const;
     //@}
 
-#if __FSF_COMPATABILITY__
+#if __FSF_COMPATIBILITY__
     const char operator [] (int i) const { return memory[i]; }
     char &operator () (int i) { return memory(i); }
 #else
@@ -414,6 +461,7 @@ public:
 
     /// Cast to const char * by simply giving access to pointer.
     operator const char*() const {return str(); }
+    operator const char*() {return str(); }
     /// Cast to char *, may involve copying.
     operator char*() { return updatable_str(); }
 
@@ -425,14 +473,16 @@ public:
     EST_String &operator += (const EST_String b);
     //@}
 
-    /**@name Asignment */
+    /**@name Assignment */
     //@{
     /// Assign C string to EST_String
     EST_String &operator = (const char *str);
     /// Assign single character to EST_String
+    #if 0
     EST_String &operator = (const char c);
     /// Assign EST_String to EST_String.
     EST_String &operator = (const EST_String &s);
+    #endif
     //@}
 
     /**@name Concatenation */
@@ -528,10 +578,14 @@ public:
       */
     //@{
     friend int fcompare(const EST_String &a, const EST_String &b, 
-			const unsigned char *table=NULL);
+			const unsigned char *table);
+
+	friend inline int fcompare(const EST_String &a, const EST_String &b)
+		{ return fcompare(a,b,NULL); }
+
 
     friend int fcompare(const EST_String &a, const char *b, 
-				const unsigned char *table=NULL);
+				const unsigned char *table);
     ///
     friend inline int fcompare(const EST_String &a, const EST_String &b, 
 			       const EST_String &table) 
@@ -550,11 +604,15 @@ public:
     /// Split at a given separator.
     friend int split(const EST_String & s, EST_String result[], 
 		     int max, const EST_String& seperator, char quote=0)
-	{ return s.split_internal(result, max, &seperator, NULL, quote); }
+	{ return s.split_internal(result, max, (const char *)seperator, seperator.length(), NULL, quote); }
+    /// Split at a given separator.
+    friend int split(const EST_String &s, EST_String result[], 
+		     int max, const char *seperator, char quote=0)
+	{ return s.split_internal(result, max, seperator, strlen(seperator), NULL, quote); }
     /// Split at each match of the regular expression.
     friend int split(const EST_String & s, EST_String result[], int max, 
 		     EST_Regex& seperator, char quote=0)
-	{ return s.split_internal(result, max, NULL, &seperator, quote); }
+	{ return s.split_internal(result, max, NULL, 0, &seperator, quote); }
     //@}
 
     /// Convert to upper case.
@@ -563,24 +621,32 @@ public:
     friend EST_String downcase(const EST_String &s);
 
     /** Concatenate a number of strings. 
-      * This is more efficiant than multiple uses of + or +=
+      * This is more efficient than multiple uses of + or +=
       */
-    static EST_String cat(EST_String s1, 
-			  EST_String s2 = Empty, 
-			  EST_String s3 = Empty, 
-			  EST_String s4 = Empty, 
-			  EST_String s5 = Empty,
-			  EST_String s6 = Empty,
-			  EST_String s7 = Empty,
-			  EST_String s8 = Empty,
-			  EST_String s9 = Empty
+    static EST_String cat(const EST_String s1, 
+			  const EST_String s2 = Empty, 
+			  const EST_String s3 = Empty, 
+			  const EST_String s4 = Empty, 
+			  const EST_String s5 = Empty,
+			  const EST_String s6 = Empty,
+			  const EST_String s7 = Empty,
+			  const EST_String s8 = Empty,
+			  const EST_String s9 = Empty
 	);
 
+  /*  Hacky way to ignore volatile */
+      EST_String & ignore_volatile(void) volatile { return *((EST_String *)(void *)this); }
+
     /// Stream output for EST_String.
     friend ostream &operator << (ostream &s, const EST_String &str);
     friend class EST_Regex;
+
 }; 
 
+EST_ChunkPtr chunk_allocate(int bytes);
+EST_ChunkPtr chunk_allocate(int bytes, const char *initial, int initial_len);
+EST_ChunkPtr chunk_allocate(int bytes, const EST_ChunkPtr &initial, int initial_start, int initial_len);
+
 int operator == (const char *a, const EST_String &b);
 int operator == (const EST_String &a, const EST_String &b);
 ostream &operator << (ostream &s, const EST_String &str);
