@@ -4,15 +4,19 @@
  /*                     University of Edinburgh, UK                       */
  /*                      Copyright (c) 1995,1996                          */
  /*                        All Rights Reserved.                           */
- /*  Permission to use, copy, modify, distribute this software and its    */
- /*  documentation for research, educational and individual use only, is  */
- /*  hereby granted without fee, subject to the following conditions:     */
+ /*  Permission is hereby granted, free of charge, to use and distribute  */
+ /*  this software and its documentation without restriction, including   */
+ /*  without limitation the rights to use, copy, modify, merge, publish,  */
+ /*  distribute, sublicense, and/or sell copies of this work, and to      */
+ /*  permit persons to whom this work is furnished to do so, subject to   */
+ /*  the following conditions:                                            */
  /*   1. The code must retain the above copyright notice, this list of    */
  /*      conditions and the following disclaimer.                         */
  /*   2. Any modifications must be clearly marked as such.                */
  /*   3. Original authors' names are not deleted.                         */
- /*  This software may not be used for commercial purposes without        */
- /*  specific prior written permission from the authors.                  */
+ /*   4. The authors' names are not used to endorse or promote products   */
+ /*      derived from this software without specific prior written        */
+ /*      permission.                                                      */
  /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
  /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
  /*  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   */
@@ -39,15 +43,42 @@
 #ifndef __Tlist_H__
 #define __Tlist_H__
 
-#include <iostream.h>
+#include <iostream>
+
+using namespace std;
+
 #include "EST_common.h"
-#include "EST_String.h"
 #include "EST_UList.h"
 #include "EST_TSortable.h"
+#include "EST_TIterator.h"
 
 #include "instantiate/EST_TListI.h"
 
+class EST_String;
+
+template<class T> class EST_TList;
+
 template<class T> class EST_TItem : public EST_UItem {
+private:
+  static void *operator new(size_t not_used, void *place)
+    {(void)not_used; return place;}
+  static void *operator new(size_t size)
+    {void *p;
+    p = (void *)walloc(char,size);
+     return p;} 
+  static void operator delete(void *p)
+    { wfree(p);}
+
+  static EST_TItem *s_free;
+  static unsigned int s_nfree;
+  static unsigned int s_maxFree;
+
+protected:
+  static EST_TItem *make(const T &val);
+  static void release(EST_TItem<T> *it);
+
+  friend class EST_TList<T>;
+  
 public:
     T val;
 
@@ -57,10 +88,6 @@ public:
 	    { init();};
 };
 
-// Legacy name
-
-typedef EST_UItem EST_TBI;
-
 // pretty name
 
 typedef EST_UItem EST_Litem;
@@ -99,6 +126,7 @@ template <class T> class EST_TList : public EST_UList
     See \Ref{EST_TList_Accessing} for examples.*/
 
   //@{
+
   /** return the value associated with the EST_Litem pointer. This
     has the same functionality as the overloaded () operator.
     */
@@ -132,7 +160,7 @@ template <class T> class EST_TList : public EST_UList
   /// return const reference to item in list pointed to by {\tt ptr}
     const T  &operator () (const EST_Litem *ptr) const
 	    { return item(ptr); };
-  /// return const reference to item in list pointed to by {\tt ptr}
+  /// return non-const reference to item in list pointed to by {\tt ptr}
     T  &operator () (const EST_Litem *ptr)
  	    { return item(ptr); };
 
@@ -161,22 +189,22 @@ template <class T> class EST_TList : public EST_UList
   //@{
   /// add item onto end of list
     void append(const T &item)
-	    { EST_UList::append(new EST_TItem<T>(item)); };
+	    { EST_UList::append(EST_TItem<T>::make(item)); };
   /// add item onto start of list
     void prepend(const T &item)
-	    { EST_UList::prepend(new EST_TItem<T>(item)); };
+	    { EST_UList::prepend(EST_TItem<T>::make(item)); };
 
   /** add {\tt item} after position given by {\tt ptr}, return pointer
     to added item. */
 
     EST_Litem *insert_after(EST_Litem *ptr, const T &item)
-	    { return EST_UList::insert_after(ptr, new EST_TItem<T>(item)); };
+	    { return EST_UList::insert_after(ptr, EST_TItem<T>::make(item)); };
 
   /** add {\tt item} before position given by {\tt ptr}, return
       pointer to added item. */
 
     EST_Litem *insert_before(EST_Litem *ptr, const T &item)
-	    { return EST_UList::insert_before(ptr, new EST_TItem<T>(item)); };
+	    { return EST_UList::insert_before(ptr, EST_TItem<T>::make(item)); };
 
   //@}
 
@@ -200,18 +228,39 @@ template <class T> class EST_TList : public EST_UList
     EST_TList<T> &operator +=(const EST_TList<T> &a);
 
   /// print list
-    friend ostream& operator << (ostream &st, EST_TList<T> const &list);
+    friend ostream& operator << (ostream &st, EST_TList<T> const &list) {
+        EST_Litem *ptr; 
+        for (ptr = list.head(); ptr != 0; ptr = ptr->next()) 
+            st << list.item(ptr) << " "; 
+        return st;
+    }
 
   /// remove all items in list
     void clear(void) 
 	    { clear_and_free(free_item); };
   //@}
+
+  // Iteration support
+
+protected:
+  struct IPointer {  EST_Litem *p; };
+
+  void point_to_first(IPointer &ip) const { ip.p = head(); }
+  void move_pointer_forwards(IPointer &ip) const { ip.p = ip.p->next(); }
+  bool points_to_something(const IPointer &ip) const { return ip.p != NULL; }
+  T &points_at(const IPointer &ip) { return item(ip.p); }
+
+  friend class EST_TIterator< EST_TList<T>, IPointer, T >;
+  friend class EST_TRwIterator< EST_TList<T>, IPointer, T >;
+
+public:
+  typedef T Entry;
+  typedef EST_TIterator< EST_TList<T>, IPointer, T > Entries;
+  typedef EST_TRwIterator< EST_TList<T>, IPointer, T > RwEntries;
+
 };
 
 
-template<class T> 
-extern ostream& operator << (ostream &st, EST_TList< T > const &list);
-
 template<class T> 
 bool operator==(const EST_TList<T> &a, const EST_TList<T> &b)
 { 
@@ -267,4 +316,7 @@ void merge_sort_unique(EST_TList<T> &l, EST_TList<T> &m)
 				 EST_TList<T>::free_item); 
 }
 
+template<class T>
+const char *error_name(EST_TList<T> val) { (void)val; return "<<TLIST>>"; }
+
 #endif
