@@ -5,15 +5,19 @@
  /*                         Copyright (c) 1996                            */
  /*                        All Rights Reserved.                           */
  /*                                                                       */
- /*  Permission to use, copy, modify, distribute this software and its    */
- /*  documentation for research, educational and individual use only, is  */
- /*  hereby granted without fee, subject to the following conditions:     */
+ /*  Permission is hereby granted, free of charge, to use and distribute  */
+ /*  this software and its documentation without restriction, including   */
+ /*  without limitation the rights to use, copy, modify, merge, publish,  */
+ /*  distribute, sublicense, and/or sell copies of this work, and to      */
+ /*  permit persons to whom this work is furnished to do so, subject to   */
+ /*  the following conditions:                                            */
  /*   1. The code must retain the above copyright notice, this list of    */
  /*      conditions and the following disclaimer.                         */
  /*   2. Any modifications must be clearly marked as such.                */
  /*   3. Original authors' names are not deleted.                         */
- /*  This software may not be used for commercial purposes without        */
- /*  specific prior written permission from the authors.                  */
+ /*   4. The authors' names are not used to endorse or promote products   */
+ /*      derived from this software without specific prior written        */
+ /*      permission.                                                      */
  /*                                                                       */
  /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
  /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -37,14 +41,17 @@
 #ifndef __TMatrix_H__
 #define __TMatrix_H__
 
-#include <iostream.h>
+#include <iostream>
+
+using namespace std;
+
 #include "EST_rw_status.h"
 #include "EST_TVector.h"
 #include "instantiate/EST_TMatrixI.h"
 
 /* When set bounds checks (safe but slow) are done on matrix access */
 #ifndef TMATRIX_BOUNDS_CHECKING
-#    define TMATRIX_BOUNDS_CHECKING 1
+#    define TMATRIX_BOUNDS_CHECKING 0
 #endif
 
 #if TMATRIX_BOUNDS_CHECKING
@@ -53,6 +60,25 @@
 #define A_CHECK a_no_check
 #endif
 
+#define INLINE inline 
+
+/* This doesn't work as I thought so I have disabled it for now.
+ */
+
+#if defined(__GNUC__) && 0
+#    define mx_move_pointer(P, TY, STEP, N) \
+            ((TY *)\
+	     ((void *) (((char (*) [sizeof(TY)*STEP])P) + N) ) \
+	      )
+#    define fast_a_m_gcc(R,C) \
+	( * mx_move_pointer(mx_move_pointer(p_memory,T,p_column_step,C),T,p_row_step,R))
+#    define fast_a_m_x(R,C) (fast_a_m_gcc(R,C))
+#else
+#    define fast_a_m_x(R,C) (fast_a_m(R,C))
+#endif
+
+
+
 /** Template Matrix class.
   *
   * This is an extension of the EST_TVector class to two dimensions.
@@ -61,8 +87,6 @@
   * @see EST_TVector
   */
 
-#define INLINE inline 
-
 template <class T> 
 class EST_TMatrix : public EST_TVector<T>
 {
@@ -75,17 +99,15 @@ protected:
   unsigned int p_row_step;
 
   INLINE unsigned int mcell_pos(int r, int c,
-			       int o,
 			       int rs, int cs) const
-    { return o + (rs==1?r:(r*rs)) + (cs==1?c:(c*cs));}
+    { return (rs==1?r:(r*rs)) + (cs==1?c:(c*cs));}
 
 
   INLINE unsigned int mcell_pos(int r, int c) const
     {
 
       return mcell_pos(r, c, 
-		       p_offset,
-		       p_row_step, p_column_step);
+		       this->p_row_step, this->p_column_step);
     }
 
   INLINE unsigned int mcell_pos_1(int r, int c) const
@@ -96,11 +118,15 @@ protected:
     }
 
   /// quick method for returning {\tt x[m][n]}
-  INLINE const T &fast_a(int r, int c) const { return p_memory[mcell_pos(r,c)]; }
-  INLINE T &fast_a(int r, int c) { return p_memory[mcell_pos(r,c)]; }
+  INLINE const T &fast_a_m(int r, int c) const 
+    { return this->p_memory[mcell_pos(r,c)]; }
+  INLINE T &fast_a_m(int r, int c) 
+    { return this->p_memory[mcell_pos(r,c)]; }
 
-  INLINE const T &fast_a_1(int r, int c) const { return p_memory[mcell_pos_1(r,c)]; }
-  INLINE T &fast_a_1(int r, int c) { return p_memory[mcell_pos_1(r,c)]; }
+  INLINE const T &fast_a_1(int r, int c) const 
+    { return this->p_memory[mcell_pos_1(r,c)]; }
+  INLINE T &fast_a_1(int r, int c) 
+    { return this->p_memory[mcell_pos_1(r,c)]; }
   
 
     /// Get and set values from array
@@ -150,14 +176,16 @@ public:
   //@{
 
   /// return number of rows
-  int num_rows() const {return p_num_rows;}
+  int num_rows() const {return this->p_num_rows;}
   /// return number of columns
-  int num_columns() const {return p_num_columns;}
+  int num_columns() const {return this->p_num_columns;}
 
   /// const access with no bounds check, care recommend
-  INLINE const T &a_no_check(int row, int col) const { return fast_a(row,col); }
+  INLINE const T &a_no_check(int row, int col) const 
+    { return fast_a_m_x(row,col); }
   /// access with no bounds check, care recommend
-  INLINE T &a_no_check(int row, int col) { return fast_a(row,col); }
+  INLINE T &a_no_check(int row, int col) 
+    { return fast_a_m_x(row,col); }
 
   INLINE const T &a_no_check_1(int row, int col) const { return fast_a_1(row,col); }
   INLINE T &a_no_check_1(int row, int col) { return fast_a_1(row,col); }
@@ -181,7 +209,7 @@ public:
   bool have_columns_before(int n) const;
 
   /** resize matrix. If {\tt set=1}, then the current values in
-      the matirx are preserved up to the new size {\tt n}. If the
+      the matrix are preserved up to the new size {\tt n}. If the
       new size exceeds the old size, the rest of the matrix is
       filled with the {\tt def_val}
   */
@@ -189,7 +217,7 @@ public:
 
   /// fill matrix with value v
   void fill(const T &v);
-  void fill() { fill(*def_val); }
+  void fill() { fill(*this->def_val); }
 
   /// assignment operator
   EST_TMatrix &operator=(const EST_TMatrix &s); 
@@ -200,7 +228,7 @@ public:
 
   /**@name Sub-Matrix/Vector Extraction
     *
-    * All of these return matrecies and vectors which share
+    * All of these return matrices and vectors which share
     * memory with the original, so altering values them alters
     * the original. 
     */
@@ -225,15 +253,30 @@ public:
         */
     void copy_row(int r, T *buf, int offset=0, int num=-1) const;
 
+    /** Copy row <parameter>r</parameter> of matrix to
+        <parameter>buf</parameter>. <parameter>buf</parameter> should be
+        pre-malloced to the correct size.  */
+    
+    void copy_row(int r, EST_TVector<T> &t, int offset=0, int num=-1) const;
+
     /** Copy column {\tt c} of matrix to {\tt buf}. {\tt buf}
         should be pre-malloced to the correct size.
         */
     void copy_column(int c, T *buf, int offset=0, int num=-1) const;
 
+    /** Copy column <parameter>c</parameter> of matrix to
+        <parameter>buf</parameter>. <parameter>buf</parameter> should
+        be pre-malloced to the correct size.  */
+
+    void copy_column(int c,  EST_TVector<T> &t, int offset=0, int num=-1)const;
+
     /** Copy buf into row {\tt n} of matrix. 
         */
     void set_row(int n, const T *buf, int offset=0, int num=-1);
 
+    void set_row(int n, const EST_TVector<T> &t, int offset=0, int num=-1)
+      { set_row(n, t.memory(), offset, num); }
+
     void set_row(int r, 
                  const EST_TMatrix<T> &from, int from_r, int from_offset=0,
                  int offset=0, int num=-1); // set nth row
@@ -243,14 +286,15 @@ public:
       */
     void set_column(int n, const T *buf, int offset=0, int num=-1);
 
+    void set_column(int n, const EST_TVector<T> &t, int offset=0, int num=-1)
+      { set_column(n, t.memory(), offset, num); }
+    
     void set_column(int c, 
                     const EST_TMatrix<T> &from, int from_c, int from_offset=0, 
                     int offset=0, int num=-1); // set nth column
 
   /** For when you absolutely have to have access to the memory.
     */
-  const T * memory() const { return p_memory+p_offset; }
-  T * memory(){ return p_memory+p_offset; }
   void set_memory(T *buffer, int offset, int rows, int columns, 
 		  int free_when_destroyed=0);
 
@@ -265,16 +309,21 @@ public:
   /// save Matrix to file {\tt filename}
   EST_write_status save(const class EST_String &filename) const;
 
-  /// print matirx.
-  friend ostream& operator << (ostream &st,const EST_TMatrix<T> &a);
+  /// print matrix.
+  friend ostream& operator << (ostream &st,const EST_TMatrix<T> &a)
+    {int i, j; 
+        for (i = 0; i < a.num_rows(); ++i) {
+            for (j = 0; j < a.num_columns(); ++j) 
+                st << a.a_no_check(i, j) << " "; 
+            st << endl;
+        }
+        return st;
+    }
   //@}
   
 };
 
 #undef A_CHECK
 
-template <class T> 
-extern ostream& operator << (ostream &st,const EST_TMatrix<T> &a);
-
 #endif
 
