@@ -5,15 +5,19 @@
  /*                         Copyright (c) 1996                            */
  /*                        All Rights Reserved.                           */
  /*                                                                       */
- /*  Permission to use, copy, modify, distribute this software and its    */
- /*  documentation for research, educational and individual use only, is  */
- /*  hereby granted without fee, subject to the following conditions:     */
+ /*  Permission is hereby granted, free of charge, to use and distribute  */
+ /*  this software and its documentation without restriction, including   */
+ /*  without limitation the rights to use, copy, modify, merge, publish,  */
+ /*  distribute, sublicense, and/or sell copies of this work, and to      */
+ /*  permit persons to whom this work is furnished to do so, subject to   */
+ /*  the following conditions:                                            */
  /*   1. The code must retain the above copyright notice, this list of    */
  /*      conditions and the following disclaimer.                         */
  /*   2. Any modifications must be clearly marked as such.                */
  /*   3. Original authors' names are not deleted.                         */
- /*  This software may not be used for commercial purposes without        */
- /*  specific prior written permission from the authors.                  */
+ /*   4. The authors' names are not used to endorse or promote products   */
+ /*      derived from this software without specific prior written        */
+ /*      permission.                                                      */
  /*                                                                       */
  /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
  /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -37,30 +41,29 @@
 #ifndef __Wave_H__
 #define __Wave_H__
 
-#include <stdio.h>
-#include "EST_String.h"
-#include "EST_Token.h"
+#include <cstdio>
+#include "EST_Featured.h"
 #include "EST_rw_status.h"
 #include "EST_types.h"
 
+class EST_Track;
+class EST_String;
+class EST_TokenStream;
+
 
 /** A class for storing digital waveforms. The waveform is stored as
-an arraay of 16 bit shorts. Mutliple channels are supported, but if no
+an array of 16 bit shorts. Multiple channels are supported, but if no
 channel information is given the 0th channel is accessed.
 <p>
 
 The waveforms can be of any sample rate, and can be changed to another
-samping rate using the <tt>resample</tt> function.
+sampling rate using the <tt>resample</tt> function.
 
 */
 
-class EST_Wave 
+class EST_Wave : public EST_Featured
 {
 protected:
-
-  EST_String p_name;
-  EST_String p_file_type;  // original type if read from file
-
   EST_SMatrix p_values;
 
   int p_sample_rate;
@@ -69,6 +72,7 @@ protected:
   void free_wave();
   void copy_data(const EST_Wave &w);
   void copy_setup(const EST_Wave &w);
+
 public:
 
   static const int default_sample_rate;
@@ -89,7 +93,7 @@ public:
   ~EST_Wave();
     
 
-  /**@name Access functions for finding ampltiudes of samples */
+  /**@name Access functions for finding amplitudes of samples */
   //@{
 
   /** return amplitude of sample <tt>i</tt> from channel <tt>
@@ -107,6 +111,12 @@ public:
   INLINE short a_no_check_1(int i, int channel = 0) const
         { return p_values.a_no_check_1(i,channel); }
 
+  
+  /** explicit set_a, easier to wrap than assignment
+   */
+  INLINE short set_a(int i, int channel = 0, short val = 0)
+  { return a(i,channel) = val; }
+
   /** return amplitude of sample <tt>i</tt> from channel <tt>
       channel</tt>.  By default the 0th channel is selected.
   */
@@ -135,6 +145,8 @@ public:
   int num_channels() const { return p_values.num_columns(); }
   /// return the sampling rate (frequency)
   int sample_rate() const { return p_sample_rate; }
+  /// Set sampling rate to <tt>n</tt>
+  void set_sample_rate(const int n){p_sample_rate = n;}
   /// return the size of the waveform, i.e. the number of samples.
   int length() const { return num_samples();}
   /// return the time position of the last sample.
@@ -148,18 +160,17 @@ public:
       was read. If the waveform has not been read from a file, this is set
       to the default type */
 
-  static const EST_String sample_type() { return "short"; }
-  const EST_String file_type() const { return p_file_type; }
-  void set_file_type(const EST_String &t) { p_file_type = t; }
+  EST_String sample_type() const { return f_String("sample_type","short"); }
+  void set_sample_type(const EST_String t) { f_set("sample_type", t); }
 
-  /// A string indentifying the waveform, commonly used to store the filename
-  EST_String name() const { return p_name; }
+  EST_String file_type() const { return f_String("file_type","riff"); }
+  void set_file_type(const EST_String t) { f_set("file_type", t); }
+
+  /// A string identifying the waveform, commonly used to store the filename
+  EST_String name() const { return f_String("name"); }
 
   /// Sets name.
-  void set_name(const EST_String n) {p_name = n;}
-
-  /// Set sampling rate to <tt>n</tt>
-  void set_sample_rate(const int n){p_sample_rate = n;}
+  void set_name(const EST_String n){ f_set("name", n); }
 
   //@}
 
@@ -176,12 +187,21 @@ public:
   /// Resample waveform to <tt>rate</tt>
   void resample(int rate);
 
+  /// Dynamic Range Compression - SaiKrishna May 2017
+  void compress(float mu, float limit);
+
   /** multiply all samples by a factor <tt>gain</tt>. This checks for
       overflows and puts them to the maximum positive or negative value
       as appropriate.
   */
   void rescale(float gain,int normalize=0);
 
+  // multiply samples by a factor contour.  The factor_contour track
+  // should contains factor targets at time points throughout the wave,
+  // between which linear interpolation is used to calculate the factor
+  // for each sample.
+  void rescale( const EST_Track &factor_contour );
+
   /// clear waveform and set size to 0.
   void clear() {resize(0,EST_ALL);}
 
@@ -298,8 +318,15 @@ public:
 
   /// print waveform
   friend ostream& operator << (ostream& p_values, const EST_Wave &sig);
+
+  // integrity check *** debug
+  void integrity() const { p_values.integrity() ; }
+
 };
 
 typedef EST_TList<EST_Wave> EST_WaveList;
 
+int operator != (EST_Wave a, EST_Wave b);
+int operator == (EST_Wave a, EST_Wave b);
+
 #endif /* __Wave_H__ */
