@@ -5,15 +5,19 @@
 /*                     Copyright (c) 1994,1995,1996                      */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -41,7 +45,10 @@
 extern "C" {
 #endif
 
-extern char *est_tools_version;
+extern const char * const est_tools_version;
+extern const char * const est_name;
+extern const char * const est_libdir;
+extern const char * const est_ostype;
 
 #include "EST_walloc.h"
 #include "EST_system.h"
@@ -52,8 +59,6 @@ extern char *est_tools_version;
 
 char *cmake_tmp_filename();
 
-long_long watoll(const char *str);
-
 /* NOTE perqs (from Three Rivers) have the third byte order, are not  */
 /* supported, if you find a working one let me know and I'll add      */
 /* support -- awb (hoping no one responds :-)                         */
