@@ -6,15 +6,19 @@
  /*                       Copyright (c) 1996,1997                        */
  /*                        All Rights Reserved.                          */
  /*                                                                      */
- /*  Permission to use, copy, modify, distribute this software and its   */
- /*  documentation for research, educational and individual use only, is */
- /*  hereby granted without fee, subject to the following conditions:    */
+ /*  Permission is hereby granted, free of charge, to use and distribute */
+ /*  this software and its documentation without restriction, including  */
+ /*  without limitation the rights to use, copy, modify, merge, publish, */
+ /*  distribute, sublicense, and/or sell copies of this work, and to     */
+ /*  permit persons to whom this work is furnished to do so, subject to  */
+ /*  the following conditions:                                           */
  /*   1. The code must retain the above copyright notice, this list of   */
  /*      conditions and the following disclaimer.                        */
  /*   2. Any modifications must be clearly marked as such.               */
  /*   3. Original authors' names are not deleted.                        */
- /*  This software may not be used for commercial purposes without       */
- /*  specific prior written permission from the authors.                 */
+ /*   4. The authors' names are not used to endorse or promote products  */
+ /*      derived from this software without specific prior written       */
+ /*      permission.                                                     */
  /*                                                                      */
  /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK       */
  /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING     */
@@ -43,14 +47,13 @@
 
 #if defined(__EMX__)
     /* For OS/2 */
-#   include <iostream.h>
-#   include <strstrea.h>
+#   include <iostream>
+using namespace std;
 #elif defined(SYSTEM_IS_UNIX)
-#   include <iostream.h>
-#   include <strstream.h>
+#   include <iostream>
+using namespace std;
 #elif defined(SYSTEM_IS_WIN32)
 #   include "win32/EST_iostream_win32.h"
-#   include <strstrea.h>
 #else
 #   error No System Selected
 #endif
