@@ -5,15 +5,19 @@
 /*                         Copyright (c) 1996                            */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -38,14 +42,13 @@
 
 
 #include "ling_class/EST_Utterance.h"
+#include "ling_class/EST_utterance_aux.h"
+
 #include "ling_class/EST_Relation.h"
-#include "ling_class/EST_Item.h"
 #include "ling_class/EST_relation_aux.h"
 #include "ling_class/EST_relation_compare.h"
 
-#include "ling_class/EST_Stream.h"
-#include "ling_class/EST_stream_aux.h"
-#include "ling_class/EST_stream_compare.h"
-
+#include "ling_class/EST_Item.h"
+#include "ling_class/EST_item_aux.h"
 
 #endif /* __LING_CLASS_H__ */
