@@ -5,15 +5,19 @@
 /*                         Copyright (c) 1996                            */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -37,11 +41,14 @@
 #define __EST_SIMPLESTATS_H__
 
 #include "EST_String.h"
+#include "EST_Token.h"
 #include "EST_StringTrie.h"
 #include "EST_TList.h"
-#include "EST_KV.h"
+#include "EST_TKVL.h"
 #include "EST_types.h"
 
+typedef size_t int_iter; 
+
 /** A class for managing mapping string names to integers and back again,
     mainly used for representing alphabets in n-grams and grammars etc.
 
@@ -95,7 +102,7 @@ public:
     bool operator == (const EST_Discrete &d);
     bool operator != (const EST_Discrete &d);
 
-    void print(FILE *fd,int with_brackets=0);
+    EST_String print_to_string(int quote=0);
     friend ostream& operator <<(ostream& s, const EST_Discrete &d);
 
     ///
@@ -128,39 +135,47 @@ class Discretes {
  */
 class EST_SuffStats {
 private:
-    int n;
+    double n;  // allows frequencies to be non-integers
     double p_sum;
     double p_sumx;
 public:
     ///
-    EST_SuffStats() {n=0; p_sum = p_sumx = 0.0;}
+    EST_SuffStats() {n = p_sum = p_sumx = 0.0;}
+    ///
+    EST_SuffStats(double in, double isum, double isumx) 
+	{n = in; p_sum = isum; p_sumx = isumx;}
     ///
     EST_SuffStats(const EST_SuffStats &s) { copy(s); }
     ///
     void copy(const EST_SuffStats &s) 
        {n=s.n; p_sum = s.p_sum; p_sumx = s.p_sumx;}
     /// reset internal values
-    void reset(void) {n=0; p_sum = p_sumx = 0.0;}
+    void reset(void) {n = p_sum = p_sumx = 0.0;}
+    void set(double in, double isum, double isumx) 
+	{n = in; p_sum = isum; p_sumx = isumx;}
     /// number of samples in set
-    int samples(void) {return n;}
+    double samples(void) {return n;}
     /// sum of values
     double sum() { return p_sum; }
     /// sum of squared values 
     double sumx() { return p_sumx; }
     /// mean of currently cummulated values
-    double mean(void) const { return p_sum / n; }
+    double mean(void) const { return (n==0)?0.0:(p_sum / n); }
     /// variance of currently cummulated values 
     double variance(void) const 
        { return ((n*p_sumx)-(p_sum*p_sum))/((double)n*(n-1)); }
     /// standard deviation of currently cummulated values
     double stddev(void) const { return sqrt(variance()); }
 
+    void cumulate(double a,double count=1.0)
+	{ n+=count; p_sum+=a*count; p_sumx+=count*(a*a); }
+
     /// Used to cummulate new values
     EST_SuffStats &operator +=(double a) 
-         { n++;p_sum+=a; p_sumx+=a*a; return *this;}
+         { cumulate(a,1.0); return *this;}
     /// Used to cummulate new values
     EST_SuffStats &operator + (double a) 
-         { n++;p_sum+=a; p_sumx+=a*a; return *this;}
+         { cumulate(a,1.0); return *this;}
     /// 
     EST_SuffStats &operator = (const EST_SuffStats &a) 
          { copy(a); return *this;}
@@ -173,7 +188,7 @@ enum EST_tprob_type {tprob_string, tprob_int, tprob_discrete};
     This may be used to cummulate the probability distribution of a 
     class of values.  Values are actually help as frequencies so both
     frequency and probability information may be available.   Note that
-    frequencies are not integers because usings smoothing and backoff
+    frequencies are not integers because using smoothing and backoff
     integers are too restrictive so they are actually represented as
     doubles.
 
@@ -227,16 +242,17 @@ public:
     void clear(void);
     /// Initialise using given vocabulary
     bool init(const EST_StrList &vocab);
-    /// Initialise using given \Ref{EST_Distcrete} as vocabulary
+    /// Initialise using given \Ref{EST_Discrete} as vocabulary
     void init(const EST_Discrete *d);
     /// Initialise
     void init();
     /// Total number of example found.
     double samples(void) const { return num_samples; }
-    /// Add this observation, may specify number of occurences
+    /// Add this observation, may specify number of occurrences
     void cumulate(const EST_String &s,double count=1);
     /// Add this observation, i must be with in EST\_Discrete range
-    void cumulate(const int i,double count=1);
+    void cumulate(EST_Litem *i,double count=1);
+    void cumulate(int i,double count=1);
     /// Return the most probable member of the distribution
     const EST_String &most_probable(double *prob = NULL) const;
     /** Return the entropy of the distribution
@@ -252,34 +268,37 @@ public:
     /// 
     double frequency(const int i) const; 
     /// Used for iterating through members of the distribution
-    int item_start() const;
+    EST_Litem *item_start() const;
     /// Used for iterating through members of the distribution
-    int item_next(int idx) const;
+    EST_Litem *item_next(EST_Litem *idx) const;
     /// Used for iterating through members of the distribution
-    int item_end(int idx) const;
-    /// During iteration ruturns name given index 
-    const EST_String &item_name(int idx) const;
-    /// During iteration ruturns name and frequency given index  
-    void item_freq(int idx,EST_String &s,double &freq) const;
-    /// During iteration ruturns name and probability given index
-    void item_prob(int idx,EST_String &s,double &prob) const;
+    int item_end(EST_Litem *idx) const;
 
-    /// Returns discrete vocabilary of distribution
+    /// During iteration returns name given index 
+    const EST_String &item_name(EST_Litem *idx) const;
+    /// During iteration returns name and frequency given index  
+    void item_freq(EST_Litem *idx,EST_String &s,double &freq) const;
+    /// During iteration returns name and probability given index
+    void item_prob(EST_Litem *idx,EST_String &s,double &prob) const;
+
+    /// Returns discrete vocabulary of distribution
     inline const EST_Discrete *const get_discrete() const { return discrete; };
     
-    /** Sets the frequency of named item, modifes {\tt num\_samples}
+    /** Sets the frequency of named item, modifies {\tt num\_samples}
          accordingly.  This is used when smoothing frequencies.
     */
     void set_frequency(const EST_String &s,double c);
-    /** Sets the frequency of named item, modifes {\tt num\_samples}
+    /** Sets the frequency of named item, modifies {\tt num\_samples}
         accordingly.  This is used when smoothing frequencies.
     */
     void set_frequency(int i,double c); 
+    void set_frequency(EST_Litem *i,double c); 
     
     /// Sets the frequency of named item, without modifying {\tt num\_samples}.
     void override_frequency(const EST_String &s,double c);
     /// Sets the frequency of named item, without modifying {\tt num\_samples}.
     void override_frequency(int i,double c); 
+    void override_frequency(EST_Litem *i,double c); 
     
     /** Sets the number of samples.  Care should be taken on setting this
         as it will affect how probabilities are calculated.
