@@ -5,15 +5,19 @@
 /*                      Copyright (c) 1995,1996                          */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -39,8 +43,11 @@
 #include "EST_TList.h"
 #include "EST_TVector.h"
 #include "EST_String.h"
-#include "EST_KV.h"
+#include "EST_TKVL.h"
 #include "EST_FMatrix.h"
+#include "EST_DMatrix.h"
+#include "EST_IMatrix.h"
+#include "EST_SMatrix.h"
 
 typedef EST_TVector<EST_String> EST_StrVector;
 
@@ -55,7 +62,7 @@ typedef EST_TList<int> EST_IList;
 typedef EST_TList<float> EST_FList;
 typedef EST_TList<double> EST_DList;
 
-typedef KVL<int, int> EST_II_KVL;
+typedef EST_TKVL<int, int> EST_II_KVL;
 
 typedef EST_TList<EST_TList<int> > EST_IListList;
 
@@ -65,10 +72,10 @@ typedef EST_TList<float> EST_FList;
 
 typedef EST_TList<EST_TList<EST_String> > EST_StrListList;
 typedef EST_TVector<EST_StrList> EST_StrListVector;
-typedef KVL<EST_String, EST_String> EST_StrStr_KVL;
-typedef KVL<EST_String, int> EST_StrI_KVL;
-typedef KVL<EST_String, float> EST_StrF_KVL;
-typedef KVL<EST_String, double> EST_StrD_KVL;
-//typedef KVL<EST_String, EST_Val> EST_StrVal_KVL;
+typedef EST_TKVL<EST_String, EST_String> EST_StrStr_KVL;
+typedef EST_TKVL<EST_String, int> EST_StrI_KVL;
+typedef EST_TKVL<EST_String, float> EST_StrF_KVL;
+typedef EST_TKVL<EST_String, double> EST_StrD_KVL;
+//typedef EST_TKVL<EST_String, EST_Val> EST_StrVal_KVL;
 
 #endif // __EST_TYPES_H__
