@@ -60,9 +60,9 @@ class ling_class_init;
 /** A class for containing individual linguistic objects such as
 words or phones.
 
-These contain two types of infomation. This first is specific to the
+These contain two types of information. This first is specific to the
 \Ref{EST_Relation} we are viewing this ling item from, the second part
-constists of a set of features.  These features may be shared by
+consists of a set of features.  These features may be shared by
 instances of this ling item in different <link
 linkend="est-relation">EST_Relation</link> within the same <link
 linkend="est-utterance">EST_Utterances</link>
@@ -111,7 +111,7 @@ protected:
     //@{
     /// Default constructor
     EST_Item();
-    /// Copy constructure only makes reference to contents 
+    /// Copy constructor only makes reference to contents 
     EST_Item(const EST_Item &item);
     /// Includes reference to relation 
     EST_Item(EST_Relation *rel);
@@ -125,7 +125,7 @@ protected:
     /**@name Feature access functions. 
        These functions are wrap-around functions to the basic access
        functions in the \Ref{EST_Features} class.  In all these
-       functioms, if the optional argument <parameter>m} is set to 1, an
+       functions, if the optional argument <parameter>m} is set to 1, an
        error is thrown if the feature does not exist*/
 
     //@{
@@ -172,7 +172,7 @@ protected:
     //@}
 
     /**@name Feature setting functions.
-       A separate function is provided for each permissable value type
+       A separate function is provided for each permissible value type
     */
     //@{
     /** set feature <parameter>name</parameter> to <parameter>val</parameter> */
@@ -228,25 +228,29 @@ protected:
 
     /** TRUE if feature is present, FALSE otherwise */
     int f_present(const EST_String &name) const
-       {return features().present(name); }
+    {
+        if (p_contents)
+            return features().present(name);
+        else
+            return FALSE; }
 
     // Number of items (including this) until no next item.
     int length() const;
     //@}
 
     // get contents from item
-    EST_Item_Content *contents() const { return (this == 0) ? 0 : p_contents;}
+    EST_Item_Content *contents() const { return p_contents;}
     // used by tree manipulation functions
     void set_contents(EST_Item_Content *li);
 
     // These should be deleted.
     // The item's name 
     const EST_String name() const
-    { return (this == 0) ? EST_String::Empty : f("name",0).string(); }
+    { return f("name",0).string(); }
 
     // Set item's name
     void set_name(const EST_String &name) const
-    { if (this != 0) p_contents->set_name(name); }
+    { p_contents->set_name(name); }
 
     // Shouldn't normally be needed, except for iteration
     EST_Features &features() const { return p_contents->f; }
@@ -254,16 +258,11 @@ protected:
     const EST_Val f(const EST_String &name) const
     { 
 	EST_Val v;
-	if (this == 0) 
-	    EST_error("item is null so has no %s feature",(const char *)name);
-        else
-        {
-	    for (v=p_contents->f.val_path(name);
-		 v.type() == val_type_featfunc && featfunc(v) != NULL;
-		 v=(featfunc(v))((EST_Item *)(void *)this));
-	    if (v.type() == val_type_featfunc)
-		EST_error("NULL %s function",(const char *)name);
-	}
+        for (v=p_contents->f.val_path(name);
+             v.type() == val_type_featfunc && featfunc(v) != NULL;
+             v=(featfunc(v))((EST_Item *)(void *)this));
+        if (v.type() == val_type_featfunc)
+            EST_error("NULL %s function",(const char *)name);
 	return v;
     }
 
@@ -287,18 +286,13 @@ protected:
 
     const EST_Val f(const EST_String &name, const EST_Val &def) const
     { 
-	if (this == 0) 
-	    return def;
-        else
-        {
-	    EST_Val v; 
-	    for (v=p_contents->f.val_path(name, def);
-		 v.type() == val_type_featfunc && featfunc(v) != NULL;
-		 v=(featfunc(v))((EST_Item *)(void *)this));
-	    if (v.type() == val_type_featfunc)
-		v = def;
-	    return v;
-	}
+        EST_Val v; 
+        for (v=p_contents->f.val_path(name, def);
+             v.type() == val_type_featfunc && featfunc(v) != NULL;
+             v=(featfunc(v))((EST_Item *)(void *)this));
+        if (v.type() == val_type_featfunc)
+            v = def;
+        return v;
     }
 
     /**@name Cross relational access */
@@ -306,11 +300,11 @@ protected:
 
     /// View item from another relation (const char *) method
     EST_Item *as_relation(const char *relname) const
-    { return (this == 0) ? 0 : p_contents->Relation(relname); }
+    { return p_contents->Relation(relname); }
 
     /// TRUE if this item is in named relation
     int in_relation(const EST_String &relname) const
-    { return (this == 0) ? 0 : p_contents->in_relation(relname); }
+    { return p_contents->in_relation(relname); }
 
     /// Access to the relation links
     EST_TKVL<EST_String, EST_Val> &relations() {return p_contents->relations;}
@@ -323,8 +317,8 @@ protected:
        { return (this == 0) ? 0 : p_relation; }
 
     /// True if li is the same item ignoring its relation viewpoint
-    int same_item(const EST_Item *li) const
-    { return contents() && li->contents() && (contents() == li->contents()); }
+      int same_item(const EST_Item *li) const
+      { return contents() && li->contents() && (contents() == li->contents()); }
     //@}
 
     // The remaining functions should not be accessed, they are should be
@@ -335,32 +329,14 @@ protected:
     static void splice(EST_Item *a, EST_Item *b)
 	{ if(a !=NULL) a->n = b; if (b != NULL) b->p=a; }
 
-    // Internal traversal - nnot recommended - use relation traversal functions
-    //
-    EST_Item *next() const { return this == 0 ? 0 : n; }
-    //
-    EST_Item *prev() const { return this == 0 ? 0 : p; }
-    //
-    EST_Item *down() const { return this == 0 ? 0 : d; }
-    //
-    EST_Item *up() const   { return this == 0 ? 0 : u; }
-    // Last item (most next) at this level
-    EST_Item *last() const;
-    // First item (most prev) at this level
-    EST_Item *first() const;
-    // Highest (most up)
-    EST_Item *top() const;
-    // Lowest (most down)
-    EST_Item *bottom() const;
-    // First item which has no down, within the descendants of this item
-    EST_Item *first_leaf() const;
-    // Next item which has no down, following above this item if necessary
-    EST_Item *next_leaf() const;
-    // Last item which has no down, following above this item if necessary
-    EST_Item *last_leaf() const;
-    // Next item in pre order (root, daughters, siblings)
-    EST_Item *next_item() const;
-
+    // Internal traversal - not recommended - use relation traversal functions
+    // As these functions must be safe to NULL arguments they now (gcc6)
+    // cannot refer to this, so alway require an explicit pointer to the object
+    // These four are the only ones that require access to private fields
+    friend EST_Item *inext(const EST_Item *i);
+    friend EST_Item *iprev(const EST_Item *i);
+    friend EST_Item *idown(const EST_Item *i);
+    friend EST_Item *iup(const EST_Item *i);
 
     // Insert a new item after this, with li's contents
     EST_Item *insert_after(EST_Item *li=0);
@@ -384,9 +360,7 @@ protected:
     // Verification, double links are consistent (used after reading in)
     int verify() const;
 	
-    friend int same_item(const EST_Item *l1,const EST_Item *l2)
-    { return l1->contents() && l2->contents() && 
-	  (l1->contents() == l2->contents()); }
+    friend int i_same_item(const EST_Item *l1,const EST_Item *l2);
     friend int move_item(EST_Item *from, EST_Item *to);
     friend int merge_item(EST_Item *from, EST_Item *to);
     friend int move_sub_tree(EST_Item *from, EST_Item *to);
@@ -395,25 +369,45 @@ protected:
     EST_Item &operator=(const EST_Item &s);
     friend ostream& operator << (ostream &s, const EST_Item &a);
     friend bool operator !=(const EST_Item &a, const EST_Item &b)
-    { return !::same_item(&a,&b); }
+    { return !i_same_item(&a,&b); }
     friend bool operator ==(const EST_Item &a, const EST_Item &b)
-    { return ::same_item(&a,&b); }
+    { return i_same_item(&a,&b); }
 
     friend class EST_Relation;
     friend class ling_class_init;
 };
 
+inline int i_same_item(const EST_Item *l1,const EST_Item *l2)
+{
+  return l1->contents() && l2->contents() && 
+    (l1->contents() == l2->contents()); 
+}
+
+
 inline EST_Item *as(const EST_Item *n,const char *relname)
-     { return n->as_relation(relname); }
+{  if (n != 0) return n->as_relation(relname);
+    return 0;}
+
+EST_Item *inext(const EST_Item *i);
+EST_Item *iprev(const EST_Item *i);
+EST_Item *idown(const EST_Item *i);
+EST_Item *iup(const EST_Item *i);
+
+EST_Item *last(const EST_Item *x);
+EST_Item *first(const EST_Item *x);
+EST_Item *top(const EST_Item *x);
+EST_Item *bottom(const EST_Item *x);
+
+EST_Item *next_item(const EST_Item *node);
+EST_Item *first_leaf(const EST_Item *node);
+EST_Item *next_leaf(const EST_Item *node);
+EST_Item *last_leaf(const EST_Item *node);
 
 // Relation structure functions
 #include "ling_class/EST_Relation_list.h"
 #include "ling_class/EST_Relation_tree.h"
 #include "ling_class/EST_Relation_mls.h"
 
-inline EST_Item *next_item(const EST_Item *node) 
-    { return node->next_item(); }
-
 void remove_item(EST_Item *l, const char *relname);
 
 void copy_node_tree(EST_Item *from, EST_Item *to);
@@ -421,7 +415,6 @@ void copy_node_tree_contents(EST_Item *from, EST_Item *to);
 
 void evaluate(EST_Item *a,EST_Features &f);
 
-
 #include "ling_class/EST_FeatureFunctionPackage.h"
 
 // Feature function support
