@@ -103,7 +103,7 @@ class EST_Relation
     EST_Features f;
 
     /** Evaluate the relation's feature functions */
-    void evaluate_features();
+    //void evaluate_features();
     /** Evaluate the feature functions of all the items in the relation */
     void evaluate_item_features();
 
@@ -132,9 +132,7 @@ class EST_Relation
 
     // This have been replaced by Relation_Tree functions
     EST_Item *first() const { return head(); }
-    EST_Item *first_leaf() const;
-    EST_Item *last() const { return tail(); }
-    EST_Item *last_leaf() const;
+    EST_Item *rlast() const { return tail(); }
 
     /** Return the tail (last) item of the relation */
 //    EST_Item *id(int i);
@@ -149,7 +147,7 @@ class EST_Relation
     /** remove EST_Item <parameter>item</parameter> from relation */
     void remove_item(EST_Item *item);
 
-    /** remove all occurances of feature 
+    /** remove all occurrences of feature 
 	<parameter>name</parameter> from relation's items
     */
     void remove_item_feature(const EST_String &name);
@@ -168,9 +166,9 @@ class EST_Relation
 			 );
 
     /** Load relation from already open tokenstream */
-    EST_read_status EST_Relation::load(const EST_String &filename,
-				   EST_TokenStream &ts,
-				   const EST_String &type);
+    EST_read_status load(const EST_String &filename,
+			 EST_TokenStream &ts,
+			 const EST_String &type);
 
     /** Save relation to file */
     EST_write_status save(const EST_String &filename, 
