@@ -5,15 +5,19 @@
 /*                         Copyright (c) 1996                            */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, and modify this software and its            */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -33,32 +37,27 @@
 /*                                                                       */
 /*=======================================================================*/
 
-#include <stdlib.h>
+#include <cstdlib>
 #include "EST_math.h"
 #include "EST_tilt.h"
 #include "tilt.h"
 #include "EST_Track.h"
 #include "EST_track_aux.h"
-#include "EST_Option.h"
+#include "EST_Features.h"
+#include "EST_error.h"
 
 static int match_rf_point(EST_Track &fz, int b_start, int b_stop, 
 			  int e_start, int e_stop, 
 			  int &mi, int &mj);
 
-void print_events(EST_Relation &ev);
-void print_rfc2_events(EST_Relation &ev);
-
-float unit_curve(EST_String type, float amp, float dur, float t);
-float fncurve(float length, float t, float curve);
-
 static void make_int_item(EST_Item &e, const EST_String name, float end, 
 				     float start_pos,
 				     float start_f0, 
 				     float peak_pos, 
 				     float peak_f0);
 
-void rfc_segment_features_only(EST_Relation &ev);
-void convert_to_event(EST_Relation &ev);
+//void rfc_segment_features_only(EST_Relation &ev);
+//void convert_to_event(EST_Relation &ev);
 
 static int rf_match(EST_Track &fz, EST_Item &ev, float range);
 
@@ -66,15 +65,16 @@ static int zero_cross(EST_Track &fz);
 
 static int comp_extract(EST_Track &fz, EST_Track &part, float &start, float
 			&end, float min);
-void segment_to_event(EST_Relation &ev);
-void event_to_segment(EST_Relation &ev, float min_length = 0.01);
+//void segment_to_event(EST_Relation &ev);
+//void event_to_segment(EST_Relation &ev, float min_length = 0.01);
 
 void int_segment_to_unit(EST_Relation &a, EST_Relation &ev);
 
-void convert_to_local(EST_Relation &ev);
+static void convert_to_local(EST_Relation &ev);
+
+static void silence_f0(EST_Relation &ev, EST_Track &fz);
 
 static void add_phrases(EST_Relation &ev);
-static void silence_f0(EST_Relation &ev, EST_Track &fz);
 
 
 // find event portions of fz in contour, cut out, and send one by one
@@ -101,51 +101,58 @@ static void silence_f0(EST_Relation &ev, EST_Track &fz);
 // Now add connections between non-overlapping events. Overlapping events
 // get readjusted to make them simply adjacent
 
-
-
-
-void default_rfc_params(EST_Option &op)
+void default_rfc_params(EST_Features &op)
 {
-    op.override_fval("rfc_start_range", 0.1);
-    op.override_fval("rfc_stop_range", 0.1);
-    op.override_fval("rfc_search_range", 0.3);
-    op.override_fval("rfc_min_connection", 0.01);
-    op.override_fval("rfc_min_event", 0.03);
+    op.set("start_limit", 0.1);
+    op.set("stop_limit", 0.1);
+    op.set("range", 0.3);
+    op.set("min_event_duration", 0.03);
 }
 void print_event(EST_Item &ev);
 
-void rfc_analysis(EST_Track &fz, EST_Relation &ev, EST_Option &op)
+void rfc_analysis(EST_Track &fz, EST_Relation &ev, EST_Features &op)
 {
     EST_Item *e, *tmp, *n;
     float start_search, end_search;
     EST_Track part;
     EST_Relation eva;
 
-    cout << "rfc_recognise\n";
-    cout << ev;
+    if (op.present("debug"))
+    {
+	cout << "rfc_recognise\n";
+	cout << ev;
+    }
 
     int_segment_to_unit(ev, eva);
 
-    cout << "rfc_recognise\n";
-    cout << ev;
+    if (op.present("debug"))
+    {
+	cout << "rfc_recognise\n";
+	cout << ev;
+    }
 
     // fill values in event labels using matching algorithms
     for (e = ev.head(); e != 0; e = n)
     {
-	n = next(e);
-	cout << endl << endl;
+	n = inext(e);
+	// cout << endl << endl;
 	if (!event_item(*e))
 	    continue;
-	end_search = e->fF("end") + op.fval("rfc_stop_range");
-	start_search = e->fF("start") - op.fval("rfc_stop_range");
-	
-	cout << "start = " << e->fF("start") << " End " << e->fF("end")<< endl;
-	cout << "s start = " << start_search << " sEnd " << end_search << endl;
-	cout << *e << endl;;
+	end_search = e->F("end") + op.F("stop_limit");
+	start_search = e->F("start") - op.F("start_limit");
+
+	if (op.present("debug"))
+	{
+	    cout << "start = " << e->F("start") << " End " 
+		 << e->F("end")<< endl;
+	    cout << "s start = " << start_search << " sEnd " 
+		 << end_search << endl;
+	    cout << *e << endl;;
+	}
 	
 	if (comp_extract(fz, part, start_search, end_search, 
-			 op.fval("rfc_min_event")))
-	    rf_match(part, *e, op.fval("rfc_search_range"));
+			 op.F("min_event_duration")))
+	    rf_match(part, *e, op.F("range"));
 	else
 	    ev.remove_item(e);
     }
@@ -160,6 +167,8 @@ void rfc_analysis(EST_Track &fz, EST_Relation &ev, EST_Option &op)
 
     silence_f0(ev, fz);
     add_phrases(ev);
+
+    // cout << endl << endl << ev;
     convert_to_local(ev);
 
     // make sure events don't overlap
@@ -167,8 +176,11 @@ void rfc_analysis(EST_Track &fz, EST_Relation &ev, EST_Option &op)
 
     ev.f.set("intonation_style", "rfc");
 
-    cout << "Ater RFC analysis\n";
-    cout << ev;
+    if (op.present("debug"))
+    {
+	cout << "After RFC analysis\n";
+	cout << ev;
+    }
 }
 
 // convert intonation stream from segment type to event type description.
@@ -179,52 +191,70 @@ void rfc_analysis(EST_Track &fz, EST_Relation &ev, EST_Option &op)
 // section of contour bounded by the two times. If no fz contour exits within
 // the limits an error is returned.
 
-void convert_to_local(EST_Item *e)
+static void convert_to_local(EST_Item *e)
 {
-    if (e->f("rfc:type") == "RISEFALL")
+    if (e->S("rfc.type", "0") == "RISEFALL")
     {
-	e->fset("rfc:rise_amp", (e->fF("rfc:peak_f0") - e->fF("ev:start_f0")));
-	e->fset("rfc:rise_dur", (e->fF("rfc:peak_pos") - e->fF("start")));
-	e->fset("rfc:fall_amp", (e->fF("rfc:end_f0") - e->fF("rfc:peak_f0")));
-	e->fset("rfc:fall_dur", (e->fF("end") - e->fF("rfc:peak_pos")));
-	e->fset("ev:f0", e->fF("rfc:peak_f0") - e->fF("rfc:rise_amp"));
+	e->set("rfc.rise_amp", (e->F("rfc.peak_f0") - e->F("ev.start_f0")));
+	e->set("rfc.rise_dur", (e->F("rfc.peak_pos") - e->F("start")));
+	e->set("rfc.fall_amp", (e->F("rfc.end_f0") - e->F("rfc.peak_f0")));
+	e->set("rfc.fall_dur", (e->F("end") - e->F("rfc.peak_pos")));
+	e->set("ev.f0", e->F("rfc.peak_f0"));
+//	e->set("ev.f0", e->F("rfc.peak_f0") - e->F("rfc.rise_amp"));
+
+	e->A("rfc").remove("peak_f0");
+	e->A("rfc").remove("peak_pos");
+	e->A("rfc").remove("end_f0");
+	e->A("rfc").remove("type");
+	e->A("ev").remove("start_f0");
     }
-    else if (e->f("rfc:type") == "RISE")
+    else if (e->S("rfc.type", "0") == "RISE")
     {
-	e->fset("rfc:rise_amp", (e->fF("rfc:end_f0") - e->fF("ev:start_f0")));
-	e->fset("rfc:rise_dur", (e->fF("end") - e->fF("start")));
-	e->fset("rfc:fall_amp", 0.0);
-	e->fset("rfc:fall_dur", 0.0);
-	e->fset("ev:f0", e->fF("rfc:end_f0") - e->fF("rfc:rise_amp"));
+	e->set("rfc.rise_amp", (e->F("rfc.end_f0") - e->F("ev.start_f0")));
+	e->set("rfc.rise_dur", (e->F("end") - e->F("start")));
+	e->set("rfc.fall_amp", 0.0);
+	e->set("rfc.fall_dur", 0.0);
+	e->set("ev.f0", e->F("rfc.end_f0"));
+//	e->set("ev.f0", e->F("rfc.end_f0") - e->F("rfc.rise_amp"));
+	
+	e->A("rfc").remove("peak_f0");
+	e->A("rfc").remove("peak_pos");
+	e->A("rfc").remove("end_f0");
+	e->A("rfc").remove("type");
+	e->A("ev").remove("start_f0");
     }
-    else if (e->f("rfc:type") == "FALL")
+    else if (e->S("rfc.type", "0") == "FALL")
     {
-	e->fset("rfc:rise_amp", 0.0);
-	e->fset("rfc:rise_dur", 0.0);
-	e->fset("rfc:fall_amp", (e->fF("rfc:end_f0") - e->fF("ev:start_f0")));
-	e->fset("rfc:fall_dur", (e->fF("end") - e->fF("start")));
-	e->fset("ev:f0", e->fF("ev:start_f0"));
+	e->set("rfc.rise_amp", 0.0);
+	e->set("rfc.rise_dur", 0.0);
+	e->set("rfc.fall_amp", (e->F("rfc.end_f0") - e->F("ev.start_f0")));
+	e->set("rfc.fall_dur", (e->F("end") - e->F("start")));
+	e->set("ev.f0", e->F("ev.start_f0"));
+
+	e->A("rfc").remove("peak_f0");
+	e->A("rfc").remove("peak_pos");
+	e->A("rfc").remove("end_f0");
+	e->A("rfc").remove("type");
+	e->A("ev").remove("start_f0");
     }
-    if (!e->f_present("position"))
-	e->fset("position", (e->fF("end") - e->fF("rfc:fall_dur")));
+    if (!e->f_present("time"))
+	e->set("time", (e->F("end") - e->F("rfc.fall_dur")));
 }
 
-
-
 void convert_to_local(EST_Relation &ev)
 {
     EST_Item *e;
 
-    for (e = ev.head(); e; e = next(e))
+    for (e = ev.head(); e; e = inext(e))
 	convert_to_local(e);
 
-    cout << "c to l \n\n\n" << ev << endl << endl;
+    // cout << "c to l \n\n\n" << ev << endl << endl;
 
-    ev.remove_item_feature("rfc:peak_f0");
-    ev.remove_item_feature("rfc:peak_pos");
-    ev.remove_item_feature("ev:start_f0");
+//    ev.remove_item_feature("rfc.peak_f0");
+//    ev.remove_item_feature("rfc.peak_pos");
+    ev.remove_item_feature("ev.start_f0");
     ev.remove_item_feature("start");
-    ev.remove_item_feature("rfc:end_f0");
+//    ev.remove_item_feature("rfc.end_f0");
     ev.remove_item_feature("end");
 //    remove_item_feature(ev, "int_event");
 
@@ -240,16 +270,20 @@ static int comp_extract(EST_Track &fz, EST_Track &part, float &start, float
     int continuous = 1;
     cout.precision(6);
     EST_Track tr_tmp, tmp2;
-    
-    if (start > end)
-    {
-	cerr << "Illegal start and end times\n";
-	return 0;
-    }
 
-    cout << "full f0 = " << fz.num_frames() << endl;
+    if (start > end)
+	EST_error("Illegal start and end times: %f %f\n", start, end);
+
+//    int from = fz.index(start);
+//    int to = fz.index_below(end);
+
+    // cout << "full f0 = " << fz.num_frames() << endl;
+//    fz.copy_sub_track(tr_tmp, from, to, 0, EST_ALL);
+
+//    cout << "sub_track: " << tr_tmp;
     
     extract2(fz, start, end, tr_tmp);
+    
 //    cout << "tr_tmp 1\n" << tr_tmp;
     tr_tmp.rm_trailing_breaks();
 //    cout << "tr_tmp 2\n" << tr_tmp;
@@ -276,14 +310,14 @@ static int comp_extract(EST_Track &fz, EST_Track &part, float &start, float
     // if no breaks are found in this section return. 
     if (continuous)
     {
-//	cout << "tr_tmp\n" << tr_tmp;
 	part = tr_tmp;
-//	cout << "Part\n" << part;
 	return 1;
     }
     
-    // Otherwise, find the longest single section
-    cout << "This contour has a break in it at point\n";
+    // tracks can legitimately have breaks in them due to the 
+    // search region overlapping a silence. In this case we find
+    // the longest single section
+    // cout << "tilt_analysis: This contour has a break in it\n";
     
     int longest, s_c, s_l;
     longest = s_c = s_l = 0;
@@ -307,7 +341,7 @@ static int comp_extract(EST_Track &fz, EST_Track &part, float &start, float
 	s_l = s_c;
     } 
     
-    //    cout << "Lonest fragment is " << longest << " starting at " << s_l <<endl;
+    //    cout << "Longest fragment is " << longest << " starting at " << s_l <<endl;
     //    cout << "Times: " << tr_tmp.t(s_l) << " : " <<tr_tmp.t(s_l + longest) << endl;
     
     extract2(tr_tmp, tr_tmp.t(s_l), tr_tmp.t(s_l + longest), part);
@@ -340,26 +374,28 @@ static int rf_match(EST_Track &fz, EST_Item &ev, float range)
     EST_Track diff;
     int start, stop;
     int b_start, b_stop, e_start, e_stop, region;
+    EST_Features empty;
     
     if (fz.num_frames() <= 0)
     {
-	ev.fset("ev:start_f0", 0.0);
-	ev.fset("start", 0.0);
-	ev.fset("rfc:peak_f0", 0.0);
-	ev.fset("rfc:peak_pos", 0.0);
+	ev.set("start", 0.0);
+	ev.set("ev", empty);
+	ev.set("rfc", empty);
+	ev.set("ev.start_f0", 0.0);
+	ev.set("rfc.peak_f0", 0.0);
+	ev.set("rfc.peak_pos", 0.0);
     }
     
     diff = differentiate(fz);
-    diff.set_contour_type(ct_diff_f0);
 
-    cout << "Extracted Contour:\n";
+    // cout << "Extracted Contour:\n";
     //    cout << fz;
     
     n = zero_cross(diff);
     
     if (n >= 0)			// rise + fall combination
     {
-	cout << "zero crossing at point " << n << " time " << fz.t(n) << endl;
+	// cout << "zero crossing at point " << n << " time " << fz.t(n) << endl;
 	b_start = 0;
 	stop = n;
 	// find rise part
@@ -373,13 +409,13 @@ static int rf_match(EST_Track &fz, EST_Item &ev, float range)
 	// ensure regions are separate
 	e_start = (e_start < b_stop)? b_stop : e_start;
 	
-	printf("rise: b_start  %d, b_stop %d, end %d, end stop%d\n", b_start,
-	       b_stop, e_start, e_stop);
+	//printf("rise: b_start  %d, b_stop %d, end %d, end stop%d\n", b_start,
+	//       b_stop, e_start, e_stop);
 	match_rf_point(fz, b_start, b_stop, e_start, e_stop, start, stop);
-	cout << "Rise is at start: " << start << " Stop = " << stop << endl;
+	// cout << "Rise is at start: " << start << " Stop = " << stop << endl;
 	
-	ev.fset("ev:start_f0", fz.a(start));
-	ev.fset("start", fz.t(start));
+	ev.set("ev.start_f0", fz.a(start));
+	ev.set("start", fz.t(start));
 	
 	// find fall part. The start of the search is FIXED by the position
 	// of where the rise stopped
@@ -391,22 +427,34 @@ static int rf_match(EST_Track &fz, EST_Item &ev, float range)
 	region = region > 0 ? region : 1;
 	e_start = e_stop - region;
 	
-	printf("fall: b_start  %d, b_stop %d, end %d, end stop%d\n", b_start,
-	       b_stop, e_start, e_stop);
+	// printf("fall: b_start  %d, b_stop %d, end %d, end stop%d\n", b_start,
+	//       b_stop, e_start, e_stop);
 	
 	match_rf_point(fz, b_start, b_stop, e_start, e_stop, start, stop);
-	cout << "Fall is at start: " << start << " Stop = " << stop << endl;
-	cout << "region: " << region << endl;
-	cout << "stop could be " << e_stop << " value " << fz.t(e_stop) << endl;
-	cout << "start could be " << e_start << " value " << fz.t(e_start) << endl;
-	ev.fset("rfc:peak_f0",fz.a(start));
-	ev.fset("rfc:peak_pos", fz.t(start));
-	
-	ev.fset("rfc:end_f0", fz.a(stop));
-	ev.fset("end", fz.t(stop));
-	
-	ev.fset("rfc:type", "RISEFALL");
-	cout << "labelled event: " << ev << endl;
+	// cout << "Fall is at start: " << start << " Stop = " << stop << endl;
+	// cout << "region: " << region << endl;
+	// cout << "stop could be " << e_stop << " value " << fz.t(e_stop) << endl;
+	// cout << "start could be " << e_start << " value " << fz.t(e_start) << endl;
+
+	ev.set("rfc.peak_f0", fz.a(start));
+	ev.set("rfc.peak_pos", fz.t(start));
+	ev.set("rfc.end_f0", fz.a(stop));
+
+	ev.set("end", fz.t(stop));
+	ev.set("rfc.type", "RISEFALL");
+
+/*	ev.set("rfc.setpeak_f0", fz.a(start));
+ 	ev.fA("rfc").set("peak_pos", fz.t(start));
+	 ev.fA("rfc",1).set("end_f0", fz.a(stop));
+
+	 ev.set("end", fz.t(stop));
+
+	 ev.fA("rfc").set("type", "RISEFALL");
+*/
+	// cout << "peak pos: " << ev.F("rfc.peak_pos") << endl;
+	// cout << "peak pos: " << ev.A("rfc").F("peak_pos") << endl;
+	// cout << "rfc:\n" << ev.A("rfc") << endl;
+	// cout << "labelled event: " << ev << endl;
     }
     else			// separate rise or fall
     {
@@ -419,27 +467,27 @@ static int rf_match(EST_Track &fz, EST_Item &ev, float range)
 	b_stop = b_start + region;
 	e_start = e_stop - region;
 	
-	printf("b_start  %d, b_stop %d, end start %d, end stop%d\n", b_start,
-	       b_stop, e_start, e_stop);
+	// printf("b_start  %d, b_stop %d, end start %d, end stop%d\n", b_start,
+	//        b_stop, e_start, e_stop);
 	
 	match_rf_point(fz, b_start, b_stop, e_start, e_stop, start, stop);
 	
-	ev.fset("ev:start_f0", fz.a(start));
-	ev.fset("start", fz.t(start));
-	ev.fset("rfc:peak_f0", 0.0);
-	ev.fset("rfc:peak_pos", 0.0);
+	ev.set("start", fz.t(start));
+	ev.set("ev.start_f0", fz.a(start));
+	ev.set("rfc.peak_f0", 0.0);
+	ev.set("rfc.peak_pos", 0.0);
 	
-	ev.fset("rfc:end_f0", fz.a(stop));
-	ev.fset("end", fz.t(stop));
+	ev.set("rfc.end_f0", fz.a(stop));
+	ev.set("end", fz.t(stop));
 	
-	cout  << "start " << fz.t(start) << " end " << fz.t(stop) << endl;
+	// cout  << "start " << fz.t(start) << " end " << fz.t(stop) << endl;
 	
 	if (fz.a(fz.index(fz.start())) < fz.a(fz.index(fz.end())))
-	    ev.fset("rfc:type", "RISE");
+	    ev.set("rfc.type", "RISE");
 	else
-	    ev.fset("rfc:type", "FALL");
+	    ev.set("rfc.type", "FALL");
 	
-	cout << "labelled event: " << ev << endl;
+	// cout << "labelled event: " << ev << endl;
     }
     return 0;
 }
@@ -449,61 +497,112 @@ static void silence_f0(EST_Relation &ev, EST_Track &fz)
     EST_Item * e;
     int i;
 
-    for (e = ev.head(); e; e = next(e))
+    for (e = ev.head(); e; e = inext(e))
 	if (sil_item(*e))
 	{
-	    i = fz.prev_non_break(fz.index(e->fF("start")));
-	    e->fset("ev:start_f0", fz.a(i));
-	    i = fz.next_non_break(fz.index(e->fF("end")));
-	    e->fset("ev:end_f0", fz.a(i));
+	    i = fz.prev_non_break(fz.index(e->F("start")));
+
+	    e->set("ev.start_f0", fz.a(i));
+	    i = fz.next_non_break(fz.index(e->F("end")));
+	    e->set("ev.end_f0", fz.a(i));
 	}
 }
 
 static void add_phrases(EST_Relation &ev)
 {
-    EST_Item *e, *n, *s, *p;
-    float min_duration = 0.02;
+    EST_Item *e, *n, *s;
 
-    for (e = ev.head(); next(e); e = n)
+    // cout << "phrase edges: " << endl;
+    // cout << ev;
+
+    for (e = ev.head(); e; e = n)
     {
-	n = next(e);
-	p = prev(e);
-	if (!sil_item(*e))
-	    continue;
-
-	s = 0;
-	if (p && (e->fF("start") - p->fF("end")) > min_duration)
+	n = inext(e);
+	if (sil_item(*e))
 	{
-	    s = e->insert_before();
-	    s->fset("name", "phrase_end");
-	    s->fset("ev:f0", e->fF("ev:start_f0"));
-	    s->fset("position", e->fF("start"));
-	}
-
-	if (n  && ((n->fF("start") - e->fF("end")) > min_duration))
-	{
-	    s = e->insert_after();
-	    s->fset("name", "phrase_start");
-	    s->fset("ev:f0", e->fF("ev:end_f0"));
-	    s->fset("position", e->fF("end"));
-	}
-
-	if (s == 0)
-	{
-	    s = e->insert_before();
-	    s->fset("name", "pause");
-	    s->fset("position", e->fF("start"));
+	    if (e != ev.head())
+	    {
+		s = e->insert_before();
+		s->set("name", "phrase_end");
+		s->set("ev.f0", e->F("ev.start_f0"));
+		s->set("time", e->F("start"));
+	    }
+	    if (e != ev.tail())
+	    {
+		s = e->insert_after();
+		s->set("name", "phrase_start");
+		s->set("ev.f0", e->F("ev.end_f0"));
+		s->set("time", e->F("end"));
+	    }
 	}
     }
 
     for (e = ev.head(); e; e = n)
     {
-	n = next(e);
+	n = inext(e);
 	if (sil_item(*e))
 	    ev.remove_item(e);
     }
 }
 
+/*
+static void add_phrases(EST_Relation &ev, bool phrase_edges)
+{
+    EST_Item *e, *n, *s, *p;
+    float min_duration = 0.02;
+
+    cout << "phrase edges: " << phrase_edges << endl;
+    cout << ev;
+
+    for (e = ev.head(); inext(e); e = n)
+    {
+	n = inext(e);
+	p = iprev(e);
+	if (!sil_item(*e))
+	    continue;
+
+	s = 0;
+	
+	if ((e != ev.head())  && (phrase_edges
+				  || (p &&(e->F("start") - p->F("end")) 
+				      > min_duration)))
+	{
+	    s = e->insert_before();
+	    s->set("name", "phrase_end");
+	    s->set("ev.f0", e->F("ev.start_f0", 1));
+	    s->set("position", e->F("start"));
+	}
+
+	if (phrase_edges || (n &&((n->F("start")- e->F("end")) >min_duration)))
+	{
+	    s = e->insert_after();
+	    s->set("name", "phrase_start");
+	    s->set("ev.f0", e->F("ev.end_f0",1));
+	    s->set("position", e->F("end"));
+	}
+
+	if (s == 0)
+	{
+	    s = e->insert_before();
+	    s->set("name", "pause");
+	    s->set("position", e->F("start"));
+	}
+    }
+
+    s = e->insert_after();
+    s->set("name", "phrase_end");
+    s->set("ev.f0", e->F("ev.start_f0", 1));
+    s->set("position", e->F("end"));
+
+    for (e = ev.head(); e; e = n)
+    {
+	n = inext(e);
+	if (sil_item(*e))
+	    ev.remove_item(e);
+    }
+    cout << "end phrase edges\n";
+}
+*/
 static void make_int_item(EST_Item &tmp, 
 			  const EST_String name, float end, float start,
 			  float start_f0, float peak_pos, 
@@ -511,16 +610,19 @@ static void make_int_item(EST_Item &tmp,
 
 {
     tmp.set_name(name);
+    EST_Features dummy;
     
-    tmp.fset("start", start);
-    tmp.fset("end", end);
-    tmp.fset("ev:start_f0", start_f0);
+    tmp.set("start", start);
+    tmp.set("end", end);
+    tmp.set("ev", dummy);
+    tmp.set("ev.start_f0", start_f0);
     
     if ((name != "sil") && (name != "c"))
     {
-	tmp.fset("rfc:peak_pos", peak_pos);	
-	tmp.fset("rfc:peak_f0", peak_f0);
-	tmp.fset("rfc:pos", 1);
+	tmp.set("rfc", dummy);
+	tmp.set("rfc.peak_pos", peak_pos);
+	tmp.set("rfc.peak_f0", peak_f0);
+	tmp.set("rfc.pos", 1);
     }
 }
 
@@ -563,7 +665,7 @@ static int match_rf_point(EST_Track &fz, int b_start, int b_stop,
     EST_Track new_fz(fz.num_frames(), 1);
     float f_shift;
     
-    mi = mj = 0;		// set values to zero for saftey
+    mi = mj = 0;		// set values to zero for safety
     
     if ((b_start >= b_stop) || (b_start < 0))
     {
@@ -615,7 +717,9 @@ static int match_rf_point(EST_Track &fz, int b_start, int b_stop,
     return 0;
 }
 
+/*
 #if 0
+
 static void fill_f0_values(EST_Track &fz, EST_Relation &ev)
 {
     EST_Item *e;
@@ -623,7 +727,7 @@ static void fill_f0_values(EST_Track &fz, EST_Relation &ev)
     int pos;
     float prev = 0.0;
     
-    for (e = ev.head(); e != 0; e = next(e))
+    for (e = ev.head(); e != 0; e = inext(e))
     {
 	if (e->name() == "sil")
 	{
@@ -640,15 +744,15 @@ static void fill_f0_values(EST_Track &fz, EST_Relation &ev)
 	else 
 	    start_a = fz.a(prev);
 	
-	e->fset("ev:start_f0", start_a);
-	e->fset("start", prev);
+	e->set("ev:start_f0", start_a);
+	e->set("start", prev);
 	//	cout << "setting start to be " << start_a << " at pos " << pos << endl;
 	//	cout << *e << " " << *RFCS(*e) << endl;
 	
 	if (e->f("rfc:type") == "RISEFALL")
 	{
-	    start_a = fz.a(e->fF("rfc:peak_pos"));
-	    e->fset("rfc:peak_f0", start_a);
+	    start_a = fz.a(e->F("rfc:peak_pos"));
+	    e->set("rfc:peak_f0", start_a);
 	}
 	prev = e->f("end");
     }
@@ -670,20 +774,20 @@ static void insert_silence(EST_Item *n, EST_Track &fz, int i, int j)
     start_f0 = (i > 0) ? fz.a(i -1) : fz.a(i);
     new_sil = n->insert_after();
     make_int_item(*new_sil, "sil", sil_end, sil_start, start_f0, 0.0, 0.0);
-    new_sil->fset("rfc:type", "SIL");
+    new_sil->set("rfc:type", "SIL");
     
     // if silence leaves a gap, make a new element before it
-    if ((sil_start - n->fF("start")) < min_length)
+    if ((sil_start - n->F("start")) < min_length)
 	return;
     
     // make new element, setting end time to silence start time.
-    prev_item = prev(n)->insert_before();
+    prev_item = n->prev()->insert_before();
     make_int_item(*prev_item, n->name(), sil_start, 0.0, n->f("ev:start_f0"),
 			      0.0,0.0);
 
     // now tidy up values of existing element
-    n->fset("ev:start_f0", fz.a(j));
-    n->fset("start", sil_end);
+    n->set("ev:start_f0", fz.a(j));
+    n->set("start", sil_end);
 }
 
 static void add_phrases_old(EST_Relation &ev, EST_Track &fz)
@@ -693,11 +797,11 @@ static void add_phrases_old(EST_Relation &ev, EST_Track &fz)
     bool sil = false;
     float start, end;
 
-    for (e = ev.head(); next(e); e = n)
+    for (e = ev.head(); inext(e); e = n)
     {
-	n = next(e);
-	start = e->fF("end");
-	end = n->fF("start");
+	n = inext(e);
+	start = e->F("end");
+	end = n->F("start");
 	sil = false;
 
 	cout << endl << endl;
@@ -717,10 +821,10 @@ static void add_phrases_old(EST_Relation &ev, EST_Track &fz)
 		cout << "phrase_end\n";
 		sil = true;
 		s = e->insert_after();
-		s->fset("name", "phrase_end");
-		s->fset("position", fz.t(i - 1));
+		s->set("name", "phrase_end");
+		s->set("position", fz.t(i - 1));
 		if (i > (fz.index(start) + 1))
-		    s->fset("ev:f0", fz.a(i - 1));
+		    s->set("ev:f0", fz.a(i - 1));
 		e = s;
 	    }
 
@@ -729,9 +833,9 @@ static void add_phrases_old(EST_Relation &ev, EST_Track &fz)
 		cout << "phrase_start\n";
 		sil = false;
 		s = e->insert_after();
-		s->fset("name", "phrase_start");
-		s->fset("position", fz.t(i));
-		s->fset("ev:f0", fz.a(i));
+		s->set("name", "phrase_start");
+		s->set("position", fz.t(i));
+		s->set("ev:f0", fz.a(i));
 	    }
 	}
     }
@@ -754,11 +858,11 @@ static void add_silences(EST_Track &fz, EST_Relation &ev,
 	    cout << "silence between " <<i << " and " << j << endl;
 	    //	    cout << "  " << fz.t(i) << " and " << fz.t(j) << endl;
 	    
-	    for (e = ev.head(); e != 0; e = next(e))
-		if (e->fF("end") >= fz.t(j))
+	    for (e = ev.head(); e != 0; e = inext(e))
+		if (e->F("end") >= fz.t(j))
 		    break;
 	    insert_silence(e, fz, i, j);
-	    //	    for (e = ev.head(); e != 0; e = next(e))
+	    //	    for (e = ev.head(); e != 0; e = inext(e))
 	    //		cout << *e << " : " << *RFCS(*e) << endl;
 	    
 	    i = j;
@@ -772,16 +876,16 @@ static void add_silences(EST_Track &fz, EST_Relation &ev,
     e = ev.append();    
     make_int_item(*e, "sil", fz.end() + end_sil_length, fz.end(), 
 			    start_f0, 0.0, 0.0);
-    e->fset("rfc:type", "SIL");
+    e->set("rfc:type", "SIL");
 }
-
-static void minimum_duration(EST_Relation &ev, float min_dur)
+*/
+/*static void minimum_duration(EST_Relation &ev, float min_dur)
 {
     EST_Item *e, *n;
     
     for (e = ev.head(); e != 0; e = n)
     {
-	n = next(e);
+	n = inext(e);
 	if (dur(*e) < min_dur)
 	    ev.remove_item(e);
     }
@@ -792,39 +896,40 @@ static void adjust_overlaps(EST_Relation &ev)
     EST_Item *e, *n;
     float pos=0.0;
     
-    for (e = ev.head(); next(e) != 0; e = next(e))
+    for (e = ev.head(); inext(e) != 0; e = e->next())
     {
-	n = next(e);
-	if (e->fF("end") > n->fF("start"))
+	n = inext(e);
+	if (e->F("end") > n->F("start"))
 	{
+*/
 /*	    cout << "Overlapping events " << *e <<":" << *n << endl;
-	    // case a: genunine overlap
-	    if (n->fF("end") > e->fF("end"))
+	    // case a: genuine overlap
+	    if (n->F("end") > e->F("end"))
 	    {
 		cout << "case A\n";
-//		pos = (e->fF("end") + start(n)) / 2.0;
+//		pos = (e->F("end") + start(n)) / 2.0;
 	    }
 	    
 	    // case b: second element is enclosed by first
-	    else if (n->fF("end") <= e->fF("end"))
+	    else if (n->F("end") <= e->F("end"))
 	    {
 		cout << "case A\n";
 //		pos = start(n);
 	    }
 	    
 	    // case c: second element is before first
-*	    else if ((n->fF("end") < e->fF("end")) &&
+*	    else if ((n->F("end") < e->F("end")) &&
 		     (start(n) < start(e)))
 	    {
 		cout << "case A\n";
-		pos = (n->fF("end") + start(e)) / 2.0;
+		pos = (n->F("end") + start(e)) / 2.0;
 	    }
 	    else
 		cout << "No overlap conditions met\n";
 	    //	    cout << "pos =" << pos << endl;
 */
-	    e->fset("end", pos);
-	    n->fset("start", pos);
+/*	    e->set("end", pos);
+	    n->set("start", pos);
 	    cout << endl << endl;
 	}
     }
@@ -832,11 +937,11 @@ static void adjust_overlaps(EST_Relation &ev)
     // The overlap adjustments may cause the peak position to lie outside
     // the start and end points. This checks for this and makes an
     // arbitrary adjustment
-    for (e = ev.head(); next(e) != 0; e = next(e))
-	if ((e->f("rfc:type") == "RISEFALL") && (e->fF("rfc:peak_pos") <
-						 e->fF("start")))
-	    e->fset("rfc:peak_pos", 
-		    (e->fF("start") + e->fF("end") / 2.0));
+    for (e = ev.head(); inext(e) != 0; e = inext(e))
+	if ((e->f("rfc:type") == "RISEFALL") && (e->F("rfc:peak_pos") <
+						 e->F("start")))
+	    e->set("rfc:peak_pos", 
+		    (e->F("start") + e->F("end") / 2.0));
 }
 
 static void conn_ends(EST_Track &fz, EST_Relation &ev, float min_length)
@@ -847,26 +952,26 @@ static void conn_ends(EST_Track &fz, EST_Relation &ev, float min_length)
     
     cout << min_length << endl;
     
-    for (e = ev.head(); next(e) != 0; e = next(e))
+    for (e = ev.head(); inext(e) != 0; e = inext(e))
     {
-	n = next(e);
+	n = inext(e);
 	cout << *e << ":";
-	cout << "n: " << n->fF("start") << " e "<< e->fF("end") << endl;
+	cout << "n: " << n->F("start") << " e "<< e->F("end") << endl;
 	
-	if ((n->fF("start") - e->fF("end")) > min_length)
+	if ((n->F("start") - e->F("end")) > min_length)
 	{
 	    cout << "making connection\n";
 	    tmp = n->insert_before();
 	    make_int_item(*tmp, "c", n->f("start"), e->f("start"),
 				e->f("rfc:end_f0"), 0.0, 0.0);
 
-	    e = next(e);		// advance after new connection
+	    e = inext(e);		// advance after new connection
 	} 
 	else
 	{ 
-	    t = (n->fF("start") + e->fF("end")) /2.0;
-	    n->fset("start", t);
-	    e->fset("end", t);
+	    t = (n->F("start") + e->F("end")) /2.0;
+	    n->set("start", t);
+	    e->set("end", t);
 	}
     }
     
@@ -883,9 +988,9 @@ static void conn_ends(EST_Track &fz, EST_Relation &ev, float min_length)
     tmp = ev.head()->insert_after();
     make_int_item(*tmp, "c", t, 0.0, fz.a(fz.start()), 0.0, 0.0);
     
-    if ((ev.tail()->fF("end") + min_length) < fz.end())
+    if ((ev.tail()->F("end") + min_length) < fz.end())
     {
-	f0 = fz.a(ev.tail()->fF("end"));
+	f0 = fz.a(ev.tail()->F("end"));
 	// add connection after last event.
 	//ev.insert_after(ev.tail(), tmp);
 	tmp = ev.append();
@@ -898,6 +1003,7 @@ static void conn_ends(EST_Track &fz, EST_Relation &ev, float min_length)
     make_int_item(*tmp, "sil", fz.end() + ARB_DISTANCE, 
 			0.0, fz.a(fz.end()), 0.0, 0.0);
 }
+*/
+
 
-#endif
 
