@@ -5,15 +5,19 @@
 /*                      Copyright (c) 1995,1996                          */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -33,9 +37,9 @@
 /*                                                                       */
 /*=======================================================================*/
 
-#include <iostream.h>
-#include <fstream.h>
-#include <math.h>
+#include <iostream>
+#include <fstream>
+#include <cmath>
 #include "EST_types.h"
 #include "EST_FMatrix.h"
 #include "ling_class/EST_Relation.h"
@@ -48,15 +52,15 @@
 
 int close_enough(EST_Item &a, EST_Item &b)
 {
-    return ((start(&b) < a.fF("end")) && (start(&a) < b.fF("end")));
+    return ((start(&b) < a.F("end")) && (start(&a) < b.F("end")));
 }
 
 // WATCH - this uses what should be private access to Keyval class.
 void monotonic_match(EST_II_KVL &a, EST_II_KVL &b)
 {
-    EST_TBI *ptr;
+    EST_Litem *ptr;
 
-    for (ptr = a.list.head(); ptr != 0; ptr= next(ptr))
+    for (ptr = a.list.head(); ptr != 0; ptr = ptr->next())
     {
 	if (a.val(ptr) == -1)
 	    continue;
@@ -66,7 +70,7 @@ void monotonic_match(EST_II_KVL &a, EST_II_KVL &b)
 	else
 	    a.change_key(ptr, -1);
     }
-    for (ptr = b.list.head(); ptr != 0; ptr= next(ptr))
+    for (ptr = b.list.head(); ptr != 0; ptr = ptr->next())
     {
 	if (b.val(ptr) == -1)
 	    continue;
@@ -85,16 +89,16 @@ void function_match(EST_II_KVL &u, EST_Relation &a, EST_Relation &b)
     (void)b;
 #if 0
     EST_Item *a_ptr;
-    EST_TBI *i_ptr;
+    EST_Litem *i_ptr;
     int i;
 
-    for (i = 0, a_ptr = a.head(); a_ptr != 0; a_ptr = next(a_ptr), ++i)
+    for (i = 0, a_ptr = a.head(); a_ptr != 0; a_ptr = inext(a_ptr), ++i)
     {
 	if (a_ptr->f("pos")==1)
 	{
 	    u.add_item(a_ptr->addr(), -1);
 	    for (i_ptr = a_ptr->link(b.stream_name())->head(); i_ptr != 0; 
-		 i_ptr = next(i_ptr))
+		 i_ptr = i_ptr->next())
 		u.change_val(a_ptr->addr(), a_ptr->link(b.stream_name())->item(i_ptr));
 	}
     }
@@ -105,9 +109,9 @@ void relation_match(EST_Relation &a, EST_Relation &b)
 {
     EST_Item *a_ptr, *b_ptr;
 
-    for (a_ptr = a.head(); a_ptr != 0; a_ptr = next(a_ptr))
+    for (a_ptr = a.head(); a_ptr != 0; a_ptr = inext(a_ptr))
 	if (a_ptr->f("pos")==1)
-	    for (b_ptr = b.head(); b_ptr != 0; b_ptr = next(b_ptr))
+	    for (b_ptr = b.head(); b_ptr != 0; b_ptr = inext(b_ptr))
 		{
 		    if ((b_ptr->f("pos")==1)
 			&&(close_enough(*a_ptr, *b_ptr)))
@@ -170,7 +174,7 @@ EST_Item *nthpos(EST_Relation &a, int n)
 {
     EST_Item *a_ptr;
     int i = 0;
-    for (a_ptr = a.head(); a_ptr != 0; a_ptr = next(a_ptr))
+    for (a_ptr = a.head(); a_ptr != 0; a_ptr = inext(a_ptr))
 	if (a_ptr->f("pos") == 1)
 	{
 	    if (n == i)
@@ -186,7 +190,7 @@ float label_distance1(EST_Item &ref, EST_Item &test)
     float s, e;
 
     s = fabs(start(&ref) - start(&test));
-    e = fabs(ref.fF("end") - test.fF("end"));
+    e = fabs(ref.F("end") - test.F("end"));
 
     return (s + e) / duration(&ref);
 }
@@ -198,8 +202,8 @@ float label_distance2(EST_Item &ref, EST_Item &test)
     float s, e;
 
     s = (start(&test) < start(&ref)) ? start(&ref) - start(&test) : 0;
-    e = (ref.fF("end") < test.fF("end")) ? 
-	test.fF("end") - ref.fF("end") : 0;
+    e = (ref.F("end") < test.F("end")) ? 
+	test.F("end") - ref.F("end") : 0;
 
     return (s + e) / duration(&ref);
 }
@@ -354,7 +358,7 @@ void threshold_labels(EST_Relation &reflab, float t)
     float score=0.0;
     int a;
 
-    for (r_ptr = reflab.head(); r_ptr != 0; r_ptr = next(r_ptr))
+    for (r_ptr = reflab.head(); r_ptr != 0; r_ptr = inext(r_ptr))
 	if (r_ptr->f("pos")==1)
 	{
 	    // REORG - temp comment
@@ -371,7 +375,7 @@ void threshold_labels(EST_Relation &reflab, float t)
 
 /* Check through relations of each ref label, and make there aren't
 multiple ref labels related to the same test label. If this is
-discoved, compare scores of all the relevant labels and delete the
+discovered, compare scores of all the relevant labels and delete the
 relations of all but the lowest.
 
 At this stage, each ref label should have one and only one relation to the
@@ -383,13 +387,13 @@ void multiple_labels(EST_Relation &reflab)
     (void)reflab;
 #if 0
     EST_Item *r_ptr, *s_ptr, *t_ptr;;
-    EST_TBI *p;
+    EST_Litem *p;
     int a, pos, i;
     EST_TList<int> la;
     float *score;
     score = new float[reflab.length()];
 
-    for (r_ptr = reflab.head(); r_ptr != 0; r_ptr = next(r_ptr))
+    for (r_ptr = reflab.head(); r_ptr != 0; r_ptr = inext(r_ptr))
 	if (r_ptr->f("pos")==1)
 	{
 	    la.clear(); // clear list and add address of current ref label
@@ -399,7 +403,7 @@ void multiple_labels(EST_Relation &reflab)
 
 	    // check remainer of ref labels and add any that have same
 	    // relations address as r_ptr.
-	    for (s_ptr = next(r_ptr); s_ptr != 0; s_ptr = next(s_ptr))
+	    for (s_ptr = inext(r_ptr); s_ptr != 0; s_ptr = inext(s_ptr))
 		if (s_ptr->f("pos")==1)
 		    if (s_ptr->rlink("test").first() == a)
 			la.append(s_ptr->addr());
@@ -408,7 +412,7 @@ void multiple_labels(EST_Relation &reflab)
 	    if (la.length() > 1) // true if the are multiple relations
 	    {
 		// find scores of all relevant labels
-		for (i = 0, p = la.head(); p!= 0; p = next(p), ++i)
+		for (i = 0, p = la.head(); p!= 0; p = p->next(), ++i)
 		{
 		    t_ptr = reflab.item(la.item(p));
 
@@ -419,7 +423,7 @@ void multiple_labels(EST_Relation &reflab)
 
 		cout << "best is " << pos << endl;
 		// delete relations of all but lowest score
-		for (i = 0, p = la.head(); p!= 0; p = next(p), ++i)
+		for (i = 0, p = la.head(); p!= 0; p = p->next(), ++i)
 		    if (i != pos)
 		    {
 			t_ptr = reflab.item(la.item(p));
@@ -438,12 +442,12 @@ in the reference set. The lowest score for each reference is then
 recorded. A test is carried out to make sure that no two reference
 labels point to the same test label. Then any ref label above a
 certain distance is classified as incorrect. The numbers of insertions
-and delations are then calculated.  */
+and deletions are then calculated.  */
 
 EST_FMatrix matrix_compare(EST_Relation &reflab, EST_Relation &testlab, int method,
 		       float t, int v)
 {
-    int i, j, pos;
+    int i, j;
     int num_ref, num_test;
     EST_Item *r_ptr, *t_ptr;
     EST_String fns;
@@ -452,11 +456,11 @@ EST_FMatrix matrix_compare(EST_Relation &reflab, EST_Relation &testlab, int meth
     num_ref = num_test = 0;
 
     // calculate size of matrix, based on *significant* labels
-    for (r_ptr = testlab.head(); r_ptr != 0; r_ptr = next(r_ptr))
+    for (r_ptr = testlab.head(); r_ptr != 0; r_ptr = inext(r_ptr))
 	if (r_ptr->f("pos")==1)
 	    ++num_test;
 
-    for (r_ptr = reflab.head(); r_ptr != 0; r_ptr = next(r_ptr))
+    for (r_ptr = reflab.head(); r_ptr != 0; r_ptr = inext(r_ptr))
 	if (r_ptr->f("pos")==1)
 	    ++num_ref;
 
@@ -468,10 +472,10 @@ EST_FMatrix matrix_compare(EST_Relation &reflab, EST_Relation &testlab, int meth
     // fill matrix values by comparing each test with each reference
     // reference is columns, test is rows
 
-    for (i = 0, t_ptr = testlab.head(); t_ptr != 0; t_ptr = next(t_ptr))
+    for (i = 0, t_ptr = testlab.head(); t_ptr != 0; t_ptr = inext(t_ptr))
 	if (t_ptr->f("pos")==1)
 	{
-	    for (j = 0, r_ptr = reflab.head(); r_ptr != 0; r_ptr = next(r_ptr))
+	    for (j = 0, r_ptr = reflab.head(); r_ptr != 0; r_ptr = inext(r_ptr))
 		if (r_ptr->f("pos")==1)
 		{
 		    if (method == 1)
@@ -491,8 +495,10 @@ EST_FMatrix matrix_compare(EST_Relation &reflab, EST_Relation &testlab, int meth
     minimise_matrix_by_row(m);
     matrix_ceiling(m, t);
 
+#if 0
+    /* This doesn't do anything */
     // for each ref label, find closest matching test label.
-    for (j = 0, r_ptr = reflab.head(); r_ptr != 0; r_ptr = next(r_ptr))
+    for (j = 0, r_ptr = reflab.head(); r_ptr != 0; r_ptr = inext(r_ptr))
     {
 	if (r_ptr->f("pos")==1)
 	{
@@ -502,6 +508,7 @@ EST_FMatrix matrix_compare(EST_Relation &reflab, EST_Relation &testlab, int meth
 	    ++j;
 	}
     }
+#endif
     return m;
 }
 
@@ -509,7 +516,7 @@ void multiple_matrix_compare(EST_RelationList &rlist, EST_RelationList
 			     &tlist, EST_FMatrix &m, EST_String rpos,
 			     EST_String tpos, int method, float t, int v)
 {
-    EST_TBI *pr, *pt;
+    EST_Litem *pr, *pt;
     EST_String filename;
     EST_Relation reflab, testlab;
     EST_StrList rposlist, tposlist, rminorlist, tminorlist;
@@ -523,7 +530,7 @@ void multiple_matrix_compare(EST_RelationList &rlist, EST_RelationList
     int tot, del, ins, ltot, ldel, lins, lmdel, mdel, lmins, mins;
     tot = del = ins = mdel = mins = 0;
 
-    for (pt = tlist.head(); pt; pt = next(pt))
+    for (pt = tlist.head(); pt; pt = pt->next())
     {
 	pr = RelationList_ptr_extract(rlist, tlist(pt).name(), TRUE);
 	if (pr != 0)
@@ -584,24 +591,24 @@ void error_location(EST_Relation &e, EST_FMatrix &m, int ref)
     // reference
     if (ref)
     {
-	for (i = 0, s = e.head(); s; s = next(s))
+	for (i = 0, s = e.head(); s; s = inext(s))
 	    if ((int)s->f("pos"))
 	    {
 		if (column_hit(m, i) >= 0)
-		    s->fset("hit", 1);
+		    s->set("hit", 1);
 		else
-		    s->fset("hit", 0);
+		    s->set("hit", 0);
 		++i;
 	    }
     }
     else
-	for (i = 0, s = e.head(); s; s = next(s))
+	for (i = 0, s = e.head(); s; s = inext(s))
 	    if ((int)s->f("pos"))
 	    {
 		if (row_hit(m, i) >= 0)
-		    s->fset("hit", 1);
+		    s->set("hit", 1);
 		else
-		    s->fset("hit", 0);
+		    s->set("hit", 0);
 		++i;
 	    }
 }
@@ -609,9 +616,9 @@ void error_location(EST_Relation &e, EST_FMatrix &m, int ref)
 int insdel(EST_II_KVL &a)
 {
     int n = 0;
-    EST_TBI *ptr;
+    EST_Litem *ptr;
     
-    for (ptr = a.list.head(); ptr != 0; ptr= next(ptr))
+    for (ptr = a.list.head(); ptr != 0; ptr= ptr->next())
 	if (a.val(ptr) == -1)
 	    ++n;
     return n;
@@ -655,13 +662,13 @@ void reassign_links(EST_Relation &a, EST_Relation &b, EST_II_KVL &ua, EST_II_KVL
 #if 0    
     EST_Item *a_ptr, *b_ptr;
 
-    for (a_ptr = a.head(); a_ptr != 0; a_ptr = next(a_ptr))
+    for (a_ptr = a.head(); a_ptr != 0; a_ptr = inext(a_ptr))
     {
 	a_ptr->link(b.stream_name())->clear();
 	if ((a_ptr->f("pos")==1) && (ua.val(a_ptr->addr()) != -1))
 	    a_ptr->make_link(b.stream_name(), ua.val(a_ptr->addr()));
     }
-    for (b_ptr = b.head(); b_ptr != 0; b_ptr = next(b_ptr))
+    for (b_ptr = b.head(); b_ptr != 0; b_ptr = inext(b_ptr))
     {
 	b_ptr->link(a.stream_name())->clear();
 	if ((b_ptr->f("pos")==1) && (ub.val(b_ptr->addr()) != -1))
@@ -678,7 +685,7 @@ void reassign_links(EST_Relation &a, EST_II_KVL &u, EST_String stream_type)
 #if 0    
     EST_Item *a_ptr;
 
-    for (a_ptr = a.head(); a_ptr != 0; a_ptr = next(a_ptr))
+    for (a_ptr = a.head(); a_ptr != 0; a_ptr = inext(a_ptr))
     {
 	a_ptr->link(stream_type)->clear();
 	if ((a_ptr->f("pos")==1) && (u.val(a_ptr->addr()) != -1))
@@ -770,7 +777,7 @@ void test_labels(EST_Utterance &ref, EST_Utterance &test, EST_Option &op)
     int correct, n_ev, n_syl;
     
     correct = n_ev = n_syl = 0;
-    for (a_ptr = ref.stream("Event").head(); a_ptr != 0; a_ptr = next(a_ptr))
+    for (a_ptr = ref.stream("Event").head(); a_ptr != 0; a_ptr = inext(a_ptr))
 	if (a_ptr->f("pos")==1)
 	{
 	    ++n_ev;
@@ -778,7 +785,7 @@ void test_labels(EST_Utterance &ref, EST_Utterance &test, EST_Option &op)
 		== commutate(a_ptr, f1, f2, lref, ltest))
 		++correct;
 	}
-    for (a_ptr = ref.stream("Syllable").head();a_ptr != 0; a_ptr = next(a_ptr))
+    for (a_ptr = ref.stream("Syllable").head();a_ptr != 0; a_ptr = inext(a_ptr))
 	if (a_ptr->f("pos")==1)
 	    ++n_syl;
     
@@ -792,7 +799,7 @@ void test_labels(EST_Utterance &ref, EST_Utterance &test, EST_Option &op)
     
     if (op.present("print_derivation"))
     {
-	for (a_ptr = ref.stream("Event").head();a_ptr!= 0; a_ptr = next(a_ptr))
+	for (a_ptr = ref.stream("Event").head();a_ptr!= 0; a_ptr = inext(a_ptr))
 	{
 	    if (a_ptr->f("pos")==1)
 	    {
@@ -834,7 +841,7 @@ void print_matrix_scores(EST_Relation &ref, EST_Relation &test, EST_FMatrix &a)
     EST_Item *r_ptr, *t_ptr;
     
     cout << "      ";
-    for (r_ptr = ref.head(); r_ptr != 0; r_ptr = next(r_ptr))
+    for (r_ptr = ref.head(); r_ptr != 0; r_ptr = inext(r_ptr))
     {	
 	if (r_ptr->f("pos")==1)
 	{
@@ -843,12 +850,12 @@ void print_matrix_scores(EST_Relation &ref, EST_Relation &test, EST_FMatrix &a)
 	    cout << r_ptr->name() << " ";
 	    cout.width(6);
 	    cout.setf(ios::right);
-	    cout<< r_ptr->fF("end") << " ";
+	    cout<< r_ptr->F("end") << " ";
 	}
     }
     cout << endl;
     
-    for (t_ptr = test.head(), i = 0; i < a.num_rows(); t_ptr = next(t_ptr))
+    for (t_ptr = test.head(), i = 0; i < a.num_rows(); t_ptr = inext(t_ptr))
     {
 	if (t_ptr->f("pos")==1)
 	{
@@ -928,7 +935,7 @@ void make_hit_and_miss(EST_Relation &a)
 {
     EST_Item *s;
     
-    for (s = a.head(); s; s = next(s))
+    for (s = a.head(); s; s = inext(s))
     {
 	if (s->f("pos") == 0)
 	    s->set_name(".");
@@ -946,7 +953,7 @@ void pos_only(EST_Relation &lab)
     
     for (a = lab.head(); a; a = n)
     {
-	n = next(a);
+	n = inext(a);
 	if (!a->f_present("pos"))
 	    lab.remove_item(a);
     }
@@ -971,7 +978,7 @@ void print_aligned_trans(EST_Relation &ref, EST_Relation &test, EST_FMatrix &m)
     pos_only(test);
     pos_only(ref);
 
-    // first check for emtpy matrices - indicates all insertions or deletions
+    // first check for empty matrices - indicates all insertions or deletions
 
     if ((m.num_columns() == 0) && (m.num_rows() != 0))
     {
@@ -1004,7 +1011,7 @@ void print_aligned_trans(EST_Relation &ref, EST_Relation &test, EST_FMatrix &m)
     
     pos.set_name(ref.head()->name());
     refal.append(pos);	
-    for (i = 1, p = next(ref.head()); p; p = next(p), ++i)
+    for (i = 1, p = ref.head()->next(); p; p = p->next(), ++i)
     {
 	n = num_b_insertions(m, l, i);
 	
@@ -1021,7 +1028,7 @@ void print_aligned_trans(EST_Relation &ref, EST_Relation &test, EST_FMatrix &m)
     l = 0;
     pos.set_name(test.head()->name());
     testal.append(pos);
-    for (i = 1, p = next(test.head()); p; p = next(p), ++i)
+    for (i = 1, p = test.head()->next(); p; p = p->next(), ++i)
     {
 	n = num_b_deletions(m, l, i);
 	
