@@ -5,15 +5,19 @@
  /*                       Copyright (c) 1996,1997                        */
  /*                        All Rights Reserved.                          */
  /*                                                                      */
- /*  Permission to use, copy, modify, distribute this software and its   */
- /*  documentation for research, educational and individual use only, is */
- /*  hereby granted without fee, subject to the following conditions:    */
+ /*  Permission is hereby granted, free of charge, to use and distribute */
+ /*  this software and its documentation without restriction, including  */
+ /*  without limitation the rights to use, copy, modify, merge, publish, */
+ /*  distribute, sublicense, and/or sell copies of this work, and to     */
+ /*  permit persons to whom this work is furnished to do so, subject to  */
+ /*  the following conditions:                                           */
  /*   1. The code must retain the above copyright notice, this list of   */
  /*      conditions and the following disclaimer.                        */
  /*   2. Any modifications must be clearly marked as such.               */
  /*   3. Original authors' names are not deleted.                        */
- /*  This software may not be used for commercial purposes without       */
- /*  specific prior written permission from the authors.                 */
+ /*   4. The authors' names are not used to endorse or promote products  */
+ /*      derived from this software without specific prior written       */
+ /*      permission.                                                     */
  /*                                                                      */
  /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK       */
  /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING     */
@@ -36,24 +40,56 @@
  /*************************************************************************/
 
 
-#include <stdio.h>
+#include <cstdio>
 #include "EST.h"
 #include "EST_String.h"
 #include "EST_error.h"
 
 #define BUFFER_SIZE (1024)
 
+
+/** @name <command>bcat</command> <emphasis>Binary safe version of cat</emphasis>
+    @id bcat-manual
+  * @toc
+ */
+
+//@{
+
+
+/**@name Synopsis
+  */
+//@{
+
+//@synopsis
+
+/**
+bcat is a trivial file concatenation program. It exists to allow testing
+of various file splitting operations under the cygwin environment on Windows
+where the distinction between binary and text data is important.
+ */
+
+//@}
+
+/**@name OPTIONS
+  */
+//@{
+
+//@options
+
+//@}
+
+
 int main(int argc, char *argv[]) 
 { 
     EST_StrList files;
     EST_Option settings, cmd_line;
 
-    parse_command_line(argc, argv, 
-       EST_String("Usage:   ")+
-       "bcat -o <ofile> {files...} \n"+
-       "Summary; concatenate files in binary mode\n"+
-       "-o <ofile>       Ouptut file of binary data\n",
-			files, cmd_line);
+    parse_command_line
+	(argc, argv, 
+	 EST_String("-o [ofile] [files...]\n")+
+	 "Summary; concatenate files in binary mode\n"+
+	 "-o <ofile>       Ouptut file of binary data\n",
+	 files, cmd_line);
 
     EST_String out_file;
 
@@ -69,7 +105,7 @@ int main(int argc, char *argv[])
 
     EST_Litem *item;
 
-    for(item=files.head(); item; item = next(item))
+    for(item=files.head(); item; item = item->next())
       {
 	FILE *src;
 
