@@ -5,15 +5,19 @@
 /*                      Copyright (c) 1995,1996                          */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -33,9 +37,9 @@
 /*                                                                       */
 /*=======================================================================*/
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <math.h>
+#include <cstdlib>
+#include <cstdio>
+#include <cmath>
 #include "EST.h"
 
 EST_read_status load_TList_of_StrVector(EST_TList<EST_StrVector> &w,
@@ -44,8 +48,8 @@ EST_read_status load_TList_of_StrVector(EST_TList<EST_StrVector> &w,
 
 static void print_results(EST_Relation &wstream);
 static bool do_search(EST_Relation &wstream);
-static EST_VTPath *vit_npath(EST_VTPath *p,EST_VTCandidate *c);
-static EST_VTCandidate *vit_candlist(EST_Item &s);
+static EST_VTPath *vit_npath(EST_VTPath *p,EST_VTCandidate *c,EST_Features &f);
+static EST_VTCandidate *vit_candlist(EST_Item *s,EST_Features &f);
 static void top_n_candidates(EST_VTCandidate* &all_c);
 static void load_vocab(const EST_String &vfile);
 
@@ -89,7 +93,7 @@ static double ob_log_prob_floor2 = SAFE_LOG_ZERO;
 static double lm_log_prob_floor = SAFE_LOG_ZERO;  
 
 int btest_debug = FALSE;
-static EST_String outfile = "";
+static EST_String out_file = "";
 static EST_StrList vocab;
 static EST_Track observations;  
 static EST_Track observations2;  
@@ -100,6 +104,73 @@ int using_given=FALSE;
 int take_logs = FALSE;
 int num_obs = 1;
 
+
+
+
+/** @name  <command>viterbi</command> <emphasis>Combine n-gram model and likelihoods to estimate posterior probabilities</emphasis>
+  * @id viterbi-manual
+  * @toc
+ */
+
+//@{
+
+/**@name Synopsis
+  */
+//@{
+
+//@synopsis
+
+/**
+viterbi is a simple time-synchronous Viterbi decoder. It finds the
+most likely sequence of items drawn from a fixed vocabulary, given
+frame-by-frame observation probabilities for each item in that
+vocabulary, and a ngram grammar. Possible uses include:
+
+<itemizedlist>
+<listitem><para>Simple speech recogniser back end</para></listitem>
+</itemizedlist>
+
+viterbi can optionally use two sets of frame-by-frame observation
+probabilities in a weighted-sum fashion. Also, the ngram language model
+is not restricted to the conventional sliding window type in which the
+previous n-1 items are the ngram context. Items in the ngram context
+at each frame may be given. In this case, the user must provide a file
+containing the ngram context: one (n-1) tuple per line. To include
+items from the partial Viterbi path so far (i.e. found at recognition
+time, not given) the special notation <-N> is used where N indicates
+the distance back to the item required. For example <-1> would
+indicate the item on the partial Viterbi path at the last frame. See
+\Ref{Examples}.
+
+<formalpara>
+<para><title>Pruning</title></para>
+
+<para>
+Three types of pruning are available to reduce the size of the search
+space and therefore speed up the search:
+
+<itemizedlist>
+<listitem><para>Observation pruning</para></listitem>
+<listitem><para>Top-N pruning at each frame</para></listitem>
+<listitem><para>Fixed width beam pruning</para></listitem>
+</itemizedlist>
+
+</para>
+</formalpara>
+
+*/
+
+//@}
+
+/**@name Options
+  */
+//@{
+
+//@options
+
+
+//@}
+
 int main(int argc, char **argv)
 {
     EST_StrList files;
@@ -108,19 +179,14 @@ int main(int argc, char **argv)
     double floor; // a temporary
 
     parse_command_line(argc, argv, 
-       EST_String("Usage:\n")+
-       "viterbi  <options>\n"+
-       "Find the best path through a set of candidates.\n"+
+       EST_String("[observations file] -o [output file]\n")+
+       "Summary: find the most likely path through a sequence of\n"+
+       "         observations, constrained by a language model.\n"+
        "-ngram <string>     Grammar file, required\n"+
        "-given <string>     ngram left contexts, per frame\n"+
-       "-observes <string>  Observations file, Ascii Track, required\n"+
-       "                    One line per frame, probabilities in\n"+
-       "                    order of vocab.\n"+
        "-vocab <string>     File with names of vocabulary, this\n"+
        "                    must be same number as width of observations, required\n"+
-       "-output <string>    File to save output to, if unspecified\n"+
-       "                    output goes to stdout\n"+
-       "-ob_type <string>   Observation type : \"probs\" or \"logs\" (default is \"logs\")\n"+
+       "-ob_type <string>   Observation type : likelihood .... and change doc\"probs\" or \"logs\" (default is \"logs\")\n"+
        "\nFloor values and scaling (scaling is applied after floor value)\n"+
        "-lm_floor <float>   LM floor probability\n"+
        "-lm_scale <float>   LM scale factor factor (applied to log prob)\n"+
@@ -136,7 +202,7 @@ int main(int argc, char **argv)
 			SENTENCE_END_MARKER+" and "+SENTENCE_END_MARKER+"\n"+
        "                 respectively\n\n"+
 
-       "-observes2 <string> second observations (overlays first, ob_type must be same)\n"+
+       "-observes2  <string> second observations (overlays first, ob_type must be same)\n"+
        "-ob_floor2 <float>  \n"+
        "-ob_scale2 <float>  \n\n"+
        "-ob_prune  <float> observation pruning beam width (log) probability\n"+
@@ -145,6 +211,17 @@ int main(int argc, char **argv)
        "-trace             show details of search as it proceeds\n",
 			files, al);
 
+    out_file = al.present("-o") ? al.val("-o") : (EST_String)"-";
+
+    if (files.length() != 1)
+      {
+	cerr << argv[0];
+	cerr << ": you must give exactly one observations file on the command line";
+	cerr << endl;
+	cerr << "(use -observes2 for optional second observations)" << endl;
+	exit(-1);
+      }
+
     if (al.present("-ngram"))
     {
 	ngram.load(al.val("-ngram"));
@@ -155,20 +232,17 @@ int main(int argc, char **argv)
 	exit(-1);
     }
 
-    if (al.present("-observes") &&
-	al.present("-vocab"))
-    {
-	load_wstream(al.val("-observes"),al.val("-vocab"),wstream,observations);
-	if (al.present("-observes2"))
-	{
-	    load_wstream(al.val("-observes2"),al.val("-vocab"),wstream,observations2);
-	    num_obs = 2;
-	}
-    }
-    else
-    {
-	cerr << argv[0] << ": no observations or vocab file specified" << endl;
+    if(!al.present("-vocab"))
+      {
+	cerr << "You must provide a vocabulary file !" << endl;
 	exit(-1);
+      }
+
+    load_wstream(files.first(),al.val("-vocab"),wstream,observations);
+    if (al.present("-observes2"))
+    {
+	load_wstream(al.val("-observes2"),al.val("-vocab"),wstream,observations2);
+	num_obs = 2;
     }
 
     if (al.present("-given"))
@@ -177,11 +251,6 @@ int main(int argc, char **argv)
 	using_given=TRUE;
     }
 
-    if (al.present("-output"))
-	outfile = al.val("-output");
-    else
-	outfile = "";
-
     if (al.present("-lm_scale"))
 	lm_scale = al.fval("-lm_scale");
     else
@@ -310,22 +379,22 @@ static void print_results(EST_Relation &wstream)
     EST_String predict;
     FILE *fd;
 
-    if (outfile == "")
+    if (out_file == "-")
 	fd = stdout;
-    else if ((fd = fopen(outfile,"wb")) == NULL)
+    else if ((fd = fopen(out_file,"wb")) == NULL)
     {
-	cerr << "can't open \"" << outfile << "\" for output" << endl;
+	cerr << "can't open \"" << out_file << "\" for output" << endl;
 	exit(-1);
     }
 
-    for (s=wstream.head(); s != 0 ; s=next(s))
+    for (s=wstream.head(); s != 0 ; s=inext(s))
     {
 	predict = s->f("best").string();
 	pscore = s->f("best_score");
 	fprintf(fd,"%s %f\n",(const char *)predict,pscore);
     }
 
-    if (outfile != "")
+    if (out_file != "")
 	fclose(fd);
 
 }
@@ -335,7 +404,7 @@ static bool do_search(EST_Relation &wstream)
     // Apply Ngram to matrix of probs 
     int states;
 
-    states = ngram.states();
+    states = ngram.num_states();
     EST_Viterbi_Decoder vc(vit_candlist,vit_npath,states);
 
     vc.initialise(&wstream);
@@ -382,8 +451,13 @@ static void load_wstream(const EST_String &filename,
     }
 	
     if(w.empty())
+    {
 	for (i=0; i < obs.num_frames(); i++)
+        {
 	    add_word(w,itoString(i),i);
+        }
+
+    }
 }
 
 
@@ -392,7 +466,7 @@ static void load_given(const EST_String &filename,
 {
 
     EST_String word, pos;
-    EST_TBI *p;
+    EST_Litem *p;
     int i,j;
 
     if (load_TList_of_StrVector(given,filename,ngram_order-1) != 0)
@@ -402,7 +476,7 @@ static void load_given(const EST_String &filename,
     }
 
     // set max history
-    for (p = given.head(); p; p = next(p))
+    for (p = given.head(); p; p = p->next())
     {
 	for(i=0;i<given(p).length();i++)
 	    if(	is_a_special( given(p)(i), j) && (-j > max_history))
@@ -424,8 +498,12 @@ static void load_vocab(const EST_String &vfile)
     }
 
     while (!ts.eof())
+    {
 	if (ts.peek() != "")
+        {
 	    vocab.append(ts.get().string());
+        }
+    }
 
     ts.close();
 }
@@ -435,21 +513,22 @@ static void add_word(EST_Relation &w, const EST_String &word, int pos)
     EST_Item *item = w.append();
     
     item->set_name(word);
-    item->fset("pos",pos);
+    item->set("pos",pos);
 } 
 
-static EST_VTCandidate *vit_candlist(EST_Item &s)
+static EST_VTCandidate *vit_candlist(EST_Item *s,EST_Features &f)
 {
     // Return a list of new candidates from this point 
     double prob=1.0,prob2=1.0;
     int i;
-    EST_TBI *p;
+    EST_Litem *p;
     int observe;
     EST_VTCandidate *all_c = 0;
     EST_VTCandidate *c;
+    (void)f;
 
-    observe = s.f("pos");  // index for observations TRACK
-    for (i=0,p=vocab.head(); i < observations.num_channels(); i++,p=next(p))
+    observe = s->f("pos");  // index for observations TRACK
+    for (i=0,p=vocab.head(); i < observations.num_channels(); i++,p=p->next())
     {
 	c = new EST_VTCandidate;
 	c->name = vocab(p);  // to be more efficient this could be the index
@@ -487,7 +566,7 @@ static EST_VTCandidate *vit_candlist(EST_Item &s)
 	    c->score = prob;
 
 	c->next = all_c;
-	c->s = &s;
+	c->s = s;
 	all_c = c;
     }
 
@@ -500,13 +579,15 @@ static EST_VTCandidate *vit_candlist(EST_Item &s)
     return all_c;
 }
 
-static EST_VTPath *vit_npath(EST_VTPath *p,EST_VTCandidate *c)
+static EST_VTPath *vit_npath(EST_VTPath *p,EST_VTCandidate *c,
+			     EST_Features &f)
 {
     // Build a (potential) new path link from this previous path and 
     // This candidate 
     EST_VTPath *np = new EST_VTPath;
     double lprob,prob;
     EST_String prev,ttt;
+    (void)f;
 
     np->c = c;
     np->from = p;
@@ -519,7 +600,6 @@ static EST_VTPath *vit_npath(EST_VTPath *p,EST_VTCandidate *c)
     else
 	prob = find_gram_prob(np,&np->state);
 
-
     lprob = safe_log10(prob);
     if (lprob < lm_log_prob_floor)
 	lprob =	lm_log_prob_floor;
@@ -735,11 +815,13 @@ static void top_n_candidates(EST_VTCandidate* &all_c)
 	    // best was head of list
 	    all_c = this_best->next;
 	else
+        {
 	    // best was not head of all_c
 	    prev_to_best->next = this_best->next;
 
 	    this_best->next = top_c;
 	    top_c = this_best;
+        }
     }
 
     delete all_c;
@@ -751,3 +833,28 @@ static void top_n_candidates(EST_VTCandidate* &all_c)
 	cerr << p->score << endl;
 */
 }
+
+
+/**@name Examples
+
+Example 'given' file (items f and g are in the vocabulary), the ngram
+is a 4-gram.
+
+<para>
+<screen>
+<-2> g g
+<-1> g f
+<-1> f g
+<-2> g g
+<-3> g g
+<-1> g f
+</screen>
+</para>
+
+*/
+//@{
+//@}
+
+
+
+//@}
