@@ -5,15 +5,19 @@
 /*                    Copyright (c) 1994,1995,1996                       */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -33,11 +37,12 @@
 /*                                                                       */
 /*=======================================================================*/
 
-#include <math.h>
-//#include <iostream.h>
-//#include <fstream.h>
+#include <cmath>
+//#include <iostream>
+//#include <fstream>
 #include "sigpr/EST_fft.h"
 #include "EST_math.h"
+#include "EST_error.h"
 
 #define PI8 0.392699081698724 /* PI / 8.0 */
 #define RT2 1.4142135623731  /* sqrt(2.0) */
@@ -81,14 +86,17 @@ static int slowFFTsub(EST_FVector &real, EST_FVector &imag, float f)
     int i,j,k,l;
     
     M = fastlog2(real.n());
-    N = (int)pow(2,(float)M);
+    N = (int)pow(float(2.0),(float)M);
 
     if (N != real.n())
+      {
+	EST_warning("Illegal FFT order %d", real.n());
 	return -1;
+      }
 
     for(l=1;l<=M;l++){
 
-	int le = (int)pow(2,(float)(M+1-l));
+      int le = (int)pow(float(2.0),(float)(M+1-l));
 	int le1=le/2;
 
 	u_real = 1.0;
@@ -102,17 +110,17 @@ static int slowFFTsub(EST_FVector &real, EST_FVector &imag, float f)
 	    for (i=j;i<=N-le1;i+=le)
 	    {
 		int ip=i+le1;
-		t_real = real(i-1) + real(ip-1);
-		t_imag = imag(i-1) + imag(ip-1);
+		t_real = real.a_no_check(i-1) + real.a_no_check(ip-1);
+		t_imag = imag.a_no_check(i-1) + imag.a_no_check(ip-1);
 
-		tmp_real = real(i-1) - real(ip-1);
-		tmp_imag = imag(i-1) - imag(ip-1);
+		tmp_real = real.a_no_check(i-1) - real.a_no_check(ip-1);
+		tmp_imag = imag.a_no_check(i-1) - imag.a_no_check(ip-1);
 
-		real[ip-1] = tmp_real*u_real - tmp_imag*u_imag;
-		imag[ip-1] = tmp_real*u_imag + tmp_imag*u_real;
+		real.a_no_check(ip-1) = tmp_real*u_real - tmp_imag*u_imag;
+		imag.a_no_check(ip-1) = tmp_real*u_imag + tmp_imag*u_real;
 
-		real[i-1] = t_real;
-		imag[i-1] = t_imag;
+		real.a_no_check(i-1) = t_real;
+		imag.a_no_check(i-1) = t_imag;
 	    }
 
 	    tmp_real = u_real*w_real - u_imag*w_imag;
@@ -185,9 +193,10 @@ int slowIFFT(EST_FVector &real, EST_FVector &imag)
     return 0;
 }
 
+
 int energy_spectrum(EST_FVector &real, EST_FVector &imag)
 {
-    if (slowFFT(real,imag) != 0)
+    if (slowFFT(real, imag) != 0)
 	return -1;
 
     int i;
@@ -197,8 +206,9 @@ int energy_spectrum(EST_FVector &real, EST_FVector &imag)
        return 0;
 }
 
-int power_spectrum(EST_FVector &real, EST_FVector &imag)
+int power_spectrum_slow(EST_FVector &real, EST_FVector &imag)
 {
+
     if (slowFFT(real,imag) != 0)
 	return -1;
 
@@ -209,17 +219,26 @@ int power_spectrum(EST_FVector &real, EST_FVector &imag)
        return 0;
 }
 
-// the following code is not by Simon King, as you can see
-/******************************************************************************
-*                                                                             *
-*       Copyright (C) 1993,1994 Tony Robinson				      *
-*                                                                             *
-*       See the file SLICENSE for conditions on distribution and usage	      *
-*                                                                             *
-******************************************************************************/
+int power_spectrum(EST_FVector &real, EST_FVector &imag)
+{
 
+    if (fastFFT(real) == 0)
+	return -1;
+
+    int i,j,k;
+    int n=real.n();
+    for(i=0,j=0, k=1;i<n;i+=2,j++,k+=2)
+	real.a_no_check(j) 
+	  = imag.a_no_check(j) 
+	  = sqrt((real.a_no_check(i)*real.a_no_check(i) 
+		  + real.a_no_check(k)*real.a_no_check(k)));
+    
+       return 0;
+}
+
+// the following code is not by Simon King, as you can see
 /*
-** EST_Discrete Fourier analysis routine
+** Discrete Fourier analysis routine
 ** from IEEE Programs for Digital Signal Processing
 ** G. D. Bergland and M. T. Dolan, original authors
 ** Translated from the FORTRAN with some changes by Paul Kube
@@ -237,8 +256,7 @@ int power_spectrum(EST_FVector &real, EST_FVector &imag)
 int fastFFT(EST_FVector &invec) 
 {
     // Tony Robinsons
-    float fn;
-    int i, in, nn, n2pow, n4pow, nthpo;
+    int i, in, nn, n2pow, n4pow;
     
     // we could modify all the code to use vector classes ....
     // ... or we could do this:
@@ -248,14 +266,16 @@ int fastFFT(EST_FVector &invec)
 
     // quick fix
     int n=invec.n(); // order
+
+#if 0    
     float *b = new float[n];
     for(i=0; i<n; i++)
 	b[i] = invec(i);
+#endif
+    float *b=invec.memory();
 
     n2pow = fastlog2(n);
     if (n2pow <= 0) return 0;
-    nthpo = n;
-    fn = nthpo;
     n4pow = n2pow / 2;
     
     /* radix 2 iteration required; do it now */  
@@ -280,10 +300,12 @@ int fastFFT(EST_FVector &invec)
     
     /* take conjugates */
     for(i = 3; i < n; i += 2) b[i] = -b[i];
-    
+
+#if 0    
     // copy back
     for(i=0; i<n; i++)
 	invec[i] = b[i];
+#endif
     
     return 1;
 }
