@@ -5,15 +5,19 @@
 /*                    Copyright (c) 1994,1995,1996                       */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -32,7 +36,7 @@
 /*                         Filter functions                              */
 /*                                                                       */
 /*=======================================================================*/
-#include <stdlib.h>
+#include <cstdlib>
 #include "EST_math.h"
 #include "sigpr/EST_filter.h"
 #include "sigpr/EST_fft.h"
@@ -41,7 +45,7 @@
 #include "sigpr/EST_Window.h"
 #include "EST_error.h"
 
-// there are multiple possible styles of this becase of different
+// there are multiple possible styles of this because of different
 // possibilities of where to change the filter coefficients.
 
 void lpc_filter(EST_Wave &sig, EST_FVector &a, EST_Wave &res)
@@ -64,11 +68,18 @@ void inv_lpc_filter(EST_Wave &sig, EST_FVector &a, EST_Wave &res)
     int i, j;
     double r;
 
-    for (i = 0; i < sig.num_samples(); ++i)
+    for (i = 0; i < a.n(); ++i)
     {
-	r = sig.a(i);
+	r = sig.a_no_check(i);
 	for (j = 1; j < a.n(); ++j)
-	    r -= a(j) * (float)sig.a_safe(i - j);
+	    r -= a.a_no_check(j) * (float)sig.a_safe(i - j);
+	res.a(i) = (short) r;
+    }
+    for (i = a.n(); i < sig.num_samples(); ++i)
+    {
+	r = sig.a_no_check(i);
+	for (j = 1; j < a.n(); ++j)
+	    r -= a.a_no_check(j) * (float)sig.a_no_check(i - j);
 	res.a(i) = (short) r;
     }
 }
@@ -93,7 +104,6 @@ void inv_lpc_filter_ola(EST_Wave &in_sig, EST_Track &lpc, EST_Wave &out_sig)
     EST_FVector filter;
     EST_FVector window_vals;
     EST_Wave in_sub, out_sub;
-//    order = lpc.num_channels() - 1;
     
     // copy attributes and size to waveform, fill with zeros.
     out_sig.resize(in_sig.num_samples(), 1);
@@ -104,20 +114,33 @@ void inv_lpc_filter_ola(EST_Wave &in_sig, EST_Track &lpc, EST_Wave &out_sig)
     {
 	start = (int)((float)lpc.t(i - 1) * in_sig.sample_rate());
 	end = (int)((float)lpc.t(i + 1) * in_sig.sample_rate());
+	if (end > out_sig.num_samples())
+	    end = out_sig.num_samples();
 	size = end - start;
 
 	lpc.frame(filter, i);
 	
+	if (size < filter.n())  
+	    break; // basically a mismatch between lpc and waveform
+
 	in_sig.sub_wave(in_sub, start, size);
 	out_sub.resize(size);
 
 	inv_lpc_filter(in_sub, filter, out_sub);
 
-	EST_Window::make_window(window_vals, size, "hanning");
+
+	int centreIndex = (int)(lpc.t(i) * (float)in_sig.sample_rate());
+
+	EST_Window::make_window(window_vals, size, "hanning", centreIndex-start);
+
+	//	printf( "%d %d %d (start centre end)\n", start, centreIndex, end );
 
 	// overlap and add using hanning window on original
-	for (k = 0, j = start; j < end; ++j, ++k)
-	    out_sig.a(j) += (int)((float)out_sub(k) * window_vals(k));
+	for (k = 0, j = start; j < end; ++j, ++k){
+	  out_sig.a_no_check(j) += 
+	    (int)((float)out_sub.a_no_check(k) * 
+		  window_vals.a_no_check(k));
+	}
     }
 }
 
@@ -167,6 +190,8 @@ void lpc_filter_1(EST_Track &lpc, EST_Wave & res, EST_Wave &sig)
     }    
 }
 
+
+
 void lpc_filter_fast(EST_Track &lpc, EST_Wave & res, EST_Wave &sig)
 {
     // An (unfortunately) faster version of the above.  This is about
@@ -180,6 +205,7 @@ void lpc_filter_fast(EST_Track &lpc, EST_Wave & res, EST_Wave &sig)
     if (order < 0) order = 0;  // when lpc as no frames
     float *buff = walloc(float,res.num_samples()+order);
     float *filt = walloc(float,order+1);
+    short *residual = res.values().memory();
 
     sig.resize(res.num_samples(),1,0);  // no reseting of values
     sig.set_sample_rate(res.sample_rate());
@@ -199,38 +225,70 @@ void lpc_filter_fast(EST_Track &lpc, EST_Wave & res, EST_Wave &sig)
 	    for (j = 1; j < n; ++j)
 		s += filt[j] * buff[k-j];
 	    // The 0.5 should be a parameter
-	    buff[k] = s + (res.a_no_check(m)*0.5);
+	    //	    buff[k] = s + (residual[m]*0.5);
+	    buff[k] = s + residual[m];
 	}
 	start = end;
     }
+    short *signal = sig.values().memory();
     for (j=0,i=order; i < k; j++,i++)
-	sig.a_no_check(j) = (short)buff[i];
+	signal[j] = (short)buff[i];
     wfree(buff);
     wfree(filt);
 }
 
-// Non-copy version
 void post_emphasis(EST_Wave &sig, float a)
 {
     double last=0;
     
-    for (int i=0; i < sig.num_samples(); i++)
-    {
-	sig.a(i) = (int)((float)sig.a(i) + a * last);
-	last = (float)sig(i);
+    for (int j = 0; j < sig.num_channels(); ++j)
+	for (int i = 0; i < sig.num_samples(); i++)
+	{
+	    sig.a(i, j) = (int)((float)sig.a(i, j) + a * last);
+	    last = (float)sig(i, j);
 	//	if (absval(sig(i) > maxval)
 	//	    maxval = absval(fddata[i]);
-    }
+	}
 }
 
 void pre_emphasis(EST_Wave &sig, float a)
 {
-    float last = 0.0;
+    float x = 0.0;
+    float x_1 = 0.0;
 
-    for (int i = 0; i < sig.num_samples(); i++)
+    for (int j = 0; j < sig.num_channels(); ++j)
+	for (int i = 0; i < sig.num_samples(); i++)
+	{
+	  x = sig.a_no_check(i, j);
+	  sig.a_no_check(i, j) = 
+	    sig.a_no_check(i, j) - int(a * x_1);
+	  x_1 = x;
+	}
+}
+
+void pre_emphasis(EST_Wave &sig, EST_Wave &out, float a)
+{
+    out.resize(sig.num_samples(), sig.num_channels());
+
+    for (int j = 0; j < sig.num_channels(); ++j)
     {
-	sig.a(i) = sig.a(i) - int(a * last);
-	last = (float)sig.a(i);
+	out.a_no_check(0, j) = sig.a_no_check(0, j);
+	for (int i = 1; i < sig.num_samples(); i++)
+	    out.a_no_check(i, j) = 
+		sig.a_no_check(i, j) - int(a * (float)sig.a_no_check(i-1, j));
+    }
+}
+
+void post_emphasis(EST_Wave &sig, EST_Wave &out, float a)
+{
+    out.resize(sig.num_samples(), sig.num_channels());
+
+    for (int j = 0; j < sig.num_channels(); ++j)
+    {
+	out.a_no_check(0, j) = sig.a_no_check(0, j);
+	for (int i = 1; i < sig.num_samples(); i++)
+	    out.a_no_check(i, j) = 
+		sig.a_no_check(i, j) + int(a * (float)sig.a_no_check(i-1, j));
     }
 }
 
@@ -241,12 +299,12 @@ void simple_mean_smooth(EST_Wave &c, int n)
     float sum;
     h = n/2;
     
-    for (i = 0; i < h; ++i)
+    for (i = 0; (i < h); ++i)
     {
 	k = (i * 2) + 1;
 	sum = 0.0;
-	for (j = 0; j < k; ++j)
-	    sum += c.a(j);
+	for (j = 0; (j < k) && (k < c.num_samples()); ++j)
+	    sum += c.a_no_check(j);
 	a[i] = sum /(float) k;
     }
     
@@ -254,7 +312,7 @@ void simple_mean_smooth(EST_Wave &c, int n)
     {
 	sum = 0.0;
 	for (j = 0; j < n; ++j)
-	    sum += c.a(i - h + j);
+	    sum += c.a_no_check(i - h + j);
 	a[i] = sum /(float) k;
     }
     
@@ -263,12 +321,12 @@ void simple_mean_smooth(EST_Wave &c, int n)
 	k = ((c.num_samples() - i)* 2) -1;
 	sum = 0.0;
 	for (j = 0; j < k; ++j)
-	    sum += c.a(i - (k/2) + j);
+	    sum += c.a_no_check(i - (k/2) + j);
 	a[i] = sum /(float) k;
     }
     
     for (i = 0; i < c.num_samples(); ++i)
-	c.a(i) = (short)(a[i] + 0.5);
+	c.a_no_check(i) = (short)(a[i] + 0.5);
     
     delete [] a;
 }
@@ -301,11 +359,24 @@ void FIRfilter(const EST_Wave &in_sig, EST_Wave &out_sig,
     // could speed up here with three loops :
     // 1 for first part (filter overlaps start of wave, one 'if')
     // 2 for middle part (filter always within wave, no 'if's)
-    // 3 for last part (filter ovelaps end of wave, one 'if')
+    // 3 for last part (filter overlaps end of wave, one 'if')
+
+    // To make this faster do the float conversion once and hold it
+    // in a conventional array.  Note this has been checked to be 
+    // safe but if you change the code below you'll have to confirm it
+    // remains safe. If access through FVector was fast I'd use them
+    // but this is about twice as fast.
+    float *in = walloc(float,n);
+    for (i=0; i < n; ++i)
+	in[i] = (float)in_sig.a_no_check(i);
+    float *numer = walloc(float,numerator.n());
+    for (i=0; i < numerator.n(); ++i)
+	numer[i] = numerator.a_no_check(i);
+    float *out = walloc(float,n);
 
     for (i = 0; i < n; ++i)
     {
-	out_sig.a(i) = 0;
+	out[i] = 0;
 
 	int jlow=0;
 	int jhigh=numerator.n();
@@ -319,14 +390,28 @@ void FIRfilter(const EST_Wave &in_sig, EST_Wave &out_sig,
 	for(j=jlow; j<jhigh; j++)
 	    if (((i+delay_correction - j) >= 0) &&
 		((i+delay_correction - j) < n))
-		out_sig.a(i) += (int)( (float)(in_sig.a(i+delay_correction - j))
-				     * numerator(j));
+		out[i] += in[i+delay_correction - j] * numer[j];
     }
+
+    for (i=0; i<n; i++)
+	out_sig.a_no_check(i) = (short)out[i];
     out_sig.set_sample_rate(in_sig.sample_rate());
     out_sig.set_file_type(in_sig.file_type());
+
+    wfree(in);
+    wfree(numer);
+    wfree(out);
 }
 
-
+void FIR_double_filter(EST_Wave &in_sig, EST_Wave &out_sig, 
+		       const EST_FVector &numerator)
+{
+    out_sig = in_sig;
+    FIRfilter(out_sig, numerator, 0);
+    reverse(out_sig);
+    FIRfilter(out_sig, numerator, 0);
+    reverse(out_sig);
+}
 
 EST_FVector design_FIR_filter(const EST_FVector &frequency_response, 
 			      int filter_order)
@@ -342,7 +427,7 @@ EST_FVector design_FIR_filter(const EST_FVector &frequency_response,
 	
     // check frequency_response has dimension 2^N
     int N = fastlog2(frequency_response.n());
-    if(frequency_response.n() !=  (int)pow(2,(float)N)){
+    if(frequency_response.n() !=  (int)pow(float(2.0),(float)N)){
 	cerr << "Desired frequency response must have dimension 2^N" << endl;
 	return EST_FVector(0);
     }
@@ -393,10 +478,10 @@ EST_FVector design_high_or_low_pass_FIR_filter(int sample_rate,
     int i;
     int N=10;			// good minimum size
     
-    int fft_size = (int)pow(2, N);
+    int fft_size = (int)pow(float(2.0), float(N));
     while(fft_size < order*4){	// rule of thumb !?
 	N++;
-	fft_size = (int)pow(2, N);
+	fft_size = (int)pow(float(2.0), float(N));
     }
     
     // freq response is from 0 to sampling freq and therefore
