@@ -5,15 +5,19 @@
 /*                         Copyright (c) 1996                            */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -34,7 +38,6 @@
 /*=======================================================================*/
 
 #include "sigpr/EST_sigpr_frame.h"
-#include "sigpr/EST_sig2fv.h"
 #include "sigpr/EST_fft.h"
 #include "EST_inline_utils.h"
 #include "EST_math.h"
@@ -52,13 +55,14 @@ float lpredict(float *adc, int wsize,
 		      int order);
 float ogi_lpc(float *adc, int wsize, int order,
 		      float *acf, float *ref, float *lpc);
-
+/*
 void lpc2ref(float const *a, float *b, int c)
 {
     (void) a;
     (void) b;
     (void) c;
 }
+*/
 
 void convert2lpc(const EST_FVector &in_frame, const EST_String &in_type,
 		 EST_FVector &out_frame)
@@ -203,9 +207,9 @@ void frame_convert(const EST_FVector &in_frame, const EST_String &in_type,
   if (out_type == "lpc")
       convert2lpc(in_frame, in_type, out_frame);
   else if (out_type == "lsf")
-      convert2lpc(in_frame, in_type, out_frame);
+      convert2lsf(in_frame, in_type, out_frame);
   else if (out_type == "ref")
-      convert2lpc(in_frame, in_type, out_frame);
+      convert2ref(in_frame, in_type, out_frame);
   else if (out_type == "cep")
       convert2cep(in_frame, in_type, out_frame);
   else if (out_type == "area")
@@ -310,7 +314,7 @@ void lpc2cep(const EST_FVector &lpc, EST_FVector &cep)
     for (n = 0; n < cep.length(); n++) 
     {
 	// check if NaN -- happens on some frames of silence
-	if (isnan(cep[n]) ) cep[n] = 0.0;
+	if (isnanf(cep[n]) ) cep[n] = 0.0;
 	
 	if (cep[n] >  MAX_ABS_CEPS){
 	    cerr << "WARNING : cepstral coeff " << n << " was " << 
@@ -330,10 +334,19 @@ void lpc2cep(const EST_FVector &lpc, EST_FVector &cep)
 // REORG - test this!!
 void lpc2ref(const EST_FVector &lpc, EST_FVector &ref)
 {
-    // LPC to reflection coefficients 
-    // from code from Borja Etxebarria
-    // This code does clever things with pointer and so has been
-    // left using float * arrays.
+
+  // seem to get weird output from this - best not to use it !
+  EST_error("lpc2ref Code unfinished\n");
+
+  // LPC to reflection coefficients 
+  // from code from Borja Etxebarria
+  // This code does clever things with pointer and so has been
+  // left using float * arrays.
+
+  // simonk (May 99) : fixed because lpc coeffs always have energy at
+  // coeff 0 - the code here would need changing is lpc coeff 0 was
+  // ever made optional.
+  int lpc_offset=1;
 
     int order = lpc.length() - 1;
     int i,j;
@@ -342,20 +355,20 @@ void lpc2ref(const EST_FVector &lpc, EST_FVector &ref)
     float *vn = new float[order];
     
     i = order - 1;
-    ref[i] = lpc(i);
-    ai = lpc(i);
+    ref[i] = lpc(i+lpc_offset);
+    ai = lpc(i+lpc_offset);
     f = 1-ai*ai;
     i--;
     
     for (j=0; j<=i; j++)
-	ref[j] = (lpc(j)+((ai*lpc(i-j))))/f;
+	ref[j] = (lpc(j+lpc_offset)+((ai*lpc(i-j+lpc_offset))))/f;
     
     /* vn=vtmp in previous #define */
     // Check whether this should really be a pointer
     vo = new float[order];
     for (i = 0; i < order; ++i)
 	vo[i] = ref(i);
-    
+
     for ( ;i>0; ) 
     {
 	ai=vo[i];
@@ -376,8 +389,8 @@ void lpc2ref(const EST_FVector &lpc, EST_FVector &ref)
 
 void ref2lpc(const EST_FVector &ref, EST_FVector &lpc)
 {
-    // Here we use Christopher Longet Higgin's alrgorihm converted to 
-    // an equivalent by awb.  Its doesn't have the reverse order or
+    // Here we use Christopher Longet Higgin's algorithm converted to 
+    // an equivalent by awb. It doesn't have the reverse order or
     // negation requirement.
     int order = ref.length() - 1;
     float a, b;
@@ -419,38 +432,50 @@ void lsf2lpc(const EST_FVector &lpc,  EST_FVector &lsf)
 void sig2lpc(const EST_FVector &sig, EST_FVector &acf, 
 		EST_FVector &ref, EST_FVector &lpc)
 {
+
     int i, j;
     float e, ci, sum;
     int order = lpc.length() -1;
     EST_FVector tmp(order);
     int stableorder=-1;
+
+    if ((acf.length() != ref.length()) ||
+	(acf.length() != lpc.length()))
+	EST_error("sig2lpc: acf, ref are not of lpc's order");
+
+    //cerr << "sig2lpc order " << order << endl;
+
     
     for (i = 0; i <= order; i++) 
     {
 	sum = 0.0;
 	for(j = 0; j < sig.length() - i; j++) 
-	    sum += sig(j) * sig(j + i);
-	acf[i] = sum;
+	    sum += sig.a_no_check(j) * sig.a_no_check(j + i);
+	acf.a_no_check(i) = sum;
     }
     
     // find lpc coefficients 
-    e = acf(0);
-    lpc[0] = 1.0;
+    e = acf.a_no_check(0);
+    lpc.a_no_check(0) = 1.0;
 
     for (i = 1; i <= order; i++) 
     {
 	ci = 0.0;
 	for(j = 1; j < i; j++) 
-	    ci += lpc[j] * acf(i-j);
-	ref[i] = ci = (acf(i) - ci) / e;
+	    ci += lpc.a_no_check(j) * acf.a_no_check(i-j);
+	if (e == 0)
+	    ref.a_no_check(i) = ci = 0.0;
+	else
+	    ref.a_no_check(i) = ci = (acf.a_no_check(i) - ci) / e;
 	//Check stability of the recursion
 	if (absval(ci) < 1.000000) 
 	{
-	    lpc[i] = ci;
+	    lpc.a_no_check(i) = ci;
 	    for (j = 1; j < i; j++) 
-		tmp[j] = lpc[j] - ci * lpc[i-j];
+		tmp.a_no_check(j) = lpc.a_no_check(j) - 
+		    (ci * lpc.a_no_check(i-j));
 	    for( j = 1; j < i; j++) 
-		lpc[j] = tmp[j];
+		lpc.a_no_check(j) = tmp.a_no_check(j);
 
 	    e = (1 - ci * ci) * e;
 	    stableorder = i;
@@ -463,17 +488,18 @@ void sig2lpc(const EST_FVector &sig, EST_FVector &acf,
 		"warning:levinson instability, order restricted to %d\n",
 		stableorder);
 	for (; i <= order; i++)
-	    lpc[i] = 0.0;
+	    lpc.a_no_check(i) = 0.0;
     }
 
-    lpc[0] = e;
+    // normalisation for frame length
+    lpc.a_no_check(0) = e / sig.length();
 }
 
 void sig2pow(EST_FVector &frame, float &power)
 {
     power = 0.0;
     for (int i = 0; i < frame.length(); i++)
-	power += pow(frame(i), 2.0);
+      power += pow(frame(i), float(2.0));
 
     power /= frame.length();
 }
@@ -583,7 +609,7 @@ void sig2fbank(const EST_FVector &sig,
     
     if(take_log)
 	for(i=0;i<fbank_frame.length();i++)
-	    fbank_frame[i] = log(fbank_frame[i]);
+	    fbank_frame[i] = safe_log(fbank_frame[i]);
 
 }
 
@@ -686,8 +712,6 @@ void fbank2melcep(const EST_FVector &fbank_vec,
 		  const float liftering_parameter,
 		  const bool include_c0)
 {
-    
-
     // a cosine transform of the fbank output
     // remember to pass LOG fbank params (energy or power)
 
@@ -708,7 +732,6 @@ void fbank2melcep(const EST_FVector &fbank_vec,
 
     for(i=0;i<mfcc_vec.length();i++)
     {
-
 	actual_mfcc_index = include_c0 ? i : i+1;
 
 	pi_i_over_N  = 
