@@ -5,15 +5,19 @@
 /*                         Copyright (c) 1996                            */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -33,50 +37,29 @@
 /*                                                                       */
 /*=======================================================================*/
 
-#include <iostream.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <math.h>
-#include <string.h>
+#include <iostream>
+#include <cstdlib>
+#include <cstdio>
+#include <cmath>
+#include <cstring>
 #include "EST_cutils.h"
 #include "EST_Wave.h"
 #include "EST_wave_utils.h"
+#include "EST_wave_aux.h"
 #include "EST_TNamedEnum.h"
 #include "EST_WaveFile.h"
 
+#include "EST_Track.h"
+
 #include "waveP.h"
 
-const EST_String DEF_FILE_TYPE = "nist";
+#define sgn(x) (x>0?1:x?-1:0)
+
+const EST_String DEF_FILE_TYPE = "riff";
 const EST_String DEF_SAMPLE_TYPE = "short";
 
 const int EST_Wave::default_sample_rate=16000;
 
-static EST_TValuedEnumDefinition<EST_sample_type_t, const char *,NO_INFO> st_names[] = {
-{st_unknown,  {"undef"}},
-{st_schar, {"schar"}},
-{st_uchar, {"uchar"}},
-{st_short, {"short"}},
-{st_shorten, {"shorten"}},
-{st_int, {"int"}},
-{st_float, {"float"}},
-{st_double, {"double"}},
-{st_mulaw, {"mulaw"}},
-{st_adpcm, {"adpcm"}},
-{st_alaw, {"alaw"}},
-{st_ascii, {"ascii"}},
-{st_unknown, {0}}
-};
-
-static EST_TNamedEnum<EST_sample_type_t> sample_type_map(st_names);
-
-#if defined(INSTANTIATE_TEMPLATES)
-#include "../base_class/EST_TNamedEnum.cc"
-template class EST_TNamedEnum<EST_sample_type_t>; 
-template class EST_TNamedEnumI<EST_sample_type_t, NO_INFO>; 
-template class EST_TValuedEnum<EST_sample_type_t,const char *>; 
-template class EST_TValuedEnumI<EST_sample_type_t,const char *, NO_INFO>; 
-#endif
-
 EST_Wave::EST_Wave()
 {
     default_vals();
@@ -105,27 +88,29 @@ EST_Wave::EST_Wave(int samps, int chans,
 
 void EST_Wave::default_vals(int n, int c) 
 {
-    // real defaults
+  // real defaults
   p_values.resize(n,c);
-
   p_sample_rate = default_sample_rate;
-  p_file_type = "nist";
-  p_name = "";
+
+  init_features();
 }
 
 void EST_Wave::free_wave()
 {
+  if (!p_values.p_sub_matrix)
     p_values.resize(0,0);
+  clear_features();
 }
 
 EST_Wave::~EST_Wave()
 {
+  free_wave();
 }
 
 void EST_Wave::copy_setup(const EST_Wave &w)
 {
     p_sample_rate = w.p_sample_rate;
-    p_name = w.p_name;
+    copy_features(w);
 }
 
 void EST_Wave::copy_data(const EST_Wave &w)
@@ -181,7 +166,12 @@ short &EST_Wave::a_safe(int i, int channel)
 void EST_Wave::fill(short v, int channel)
 {
     if (channel == EST_ALL)
-      p_values.fill(v);
+    {
+	if (v == 0)  // this is *much* more efficient and common
+	    memset(values().memory(),0,num_samples()*num_channels()*2);
+        else
+	    p_values.fill(v);
+    }
     else 
       for (int i = 0; i < num_samples(); ++i)
 	p_values.a_no_check(i,channel) = v;
@@ -239,6 +229,8 @@ EST_read_status EST_Wave::load(EST_TokenStream &ts,
 	    set_file_type(EST_WaveFile::map.value(t));
 	    break;
 	}
+	else if (stat == read_error)
+	    break;
     }
     
     return stat;
@@ -322,7 +314,7 @@ EST_read_status EST_Wave::load_file(EST_TokenStream &ts,
 {
   EST_WaveFileType t = EST_WaveFile::map.token(type);
 
-  EST_sample_type_t values_type = sample_type_map.token(stype);
+  EST_sample_type_t values_type = EST_sample_type_map.token(stype);
 
   if (t == wff_none)
     {
@@ -426,7 +418,7 @@ EST_write_status EST_Wave::save_file(FILE *fp,
 				     EST_String stype, int obo)
 {
     EST_WaveFileType t = EST_WaveFile::map.token(ftype);
-    EST_sample_type_t sample_type = sample_type_map.token(stype);
+    EST_sample_type_t sample_type = EST_sample_type_map.token(stype);
 
     if (t == wff_none)
       {
@@ -448,23 +440,37 @@ EST_write_status EST_Wave::save_file(FILE *fp,
 
 void EST_Wave::resample(int new_freq)
 {
-    // Rseample wave to new sample rate
-  if (new_freq != p_sample_rate)
+    // Resample wave to new sample rate
+    if (new_freq != p_sample_rate)
     {
-      if (p_values.rateconv(p_sample_rate, new_freq) != 0)
-	{
-	  cerr << "rateconv: failed to convert\n";
-	  return;
-	}
-	set_sample_rate(new_freq);
+	if (p_values.rateconv(p_sample_rate, new_freq) != 0)
+	    cerr << "rateconv: failed to convert from " << p_sample_rate <<
+		" to " << new_freq << "\n";
+	else
+	    set_sample_rate(new_freq);
     }
     
 }
 
+void EST_Wave::compress(float mu, float lim)
+{
+    int x;
+ 
+    for (int i = 0; i < num_samples(); ++i)
+    {
+        for (int j = 0; j< num_channels(); ++j)
+        {
+            x = a_no_check(i,j);
+            a_no_check(i,j) = lim * (sgn(x)*(log(1+(mu/lim)*abs(x))/log(1+mu)));
+        }
+    }    
+}  
+
 void EST_Wave::rescale(float gain, int normalize)
 {
     int ns;
     float factor = gain;
+    float nsf;
     
     if (normalize)
     {
@@ -473,13 +479,27 @@ void EST_Wave::rescale(float gain, int normalize)
 	  for (int j = 0; j < num_channels(); ++j)
 	    if (abs(a_no_check(i,j)) > max)
 		max = abs(a_no_check(i,j));
-	factor *= 32766.0/(float)max;
+	if (fabs(max/32766.0-gain) < 0.001)
+	    return; /* already normalized */
+	else
+	    factor *= 32766.0/(float)max;
     }
     
     for (int i = 0; i < num_samples(); ++i)
       for (int j = 0; j < num_channels(); ++j)
 	{
-	  ns = (int) ((float)a_no_check(i,j) * factor + 0.5);
+            if (factor == 1.0)
+                ns = a_no_check(i,j);  // avoid float fluctuations
+            else if (factor == -1.0)
+                ns = -a_no_check(i,j);  // avoid float fluctuations
+            else
+            {
+                nsf = (float)a_no_check(i,j) * factor;
+                if (nsf < 0.0)
+                    ns = (int)(nsf - 0.5);
+                else
+                    ns = (int)(nsf + 0.5);
+            }
 	  if (ns < -32766)
 	    a_no_check(i,j)= -32766;
 	  else if (ns > 32766)
@@ -489,6 +509,58 @@ void EST_Wave::rescale(float gain, int normalize)
 	}
 }
 
+void EST_Wave::rescale( const EST_Track &fc )
+{
+  int ns, start_sample, end_sample;
+  float target1, target2, increment, factor, nsf;
+  
+  int fc_length = fc.length();
+  int _num_channels = num_channels();
+
+  cerr << ((int)(fc.t(fc_length-1) * p_sample_rate)) << endl;
+
+  if( ((int)(fc.t(fc_length-1) * p_sample_rate)) > num_samples() )
+    EST_error( "Factor contour track exceeds waveform length (%d samples)",
+		 (fc.t(fc_length-1) * p_sample_rate) - num_samples() );
+
+  start_sample = static_cast<unsigned int>( fc.t( 0 )*p_sample_rate );
+  target1 = fc.a(0,0); // could use separate channels for each waveform channel
+
+  for ( int k = 1; k<fc_length; ++k ){
+    end_sample   = static_cast<unsigned int>( fc.t( k )*p_sample_rate );
+    target2 = fc.a(k);
+    
+    increment = (target2-target1)/(end_sample-start_sample+1);
+  
+    factor = target1;
+    for( int i=start_sample; i<end_sample; ++i, factor+=increment )
+      for( int j=0; j<_num_channels; ++j ){
+            if (factor == 1.0)
+                ns = a_no_check(i,j);  // avoid float fluctuations
+            else if (factor == -1.0)
+                ns = -a_no_check(i,j);  // avoid float fluctuations
+            else
+            {
+                nsf = (float)a_no_check(i,j) * factor;
+                if (nsf < 0.0)
+                    ns = (int)(nsf - 0.5);
+                else
+                    ns = (int)(nsf + 0.5);
+            }
+          if (ns < -32766)
+              a_no_check(i,j)= -32766;
+          else if (ns > 32766)
+              a_no_check(i,j)= 32766;
+          else
+              a_no_check(i,j)= ns;
+      }
+    start_sample = end_sample;
+    target1 = target2;
+  }
+}
+
+
+
 EST_Wave &EST_Wave::operator =(const EST_Wave &w)
 {
     copy(w);
@@ -546,3 +618,8 @@ ostream& operator << (ostream& p_values, const EST_Wave &sig)
     
     return p_values;
 }
+
+int operator != (EST_Wave a, EST_Wave b)
+   { (void)a; (void)b; return 1; }
+int operator == (EST_Wave a, EST_Wave b)
+  {  (void)a; (void)b; return 0; }
