@@ -33,17 +33,17 @@
 /*                  Author :  Alan Black and Paul Taylor                 */
 /*                  Date   :  June 1996                                  */
 /*-----------------------------------------------------------------------*/
-/*   File I/O function sspecific for various file formats                */
+/*   File I/O functions specific to various file formats                 */
 /*                                                                       */
 /*   Note that internally data will always be shorts and                 */
 /*   native byte order, conversions to/from other byte formats           */
 /*   or encodings happend at read/write time                             */
 /*                                                                       */
 /*=======================================================================*/
-#include <stdlib.h> 
-#include <stdio.h>
+#include <cstdlib> 
+#include <cstdio>
 #include "EST_unix.h"
-#include <string.h>
+#include <cstring>
 #include "EST_wave_aux.h"
 #include "EST_wave_utils.h"
 #include "EST_strcasecmp.h"
@@ -68,7 +68,7 @@ static const char *NIST_END_SIG = "end_head\n";
 
 int nist_get_param_int(const char *hdr, const char *field, int def_val)
 {
-    char *p;
+    const char *p;
     int val;
 
     if (((p=strstr(hdr,field)) != NULL) &&
@@ -84,7 +84,8 @@ int nist_get_param_int(const char *hdr, const char *field, int def_val)
 
 char *nist_get_param_str(const char *hdr, const char *field, const char *def_val)
 {
-    char *p,*val;
+    const char *p;
+    char *val;
     int size;
 
     if (((p=strstr(hdr,field)) != NULL) &&
@@ -140,6 +141,8 @@ enum EST_sample_type_t nist_to_sample_type(char *type)
 	     (EST_strcasecmp(type,"mu-law",NULL) == 0) ||
 	     (EST_strcasecmp(type,"mulaw",NULL) == 0))
 	return st_mulaw;
+    else if (strcmp(type,"alaw") == 0)
+	return st_alaw;
     else if (strcmp(type,"PCM-1") == 0)
 	return st_schar;
     else if (strcmp(type,"PCM-4") == 0)
@@ -171,9 +174,9 @@ enum EST_read_status load_wave_nist(EST_TokenStream &ts, short **data, int
 
     current_pos = ts.tell();
     if (ts.fread(header,NIST_HDR_SIZE,1) != 1)
-	return misc_read_error;
+	return wrong_format;
 
-    if (strncmp(header,NIST_SIG,sizeof(NIST_SIG)) != 0)
+    if (strncmp(header,NIST_SIG,strlen(NIST_SIG)) != 0)
 	return wrong_format;
 
     samps = nist_get_param_int(header,"sample_count",-1);
@@ -194,16 +197,26 @@ enum EST_read_status load_wave_nist(EST_TokenStream &ts, short **data, int
        This is a temporary fix which calls the unshorten program on the
        speech file and reads in the answer by calling this function.
        It would be nice to have a simple library routine which did the
-       unshortening.
+       unshortening. 
+       removed Jun 14th 2016 by awb , security risk, and only works in CSTR 
        */
     
     if (streq(sample_coding,"pcm,embedded-shorten-v1.1"))
     {
+#if 0
 	char *tmpfile, *cmdstr;
 	enum EST_read_status rval;
+#endif
 
+	fprintf(stderr,"WAVE read: nist type is shorten\n");
+	fprintf(stderr,"WAVE read: no support for shorten -- you need to use some external program to unshorten the data\n");
+	return misc_read_error;
+
+#if 0
 	tmpfile = cmake_tmp_filename();
 	cmdstr = walloc(char,strlen(tmpfile)+200);
+        /* This doesn't work, unless you have cstrshorten, and */
+        /* doesn't work if your file name is interesting */
 	sprintf(cmdstr,"cstrshorten %s %s",
 		(const char*)ts.filename(),tmpfile);
 	printf("Command: %s\n", cmdstr);
@@ -219,6 +232,7 @@ enum EST_read_status load_wave_nist(EST_TokenStream &ts, short **data, int
 	wfree(cmdstr);
 	tt.close();
 	return rval;
+#endif
     }
 
     if (length == 0)
@@ -380,7 +394,7 @@ enum EST_read_status load_wave_est(EST_TokenStream &ts, short **data, int
     
     *data = convert_raw_data((unsigned char *)file_data, 
 			     data_length, actual_sample_type, actual_bo);
-    // becuase internally data is always shorts
+    // because internally data is always shorts
     *sample_type = st_short;
     *bo = EST_NATIVE_BO;
     *word_size = 2;
@@ -422,8 +436,8 @@ enum EST_write_status save_wave_est(FILE *fp, const short *data, int offset,
 /*=======================================================================*/
 #define WAVE_FORMAT_PCM    0x0001
 #define WAVE_FORMAT_ADPCM  0x0002
-#define WAVE_FORMAT_MULAW  0x0006
-#define WAVE_FORMAT_ALAW   0x0007
+#define WAVE_FORMAT_ALAW   0x0006
+#define WAVE_FORMAT_MULAW  0x0007
 
 enum EST_read_status load_wave_riff(EST_TokenStream &ts, short **data, int
 				    *num_samples, int *num_channels, int
@@ -521,9 +535,12 @@ enum EST_read_status load_wave_riff(EST_TokenStream &ts, short **data, int
 	}
 	else
 	{
-	    fprintf(stderr,"Unsupported chunk type \"%s\" in RIFF file\n",
-		    info);
-	    return misc_read_error;
+            //	    fprintf(stderr,"Ignoring unsupported chunk type \"%c%c%c%c\" in RIFF file\n",
+            //    info[0],info[1],info[2],info[3]);
+	    //return misc_read_error;
+	    ts.fread(&dsize,4,1);
+	    if (EST_BIG_ENDIAN) dsize = SWAPINT(dsize);
+	    ts.seek(dsize+ts.tell());     /* skip this chunk */
 	}
     }
     if (length == 0)
@@ -1203,9 +1220,9 @@ enum EST_write_status save_wave_audlab(FILE *fp, const short *data, int offset,
 /* Deep thanks go to Peter Kabal from McGill University whose AF code */
 /* showed me this was even possible.  I looked at his code to find    */
 /* parts I couldn't work out myself.  Also to Rodney Johnson of       */
-/* Entrophic who's document "ESPS APPLICATION NOTE: Non-ESPS Programs */
+/* Entropic whose document "ESPS APPLICATION NOTE: Non-ESPS Programs  */
 /* and the ESPS File System" gave details of how to access ESPS files */
-/* without using the ESPS library code                                */
+/* without using the ESPS library code.                               */
 
 #include "esps_utils.h"
 enum EST_read_status load_wave_sd(EST_TokenStream &ts, short **data, int
