@@ -5,15 +5,19 @@
  /*                       Copyright (c) 1996,1997                        */
  /*                        All Rights Reserved.                          */
  /*                                                                      */
- /*  Permission to use, copy, modify, distribute this software and its   */
- /*  documentation for research, educational and individual use only, is */
- /*  hereby granted without fee, subject to the following conditions:    */
+ /*  Permission is hereby granted, free of charge, to use and distribute */
+ /*  this software and its documentation without restriction, including  */
+ /*  without limitation the rights to use, copy, modify, merge, publish, */
+ /*  distribute, sublicense, and/or sell copies of this work, and to     */
+ /*  permit persons to whom this work is furnished to do so, subject to  */
+ /*  the following conditions:                                           */
  /*   1. The code must retain the above copyright notice, this list of   */
  /*      conditions and the following disclaimer.                        */
  /*   2. Any modifications must be clearly marked as such.               */
  /*   3. Original authors' names are not deleted.                        */
- /*  This software may not be used for commercial purposes without       */
- /*  specific prior written permission from the authors.                 */
+ /*   4. The authors' names are not used to endorse or promote products  */
+ /*      derived from this software without specific prior written       */
+ /*      permission.                                                     */
  /*                                                                      */
  /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK       */
  /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING     */
@@ -36,8 +40,8 @@
  /*                                                                      */
  /************************************************************************/
 
-#include <math.h>
-#include <stdlib.h>
+#include <cmath>
+#include <cstdlib>
 #include "EST_Wave.h"
 #include "EST_wave_aux.h"
 #include "EST_simplestats.h"
@@ -79,7 +83,7 @@ void meansd(EST_Wave &tr, float &mean, float &sd, int channel)
     mean /= n;
     
     for (i = 0, mean = 0.0; i < tr.num_samples(); ++i)
-	var += pow(tr.a(i, channel) - mean, 2.0);
+      var += pow(tr.a(i, channel) - mean, float(2.0));
     
     var /= n;
     sd = sqrt(var);
@@ -92,7 +96,7 @@ float rms_error(EST_Wave &a, EST_Wave &b, int channel)
     float sum = 0;
     
     for (i = 0; i < size; ++i)
-      sum += pow((a.a(i, channel) - b.a(i, channel)), 2.0);
+      sum += pow(float(a.a(i, channel) - b.a(i, channel)), float(2.0));
     
     sum = sqrt(sum / size);
     return sum;
@@ -106,7 +110,7 @@ float abs_error(EST_Wave &a, EST_Wave &b, int channel)
     for (i = 0; i < size; ++i)
     {
       // cout << i << " " << a.a(i, channel) << " " << b.a(i, channel) << endl;
-	sum += fabs(a.a(i, channel) - b.a(i, channel));
+      sum += fabs(float(a.a(i, channel) - b.a(i, channel)));
     }
     return sum / size;
 }
