@@ -5,15 +5,19 @@
 /*                   Copyright (c) 1994,1995,1996                        */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -29,17 +33,17 @@
 /*                    Author :  Alan W Black (and Paul Taylor)           */
 /*                    Date   :  June 1996                                */
 /*-----------------------------------------------------------------------*/
-/*  These routines form a basis for reading and writing Entropics ESPS   */
+/*  These routines form a basis for reading and writing Entropic's ESPS  */
 /*  headered files.  The reason we wrote them was to avoid including     */
-/*  Entropics own (large and combersome) code into all our programs      */
+/*  Entropic's own (large and cumbersome) code into all our programs.    */
 /*  No Entropic proprietary code is included in this code which means    */
 /*  you do not needs an Entropic licence to use it.                      */
 /*                                                                       */
-/*  However this should not be seen as anti-entropic in anyway it is for */
+/*  However this should not be seen as anti-Entropic in anyway it is for */
 /*  our and your convenience.  We would like to specifically thank       */
 /*  Rodney Johnson of Entropic for giving us help and confirming to us   */
 /*  the header format is in fact more complex than one can imagine,      */
-/*  mostly for very bad reasons, (backward compatability cripples all    */
+/*  mostly for very bad reasons, (backward compatibility cripples all    */
 /*  software in the long run).  Hence this code is almost definitely     */
 /*  incomplete and is not guaranteed to read or create all ESPS files    */
 /*  properly but it is adequate for many tasks.                          */
@@ -52,17 +56,17 @@
 /*  octal and hex dumps now.                                             */
 /*=======================================================================*/
 
-#include <stdio.h>
-#include <stdlib.h>
+#include <cstdio>
+#include <cstdlib>
 #include "EST_unix.h"
-#include <string.h>
+#include <cstring>
 #include <time.h>
 #include "EST_wave_utils.h"
 #include "esps_utils.h"
 
 /* First you must realise there is in fact a number of very similar but */
 /* subtly different header formats that appear on ESPS files.           */
-/* ESPS_FEA and ESPS_SD (others for filters, spectragrams, etc)         */
+/* ESPS_FEA and ESPS_SD (others for filters, spectrograms, etc)         */
 /* The basic format is                                                  */
 /*     preamble                                                         */
 /*     fixed header                                                     */
@@ -129,7 +133,7 @@ void print_esps_fea(esps_fea r)
 	}
 }
 
-void add_field(esps_hdr hdr, char *name, int type, int dimension)
+void add_field(esps_hdr hdr, const char *name, int type, int dimension)
 {
     /* Add a new field to the record */
     char **names = hdr->field_name;
@@ -158,7 +162,7 @@ void add_field(esps_hdr hdr, char *name, int type, int dimension)
 
 }
 
-void add_fea_d(esps_hdr hdr,char *name, int pos, double d)
+void add_fea_d(esps_hdr hdr,const char *name, int pos, double d)
 {
     /* Add a double FEA field to the header */
     esps_fea t = new_esps_fea();
@@ -166,8 +170,7 @@ void add_fea_d(esps_hdr hdr,char *name, int pos, double d)
 
     t->type = 13;  /* must be lucky for some !! */
     t->clength = strlen(name);
-    t->name = walloc(char,t->clength+1);
-    sprintf(t->name,"%s",name);
+    t->name = wstrdup(name);
     if (t->count < pos+1)
     {
 	double *dval = t->v.dval;
@@ -188,7 +191,7 @@ void add_fea_d(esps_hdr hdr,char *name, int pos, double d)
     return;
 }
 
-void add_fea_f(esps_hdr hdr,char *name, int pos, float d)
+void add_fea_f(esps_hdr hdr,const char *name, int pos, float d)
 {
     /* Add a float FEA field to the header */
     esps_fea t = new_esps_fea();
@@ -196,8 +199,7 @@ void add_fea_f(esps_hdr hdr,char *name, int pos, float d)
 
     t->type = 13;
     t->clength = strlen(name);
-    t->name = walloc(char,t->clength+1);
-    sprintf(t->name,"%s",name);
+    t->name = wstrdup(name);
     if (t->count < pos+1)
     {
 	float *fval = t->v.fval;
@@ -218,7 +220,7 @@ void add_fea_f(esps_hdr hdr,char *name, int pos, float d)
     return;
 }
 
-void add_fea_i(esps_hdr hdr,char *name, int pos, int d)
+void add_fea_i(esps_hdr hdr,const char *name, int pos, int d)
 {
     /* Add an int FEA field to the header */
     esps_fea t = new_esps_fea();
@@ -226,8 +228,7 @@ void add_fea_i(esps_hdr hdr,char *name, int pos, int d)
 
     t->type = 13;
     t->clength = strlen(name);
-    t->name = walloc(char,t->clength+1);
-    sprintf(t->name,"%s",name);
+    t->name = wstrdup(name);
     if (t->count < pos+1)
     {
 	int *ival = t->v.ival;
@@ -248,7 +249,7 @@ void add_fea_i(esps_hdr hdr,char *name, int pos, int d)
     return;
 }
 
-void add_fea_s(esps_hdr hdr,char *name, int pos, short d)
+void add_fea_s(esps_hdr hdr,const char *name, int pos, short d)
 {
     /* Add a short FEA field to the header */
     esps_fea t = new_esps_fea();
@@ -256,8 +257,7 @@ void add_fea_s(esps_hdr hdr,char *name, int pos, short d)
 
     t->type = 13;
     t->clength = strlen(name);
-    t->name = walloc(char,t->clength+1);
-    sprintf(t->name,"%s",name);
+    t->name = wstrdup(name);
     if (t->count < pos+1)
     {
 	short *sval = t->v.sval;
@@ -278,7 +278,7 @@ void add_fea_s(esps_hdr hdr,char *name, int pos, short d)
     return;
 }
 
-void add_fea_c(esps_hdr hdr,char *name, int pos, char d)
+void add_fea_c(esps_hdr hdr,const char *name, int pos, char d)
 {
     /* Add a char FEA field to the header */
     esps_fea t = new_esps_fea();
@@ -286,8 +286,7 @@ void add_fea_c(esps_hdr hdr,char *name, int pos, char d)
 
     t->type = 13;
     t->clength = strlen(name);
-    t->name = walloc(char,t->clength+1);
-    sprintf(t->name,"%s",name);
+    t->name = wstrdup(name);
     if (t->count < pos+1)
     {
 	char *cval = t->v.cval;
@@ -308,15 +307,14 @@ void add_fea_c(esps_hdr hdr,char *name, int pos, char d)
     return;
 }
 
-void add_fea_special(esps_hdr hdr,int type,char *name)
+void add_fea_special(esps_hdr hdr,int type,const char *name)
 {
     /* Add a special FEA field to the header */
     esps_fea t = new_esps_fea();
 
     t->type = type;
     t->clength = strlen(name);
-    t->name = walloc(char,t->clength+1);
-    sprintf(t->name,"%s",name);
+    t->name = wstrdup(name);
     t->count = 0;
     
     t->next = hdr->fea;
@@ -325,7 +323,7 @@ void add_fea_special(esps_hdr hdr,int type,char *name)
     return;
 }
 
-int fea_value_d(char *name,int pos,esps_hdr hdr, double *d)
+int fea_value_d(const char *name,int pos,esps_hdr hdr, double *d)
 {
     /* Get value of double FEA */
     esps_fea t;
@@ -346,7 +344,7 @@ int fea_value_d(char *name,int pos,esps_hdr hdr, double *d)
     return -1;  /* failed to find it */
 }
 
-int fea_value_f(char *name,int pos,esps_hdr hdr, float *d)
+int fea_value_f(const char *name,int pos,esps_hdr hdr, float *d)
 {
     /* Get value of float FEA */
     esps_fea t;
@@ -367,7 +365,7 @@ int fea_value_f(char *name,int pos,esps_hdr hdr, float *d)
     return -1;  /* failed to find it */
 }
 
-int fea_value_s(char *name,int pos,esps_hdr hdr, short *d)
+int fea_value_s(const char *name,int pos,esps_hdr hdr, short *d)
 {
     /* Get value of short FEA */
     esps_fea t;
@@ -388,7 +386,7 @@ int fea_value_s(char *name,int pos,esps_hdr hdr, short *d)
     return -1;  /* failed to find it */
 }
 
-int fea_value_i(char *name,int pos,esps_hdr hdr, int *d)
+int fea_value_i(const char *name,int pos,esps_hdr hdr, int *d)
 {
     /* Get value of int FEA */
     esps_fea t;
@@ -409,7 +407,7 @@ int fea_value_i(char *name,int pos,esps_hdr hdr, int *d)
     return -1;  /* failed to find it */
 }
 
-int fea_value_c(char *name,int pos,esps_hdr hdr, char *d)
+int fea_value_c(const char *name,int pos,esps_hdr hdr, char *d)
 {
     /* Get value of int FEA */
     esps_fea t;
@@ -634,7 +632,7 @@ static char *esps_get_field_name(FILE *fd, esps_hdr hdr, int expect_source)
   if (fread(&size,2,1,fd) != 1)
     {
       fputs("error reading field name size\n", stderr);
-      return strdup("ERROR");
+      return wstrdup("ERROR");
     }
   if (hdr->swapped) size = SWAPSHORT(size);
   name = walloc(char,size+1);
@@ -892,7 +890,7 @@ int esps_record_size(esps_hdr hdr)
 
 static int esps_num_of_type(int type,esps_hdr hdr)
 {
-    /* counts up the number of occurences of fields of type in a record */
+    /* counts up the number of occurrences of fields of type in a record */
     int i;
     int sum;
 
@@ -984,7 +982,7 @@ enum EST_read_status read_esps_hdr(esps_hdr *uhdr,FILE *fd)
 	hdr->field_name[0] = wstrdup("samples");
 	fseek(fd,hdr->hdr_size,SEEK_SET);
 	/* In this cases its just in the header as a float */
-	sd_sample_rate = *((float *)&fhdr.fil4[0]);
+	sd_sample_rate = *((float *)(void *)&fhdr.fil4[0]);
 	add_fea_d(hdr,"record_freq",0,(double)sd_sample_rate);
 	*uhdr = hdr;
 	return format_ok;
