@@ -5,15 +5,19 @@
 /*                      Copyright (c) 1995,1996                          */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -33,13 +37,13 @@
 /*                                                                       */
 /*=======================================================================*/
 
-#include <stdlib.h>
+#include <cstdlib>
 #include "EST_system.h"
 #include "EST_FMatrix.h"
 #include "EST_cluster.h"
-#include "fstream.h"
+#include <fstream>
 #include "EST_string_aux.h"
-#include "float.h"
+#include <cfloat>
 
 int fn_cluster(EST_FMatrix &m, EST_CBK &cbk, float d);
 int nn_cluster(EST_FMatrix &m, EST_CBK &cbk, float d);
@@ -75,9 +79,9 @@ int cluster(EST_FMatrix &m, EST_CBK &cbk, EST_TList<EST_String> &ans, EST_String
 // return true if list l contains integer n
 int contains(EST_TList<int> &l, int n)
 {
-    EST_TBI *p;
+    EST_Litem *p;
 
-    for (p = l.head(); p != 0; p = next(p))
+    for (p = l.head(); p != 0; p = p->next())
 	if (l(p) == n)
 	    return 1;
 
@@ -86,11 +90,11 @@ int contains(EST_TList<int> &l, int n)
 
 void remove_distances(EST_FMatrix &d , EST_TList<int> &group)
 {
-    EST_TBI *pi, *pj;
+    EST_Litem *pi, *pj;
     int i, j;
     
-    for (i = 0, pi = group.head(); pi != 0; pi = next(pi), ++i)
-	for (j = 0, pj = group.head(); pj != 0; pj = next(pj), ++j)
+    for (i = 0, pi = group.head(); pi != 0; pi = pi->next(), ++i)
+	for (j = 0, pj = group.head(); pj != 0; pj = pj->next(), ++j)
 	    d(group(pi), group(pj)) = 0.0;
 }
 
@@ -112,13 +116,13 @@ void remove_distances(EST_FMatrix &d , EST_TList<int> &group)
 
 void collapse(EST_FMatrix &d, EST_CBK &cbk, int row, int col)
 {
-    EST_TBI *pi, *pj;
+    EST_Litem *pi, *pj;
 
-    for (pi = cbk.head(); pi != 0; pi = next(pi))
+    for (pi = cbk.head(); pi != 0; pi = pi->next())
 	if (contains(cbk(pi), row))
 	    break;
 
-    for (pj = cbk.head(); pj != 0; pj = next(pj))
+    for (pj = cbk.head(); pj != 0; pj = pj->next())
 	if (contains(cbk(pj), col))
 	    break;
 
@@ -129,13 +133,11 @@ void collapse(EST_FMatrix &d, EST_CBK &cbk, int row, int col)
 
 float min(float a, float b)
 {
-    cout << "Min\n";
     return (a < b) ? a: b;
 }
 
 float max(float a, float b)
 {
-    cout << "Max\n";
     return (a > b) ? a: b;
 }
 
@@ -144,7 +146,7 @@ float max(float a, float b)
 void collapse3(EST_FMatrix &d, EST_CBK &cbk, int row, int col, EST_String method)
 {
     int i;
-    EST_TBI *pi;
+    EST_Litem *pi;
     EST_TList<int> v;
     float fm;
 
@@ -162,7 +164,7 @@ void collapse3(EST_FMatrix &d, EST_CBK &cbk, int row, int col, EST_String method
 
     cout << "row " << row << " col " << col << " left out " << v;
     
-    for (pi = v.head(); pi != 0; pi = next(pi))
+    for (pi = v.head(); pi != 0; pi = pi->next())
     {
 	if (method == "nearest")
 	    fm = min(d(row,v(pi)),d(col,v(pi)));
@@ -184,7 +186,7 @@ void collapse3(EST_FMatrix &d, EST_CBK &cbk, int row, int col, EST_String method
 int nn_cluster2(EST_FMatrix &m, EST_CBK &cbk, float d)
 {
     static float smallest = 0.0;
-    int row, col;
+    int row=0, col=0;
     (void)d;
 
 // Change so that all values aprt from lowest in codebook get set to
@@ -195,7 +197,7 @@ int nn_cluster2(EST_FMatrix &m, EST_CBK &cbk, float d)
     cout << "row = " << row << " col " << col << endl;
     collapse(m, cbk, row, col);
 
-    for (EST_TBI *p = cbk.head(); p != 0; p = next(p))
+    for (EST_Litem *p = cbk.head(); p != 0; p = p->next())
 	cout << cbk(p);
     cout << "New matrix\n" << m;
 
@@ -206,7 +208,7 @@ int nn_cluster2(EST_FMatrix &m, EST_CBK &cbk, float d)
 float nn_cluster3(EST_FMatrix &m, EST_CBK &cbk, EST_String method)
 {
     static float smallest = 0.0;
-    int row, col;
+    int row=0, col=0;
 
 // Change so that all values aprt from lowest in codebook get set to
 // Nan (or whatever)
@@ -217,7 +219,7 @@ float nn_cluster3(EST_FMatrix &m, EST_CBK &cbk, EST_String method)
     cout << "row = " << row << " col " << col << endl;
     collapse3(m, cbk, row, col, method);
 
-    for (EST_TBI *p = cbk.head(); p != 0; p = next(p))
+    for (EST_Litem *p = cbk.head(); p != 0; p = p->next())
 	cout << cbk(p);
     cout << "New matrix\n" << m << endl << endl;
 
@@ -228,14 +230,14 @@ float nn_cluster3(EST_FMatrix &m, EST_CBK &cbk, EST_String method)
 int nn_cluster(EST_FMatrix &m, EST_CBK &cbk, float d)
 {
     int i;
-    EST_TBI *pi, *pj;
+    EST_Litem *pi, *pj;
     float smallest;
     int c = 0;
     
     i = 0;
-    for (pi = cbk.head(); pi != 0; pi = next(pi), ++i)
+    for (pi = cbk.head(); pi != 0; pi = pi->next(), ++i)
     {
-	for (pj = next(pi); pj != 0; pj = next(pj))
+	for (pj = pi->next(); pj != 0; pj = pj->next())
 	{
 	    smallest = lowestval(m, cbk(pj), cbk(pi));
 	    if (smallest < d)
@@ -246,7 +248,7 @@ int nn_cluster(EST_FMatrix &m, EST_CBK &cbk, float d)
 	}
     }
     
-    for (pi = cbk.head(); pi != 0; pi = next(pi))
+    for (pi = cbk.head(); pi != 0; pi = pi->next())
     {
 	if (cbk(pi).empty())
 	{
@@ -263,14 +265,14 @@ int nn_cluster(EST_FMatrix &m, EST_CBK &cbk, float d)
 int fn_cluster(EST_FMatrix &m, EST_CBK &cbk, float d)
 {
     int i;
-    EST_TBI *pi, *pj;
+    EST_Litem *pi, *pj;
     float smallest;
     int c = 0;
     
     i = 0;
-    for (pi = cbk.head(); pi != 0; pi = next(pi), ++i)
+    for (pi = cbk.head(); pi != 0; pi = pi->next(), ++i)
     {
-	for (pj = next(pi); pj != 0; pj = next(pj))
+	for (pj = pi->next(); pj != 0; pj = pj->next())
 	{
 	    smallest = highestval(m, cbk(pj), cbk(pi));
 	    if (smallest < d)
@@ -281,7 +283,7 @@ int fn_cluster(EST_FMatrix &m, EST_CBK &cbk, float d)
 	}
     }
     
-    for (pi = cbk.head(); pi != 0; pi = next(pi))
+    for (pi = cbk.head(); pi != 0; pi = pi->next())
     {
 	if (cbk(pi).empty())
 	{
@@ -312,7 +314,7 @@ EST_FVector sort_matrix(EST_FMatrix &m)
     float *v;
     int n_vals;
     
-    // determine size of triangualr part of matrix, excluding diagonal
+    // determine size of triangular part of matrix, excluding diagonal
     int size = m.num_rows() - 1;
     
     n_vals = 0;
@@ -344,21 +346,21 @@ EST_FVector sort_matrix(EST_FMatrix &m)
 
 EST_String print_codebook(EST_CBK &cbk, float d, EST_TList<EST_String> &names)
 {
-    EST_TBI *pi;
-    EST_TBI *pj;
+    EST_Litem *pi;
+    EST_Litem *pj;
     EST_String s;
     
     s = ftoString(d) + " ";
-    for (pi = cbk.head(); pi != 0; pi = next(pi))
+    for (pi = cbk.head(); pi != 0; pi = pi->next())
     {
 	s += "(";
-	for (pj = cbk(pi).head(); pj != 0; pj = next(pj))
+	for (pj = cbk(pi).head(); pj != 0; pj = pj->next())
 	{
 	    if (names.empty())
 		s += itoString(cbk.item(pi).item(pj));
 	    else 
 		s += names.nth(cbk.item(pi).item(pj));
-	    if (next(pj) !=  0)
+	    if (pj->next() !=  0)
 		s += "   ";
 	}
 	s += ") ";
@@ -405,7 +407,7 @@ void cluster3(EST_FMatrix &m, float d)
    EST_TList<int> newcbk[12];
    float sortval = {2.0, 3.0, 4.0, 5.0, 6.0};
    int i, j, n, m;
-   EST_TBI *p;
+   EST_Litem *p;
    
    for (i = 0; i < n; ++i)
    oldcbk[i].append(i);
@@ -433,13 +435,13 @@ void cluster3(EST_FMatrix &m, float d)
    */
 float lowestval(EST_FMatrix &m, EST_TList<int> &a, EST_TList<int> &b)
 {
-    EST_TBI *pa, *pb;
+    EST_Litem *pa, *pb;
     float lowest = 100000.0;
     
     cout << "list a:" << a << "list b:" << b;
     
-    for (pa = a.head(); pa != 0; pa = next(pa))
-	for (pb = b.head(); pb != 0; pb = next(pb))
+    for (pa = a.head(); pa != 0; pa = pa->next())
+	for (pb = b.head(); pb != 0; pb = pb->next())
 	{
 	    //      cout << "m:" << a(pa) << " " << b(pb) << " " << m.x[a(pa)][b(pb)] << endl;
 	    if (m(a(pa), b(pb)) < lowest)
@@ -469,13 +471,13 @@ float lval(EST_FMatrix &a, float floor, int &row, int &col)
 
 float highestval(EST_FMatrix &m, EST_TList<int> &a, EST_TList<int> &b)
 {
-    EST_TBI *pa, *pb;
+    EST_Litem *pa, *pb;
     float h = 0.0;
     
     cout << "list a:" << a << "list b:" << b;
     
-    for (pa = a.head(); pa != 0; pa = next(pa))
-	for (pb = b.head(); pb != 0; pb = next(pb))
+    for (pa = a.head(); pa != 0; pa = pa->next())
+	for (pb = b.head(); pb != 0; pb = pb->next())
 	{
 	    if (m(a(pa), b(pb)) > h)
 		h = m(a(pa), b(pb));
@@ -486,10 +488,10 @@ float highestval(EST_FMatrix &m, EST_TList<int> &a, EST_TList<int> &b)
 /*
    float nearest(EST_FMatrix &m, EST_TList<int> &cbk)
    {
-   EST_TBI *p;
+   EST_Litem *p;
    float lowest = 100000.0;
    
-   for (p = cbk.head(); p != 0; p = next(p))
+   for (p = cbk.head(); p != 0; p = p->next())
    {
    cout << "cbk(p) " << cbk(p) << endl;
    if (cbk(p) < lowest)
@@ -502,9 +504,9 @@ float highestval(EST_FMatrix &m, EST_TList<int> &a, EST_TList<int> &b)
    */
 void merge(EST_TList<int> cbk[], int i, int j)
 {
-    EST_TBI *p;
+    EST_Litem *p;
     
-    for (p = cbk[j].head(); p != 0; p = next(p))
+    for (p = cbk[j].head(); p != 0; p = p->next())
 	cbk[i].append(cbk[j].item(p));
     
     cbk[j].clear();
@@ -514,7 +516,6 @@ int load_names(EST_String file, EST_TList<EST_String> &names)
 {
     char inbuf[1000];
     EST_String tmpstr;
-    EST_String points[1000];
     
     ifstream inf(file);
     if (!inf) cerr << "Can't open names file " << file << endl;
@@ -530,9 +531,9 @@ int load_names(EST_String file, EST_TList<EST_String> &names)
 /*int merge(EST_TList<int> &a, EST_TList<int> &b)
   {
   EST_TList<int> newgroup;
-  EST_TBI *p;
+  EST_Litem *p;
   
-  for (p = cbk[j].head(); p != 0; p = next(p))
+  for (p = cbk[j].head(); p != 0; p = p->next())
   cbk[i].append(cbk[j].item(p));
   
   cbk[j].clear();
