@@ -39,7 +39,7 @@
 /*  and it searches for the best path through the candidates             */
 /*                                                                       */
 /*=======================================================================*/
-#include <stdio.h>
+#include <cstdio>
 #include "EST_viterbi.h"
 
 static void init_paths_array(EST_VTPoint *n,int num_states);
@@ -56,7 +56,7 @@ EST_VTPoint::~EST_VTPoint()
 	for (i=0; i<num_states; i++)
 	    if (st_paths[i] != 0)
 		delete st_paths[i];
-	delete st_paths;
+	delete [] st_paths;
     }
     if (cands != 0) delete cands;
     if (next != 0) delete next;
@@ -111,7 +111,7 @@ void EST_Viterbi_Decoder::initialise(EST_Relation *p)
     EST_Item *i;
     EST_VTPoint *t = 0,*n;
 
-    for (i=p->head(); i != 0; i=next(i))
+    for (i=p->head(); i != 0; i=inext(i))
     {
 	n = new EST_VTPoint;
 	n->s = i;
@@ -165,7 +165,7 @@ const int EST_Viterbi_Decoder::betterthan(const float a,const float b) const
 	return (a < b);
 }
 
-static void init_dynamic_states(EST_VTPoint *p, EST_VTCandidate *cands)
+static int init_dynamic_states(EST_VTPoint *p, EST_VTCandidate *cands)
 {
     // In a special (hmm maybe not so special), the number of "states"
     // is the number of candidates
@@ -175,6 +175,8 @@ static void init_dynamic_states(EST_VTPoint *p, EST_VTCandidate *cands)
     for (i=0, c=cands; c != 0; c=c->next,i++)
 	c->pos = i;
     init_paths_array(p,i);
+    
+    return i;
 }
 
 void EST_Viterbi_Decoder::set_pruning_parameters(float beam, float
@@ -206,8 +208,8 @@ void EST_Viterbi_Decoder::search(void)
     EST_VTCandidate *c;
     int i=0;
 
-    double best_score,score_cutoff;
-    double best_candidate_score,candidate_cutoff=0;
+    double best_score=0.0,score_cutoff=0.0;
+    double best_candidate_score=0.0,candidate_cutoff=0;
     int dcount,pcount;
     int cand_count=0, cands_considered=0;
 
@@ -237,7 +239,7 @@ void EST_Viterbi_Decoder::search(void)
 			   betterthan(c->score,candidate_cutoff))
 			{
 			    cands_considered++;
-			    // Find path extention costs
+			    // Find path extension costs
 			    np = (*user_npath)(p->st_paths[i],c,f);
 			    if (debug) debug_output_1(p,c,np,i);
 			    if (do_pruning && betterthan(np->score,best_score))
@@ -273,7 +275,7 @@ void EST_Viterbi_Decoder::search(void)
 		if(trace)
 		{
 		    cerr << "Considered " << cands_considered << " of ";
-		    cerr << cand_count*cand_count << " candidate paths" << endl;
+		    cerr << cand_count*p->num_states << " candidate paths" << endl;
 		    cerr << "FRAME: best score " << best_score;
 		    cerr << "  score cutoff " << score_cutoff << endl;
 		    cerr << "       best candidate score " << best_candidate_score;
@@ -418,7 +420,7 @@ void EST_Viterbi_Decoder::add_path(EST_VTPoint *p, EST_VTPath *np)
 	cerr << "Viterbi: tried to add path to NULL point\n";
     else 
     {
-	if ((beam_width == 0) ||            // Add if no beam restritions or
+	if ((beam_width == 0) ||            // Add if no beam restrictions or
 	    (p->num_paths < beam_width) ||  //        beam not filled  or
 	    (betterthan(np->score,p->paths->score)))//this is better than best
 //	    (np->score > p->paths->score))  //        this is better than best
@@ -470,7 +472,7 @@ EST_VTCandidate *EST_Viterbi_Decoder::add_cand_prune(EST_VTCandidate *newcand,
     else
 	numcands = allcands->pos;
     
-    if ((cand_width == 0) ||	// Add if no candbeam restritions or
+    if ((cand_width == 0) ||	// Add if no candbeam restrictions or
 	(numcands < cand_width) || //        candbeam not filled  or
 	(betterthan(newcand->score,allcands->score))) //this better than best
     {
@@ -530,6 +532,23 @@ bool EST_Viterbi_Decoder::result(const EST_String &n)
     return TRUE;
 }
 
+bool EST_Viterbi_Decoder::result( EST_VTPath **bestPathEnd )
+{
+  // Finds best path through the search space (previously searched)
+  *bestPathEnd = 0; 
+  
+  if ((timeline == 0) || (timeline->next == 0))
+    return TRUE;  // it's an empty list so it has succeeded
+
+  *bestPathEnd = find_best_end();
+
+  if (*bestPathEnd == 0)
+    return FALSE; // there isn't any answer
+
+  return TRUE;
+}
+
+
 void EST_Viterbi_Decoder::copy_feature(const EST_String &n)
 {
     // Copy feature from path to related stream
