@@ -34,27 +34,28 @@
 /*                   Date   :  June 1998                                 */
 /*************************************************************************/
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <iostream.h>
-#include <fstream.h>
+#include <cstdlib>
+#include <cstdio>
+#include <iostream>
+#include <fstream>
 #include "EST_math.h"
 #include "ling_class/EST_Utterance.h"
 
 typedef EST_TVector<EST_Item*> EST_Item_ptr_Vector;
+static EST_Item *const def_val_item_ptr = NULL;
+template <> EST_Item* const *EST_Item_ptr_Vector::def_val = &def_val_item_ptr;
+
+static EST_Item* error_return_item_ptr = NULL;
+template <> EST_Item* *EST_Item_ptr_Vector::error_return = &error_return_item_ptr;
+
 #if defined(INSTANTIATE_TEMPLATES)
+
 #include "../base_class/EST_TVector.cc"
 
 template class EST_TVector<EST_Item*>;
 
 #endif
 
-static EST_Item *const def_val_item_ptr = NULL;
-EST_Item* const *EST_Item_ptr_Vector::def_val = &def_val_item_ptr;
-
-static EST_Item* error_return_item_ptr = NULL;
-EST_Item* *EST_Item_ptr_Vector::error_return = &error_return_item_ptr;
-
 typedef
 float (*local_cost_function)(const EST_Item *item1,
 			     const EST_Item *item2);
@@ -154,7 +155,7 @@ bool dp_match(const EST_Relation &lexical,
 
     // aligns lexical and surface forms using dynamic programming
     // i.e. the lexical form is transformed into the surface form
-    //      by substitutions, inderstions and deletions
+    //      by substitutions, insertions and deletions
 
     // makes links between associated (matching or substituted) items
     // insertions and deletions are 'left dangling'
@@ -177,9 +178,9 @@ bool dp_match(const EST_Relation &lexical,
     vr1[0] = null_sym;
     vr2[0] = null_sym;
 
-    for (p=lexical.head(),i=1; p != 0; p = next(p),i++)
+    for (p=lexical.head(),i=1; p != 0; p = inext(p),i++)
 	vr1[i] = p;
-    for (p=surface.head(),i=1; p != 0; p = next(p),i++)
+    for (p=surface.head(),i=1; p != 0; p = inext(p),i++)
 	vr2[i] = p;
 	
     DP_path_i.resize(l1,l2);
@@ -217,7 +218,7 @@ bool dp_match(const EST_Relation &lexical,
     }
     // make somewhere to record the relations
     //utt.create_relation("Match");
-    for (p = lexical.head(); p; p = next(p))
+    for (p = lexical.head(); p; p = inext(p))
 	match.append(p);
 
     /*
@@ -231,8 +232,8 @@ bool dp_match(const EST_Relation &lexical,
 */
 
     trace_back_and_link(l1-1,l2-1,
-			match.last(),
-			surface.last(),
+			match.rlast(),
+			surface.rlast(),
 			DP_path_i,DP_path_j,null_sym);
 
     return TRUE;
@@ -397,18 +398,18 @@ void trace_back_and_link(int i, int j,
 	    // match, or substitution
 	    //cerr << "sub " << p1->name() << " with " << p2->name() << endl;
 	    p1->append_daughter(p2);
-	    p1=prev(p1);
-	    p2=prev(p2);
+	    p1=iprev(p1);
+	    p2=iprev(p2);
 	}
 	else
 	    // deletion
-	    p1=prev(p1);
+	    p1=iprev(p1);
     }    
     else
     {
 	// insertion
 	// p1->append_daughter(p2); // decorative
-	p2=prev(p2);
+	p2=iprev(p2);
     }
 
     trace_back_and_link(DP_path_i(i,j), DP_path_j(i,j),
