@@ -5,15 +5,19 @@
 /*                      Copyright (c) 1996,1997                          */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify and distribute this software and its */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -32,17 +36,17 @@
 /*  Decision lists                                                       */
 /*  These are like a right branching decision tree, yes answers are not  */
 /*  partitioned any further.  These are used for the Yarowsky-style      */
-/*  homograph diambiguators                                              */
+/*  homograph disambiguators                                             */
 /*                                                                       */
 /*  Yarowsky, D. "Homograph Disambiguation in Text-to_Speech Synthesis"  */
-/*  in "Progress in Speech Synthesis" eds van Snaten, J. et al. Springer */
+/*  in "Progress in Speech Synthesis" eds van Santen, J. et al. Springer */
 /*  pp 157-172, 1996                                                     */
-/*  Rivest, R.L. "Learning descision lists", Machine Learning 2:229-246  */
+/*  Rivest, R.L. "Learning decision lists", Machine Learning 2:229-246   */
 /*  1987                                                                 */
 /*                                                                       */
 /*=======================================================================*/
-#include <iostream.h>
-#include <string.h>
+#include <iostream>
+#include <cstring>
 #include "EST_Wagon.h"
 #include "EST_FMatrix.h"
 #include "EST_multistats.h"
@@ -59,17 +63,12 @@ WNode *wgn_build_dlist(float &score,ostream *output)
 
     *output << *dlist;
 
-    if (wgn_test_file != "")  // load in test data
-    {
-	wgn_load_dataset(wgn_test_dataset,wgn_test_file);
+    if (wgn_test_dataset.width() > 0)  // load in test data
 	do_dlist_summary(dlist,wgn_test_dataset);
-    }
     else
 	do_dlist_summary(dlist,wgn_dataset);
 
-
     score = 0;
-
     return 0;
 }
 
@@ -78,13 +77,13 @@ static void do_dlist_summary(WDlist *dlist, WDataSet &dataset)
     // Test dlist against data to get summary of results
     EST_StrStr_KVL pairs;
     EST_StrList lex;
-    EST_TBI *p;
+    EST_Litem *p;
     EST_String predict,real;
     int i,type;
 
-    for (p=dataset.head(); p != 0; p=next(p))
+    for (p=dataset.head(); p != 0; p=p->next())
     {
-	predict = dlist->predict(*dataset(p));
+	predict = (EST_String)dlist->predict(*dataset(p));
 	type = dataset.ftype(0);
 	real = wgn_discretes[type].name(dataset(p)->get_int_val(0));
 	pairs.add_item(real,predict,1);
@@ -102,7 +101,7 @@ static WDlist *wagon_decision_list()
     // For each possible question, measure it using 
     // yarowsky's loglikelihood ratio
     // Abs(log(P(class_1|question_j)/P(class_2|question_j)))
-    // Output it for extranal sorting 
+    // Output it for external sorting 
     // Hmm this implies binary classes 
     int i,cl;
     WQuestion ques;
@@ -135,11 +134,11 @@ static WDlist *dlist_score_question(WQuestion &q, WVectorList &ds)
     // score this question for decision lists
     // the sum of the impurities when ds is split with this question
     WImpurity y;
-    EST_TBI *d;
+    EST_Litem *d;
     WVector *wv;
     int i;
 
-    for (i=0,d=ds.head(); d != 0; d=next(d),i++)
+    for (i=0,d=ds.head(); d != 0; d=d->next(),i++)
     {
 	wv = ds(d);
 	if (q.ask(*wv) == TRUE)
@@ -148,11 +147,11 @@ static WDlist *dlist_score_question(WQuestion &q, WVectorList &ds)
 
     if (y.samples() > wgn_min_cluster_size)
     {
-	q.set_yes(y.samples());
+	q.set_yes((int)y.samples());
 
 	EST_DiscreteProbDistribution &pd = y.pd();
 	
-	// Generalizing the formula in Yarowsky chapter we modify it
+	// Generalizing the formula in Yarowsky (pp157-172) we modify it
 	// to take absolute log-likelihood ration of the most probability
 	// of the most probable over the rest
 	EST_String t;
@@ -185,14 +184,14 @@ EST_Val WDlist::predict(const WVector &d)
     if (p_question.ask(d))
 	return p_token;
     else if (next == 0)
-	return "guess";  // should be apriori most probable as dlist can't help
+	return "guess";  // should be a priori most probable as dlist can't help
     else 
 	return next->predict(d);
 }
 
 WDlist *add_to_dlist(WDlist *l, WDlist *a)
 {
-    // Add a to l at approriate place in score order 
+    // Add a to l at appropriate place in score order 
     WDlist *p,*lp;
     
     if (l == 0)
